#include <stdio.h>

#include <dsdebug.h>
#include <dsassert.h>

#include "buffer.h"

#include "ascii.h"
#include "asciibuffer.h"

int asciiBufferAdd(ASCIIDevice dev, const DSTime time, unsigned int channels, double *channel) {
    int err;
    double *mychannel;
    DSTime mytime;
        
    ASCIIReaderBuf rb;
    
    ds_assert_param(dev);
    ds_assert_param(channels);
    
    rb = (ASCIIReaderBuf)malloc(sizeof(ASCIIReaderBufS));
    mychannel = (double*)malloc(channels*sizeof(double));

    if ((!rb)||(!mychannel)) err = DS_ERR_MEM;
    else err = ReaderBufInit((ReaderBuf)rb, (Device)dev, time);

    if (err) {
	if (rb) free(rb);
	if (mychannel) free(mychannel);
	return err;
    }
    
    
    rb->channels = channels;
    rb->channel = mychannel;
    memcpy(mychannel, channel, channels*sizeof(double));

    rb->buf.destroy = (ReaderBufDestroyFunction)asciiBufferDestroy;
    
    BufferAdd(dev->device.buffer, (ReaderBuf)rb);
    dsDebugServer(DS_DEBUG_DEBUG, "Buffer: New ASCII event");
    
    return 0;
}


void asciiBufferDestroy(ASCIIReaderBuf buffer) {
    ds_assert_return(buffer);
    free(buffer->channel);
    ReaderBufFree((ReaderBuf)buffer);
    free(buffer);
}
