#ifndef _BUFFER_H
#define _BUFFER_H

#include <dsconfig.h>
#include <dslock.h>
#include <dscondition.h>
#include <dstime.h>
#include <dsthread.h>

typedef struct ReaderBufT *ReaderBuf;
typedef struct BufferT *Buffer;

#include "device.h"

typedef void (*ReaderBufDestroyFunction)(ReaderBuf ctx);


struct ReaderBufT {
	ReaderBuf next;
	ReaderBufDestroyFunction destroy;
	
	DSTime tm;
	Device dev;

	unsigned char sf;
};
typedef struct ReaderBufT ReaderBufS;


struct BufferT {
    DSConfigContext cfg;
    DSConfigReference cfgref;
    
    DSThreadContext thread;
    DSLockContext lock;
    DSConditionContext cond;
    DSConditionContext pcond;
    ReaderBuf rb;
    ReaderBuf rbr;
};
typedef struct BufferT BufferS;


Buffer BufferCreate(DSConfigContext config, DSConfigReference ref);
void BufferDestroy(Buffer buf);

int ReaderBufInit(ReaderBuf buf, Device dev, DSTime tm);
int ReaderBufFree(ReaderBuf buf);

#define BufferLock(buf) dsLockWrite(buf->lock)
#define BufferUnlock(buf) dsLockReturn(buf->lock)

int BufferProcess(Buffer buf);
int BufferAdd(Buffer buf, ReaderBuf rb);

int BufferWait(Buffer buf, DSDuration timeout);

int BufferStartLoop(Buffer buf);
int BufferStopLoop(Buffer buf);

int BufferWaitDeviceEventsProcessed(Buffer buf, Device dev);

#endif /* _BUFFER_H */

