#ifndef _CONNECTION_H
#define _CONNECTION_H

#include <dslock.h>
#include <dsconnection.h>
#include <dstime.h>
#include <dsconfig.h>

enum ConnectionTypeT {
    CONNECTION_TYPE_UNKNOWN = 0,
    CONNECTION_TYPE_UART,
    CONNECTION_TYPE_UDP
};
typedef enum ConnectionTypeT ConnectionType;


typedef struct ConnectionT *Connection;

typedef int (*ConnectionOpenFunction)(Connection ctx);

struct ConnectionT {
    char *id;
    Connection next;
    
    ConnectionType type;
    DSConnectionContext con;
    DSLockContext lock;
    DSConfigContext cfg;
    DSConfigReference cfgref;

    ConnectionOpenFunction connect;
    
    DSDebugPriority silence;
    DSDuration timeout;
    DSDuration retry_timeout;
    DSDuration slow_write_delay;

    unsigned short retries;
    unsigned short disabled_retries;
    
    unsigned char connected;
    unsigned char llerr;
};
typedef struct ConnectionT ConnectionS;

Connection ConnectionCreate(DSConfigContext config, DSConfigReference ref);
void ConnectionDestroy(Connection ctx);
int ConnectionOpen(Connection ctx);
int ConnectionClose(Connection ctx);

Connection ConnectionFind(Connection ctx, const char *id);

#define ConnectionLock(ctx) dsLockWrite(ctx->lock)
#define ConnectionTryLock(ctx) dsLockTryWrite(ctx->lock)
#define ConnectionUnlock(ctx) dsLockReturn(ctx->lock)
#define ConnectionRead(ctx,buf,size) dsConnectionRead(ctx->con, buf, size, 0, NULL)
#define ConnectionWrite(ctx,buf,size) dsConnectionWrite(ctx->con, buf, size, 0, NULL)
#define ConnectionVariableRead(ctx,buf,size,readed) dsConnectionRead(ctx->con, buf, size, DS_CONNECTION_FLAG_OVERRIDEDEFAULT, readed)
#define ConnectionSetSignal(ctx, sig) dsConnectionSetSignal(ctx->con, sig, NULL)
#define ConnectionResetSignal(ctx, sig) dsConnectionResetSignal(ctx->con, sig, NULL)

int ConnectionReadAll(Connection ctx); /* Skip all available data */
int ConnectionSkipAll(Connection ctx); /* Performing read request, skipping read data */

int ConnectionSlowWrite(Connection ctx, const void *buf, size_t len);
int ConnectionDisableRetries(Connection ctx);
int ConnectionEnableRetries(Connection ctx);
int ConnectionSilence(Connection ctx, DSDebugPriority mdl);

#ifdef _CONNECTION_C
struct ConnectionConfigurationT {
    const char *id;
    ConnectionOpenFunction open;
};

typedef struct ConnectionConfigurationT *ConnectionConfiguration;
typedef struct ConnectionConfigurationT ConnectionConfigurationS;

#include "comconnection.h"
#include "netconnection.h"
static ConnectionConfigurationS connection_configuration[] = {
    { "COMConnection", COMConnectionOpen },
    { "NETConnection", NETConnectionOpen },
    { NULL, NULL }
};
#endif /* _CONNECTION_C */


#endif /* _CONNECTION_H */
