#ifndef _DEFAULTS_H
#define _DEFAULTS_H

#define DEFAULT_DEBUG_LEVEL	DS_DEBUG_WARNING

#ifdef _WIN32
# define LOG_FILE 		".\\adas.log"
# define DEFAULT_FILE_PATH	".\\Data"
#else /* _WIN32 */
# define LOG_FILE 		"/var/log/adas.log"
# define DEFAULT_FILE_PATH	"/var/ftp/pub"
#endif /* _WIN32 */
#define DEFAULT_FILE_EXTENSION	".dat"

#define DEFAULT_CONTROL_PORT			13011
#define DEFAULT_CLIENT_TIMEOUT			"3600s"
#define DEFAULT_DEBUGCLIENT_TIMEOUT		"0"
#define DEFAULT_SEPARATE_BUFFER_THREAD		0
#define DEFAULT_CONFIGURATION_MODE_DURATION	300 /* s */
#define DEFAULT_CONTROL_MULTIMESSAGE_DELAY	1000 /* us */
#define MAX_DATAGRAMSERVER_MANAGER_ITERATIONS 	10
#define CONTROL_QUEUE_SIZE			10

#define PROTOCOL_MINIMAL_VERSION 	4
#define PROTOCOL_MAXIMAL_VERSION 	7
#define NAMT_MINIMAL_VERSION		1
#define NAMT_MAXIMAL_VERSION		0xFF
#define SNT_MINIMAL_VERSION		1
#define SNT_MAXIMAL_VERSION		0xFF
#define ANALYZER_MINIMAL_VERSION	1
#define ANALYZER_MAXIMAL_VERSION	0xFF
  
#define MAX_NAMT_DETECTORS		16	/* in a layer */
#define MAX_NAMT_SENSORS		32	/* in all devices */
#define MAX_NAMT_COINCIDENCES		512	/* in a plain form */

#define NAMT_TIMEOUT_REINIT	500000	/* us */
#define NANM_TIMEOUT_REINIT	250000	/* us */
#define ASCII_TIMEOUT_REINIT	250000	/* us */
#define SEVAN_TIMEOUT_REINIT	250000	/* us */
#define NMV_TIMEOUT_REINIT	250000	/* us */
#define TIMEOUT_RW		80000	/* us (should be greater than UART_TIMEOUT) */
#define WRITE_RETRIES		3
#define RETRY_TIMEOUT		1000	/* us */
#define SLOW_WRITE_DELAY	1	/* us */

#define MINIMAL_SYNC_ERROR	10	/* ms */
#define MAXIMAL_SYNC_ERROR	200	/* ms */
#define SYNC_DIVIDER		1000	/* percents from whole readout time 0.1% */


#define FUJITSU_ROUND_TRIP_TIME	10	/* ms */
#define HILEVEL_SLEEP		1000	/* us */

#define NAMT_CURRENT_DATA_MINIMUM_DURATION	100	/* ms */
#define NAMT_CURRENT_DATA_ACQUISITION_TIME	100	/* ms */
#define NAMT_DATA_ACQUISITION_TIME		200	/* ms */
#define NAMT_INITIALIZATION_TIME		200	/* ms */


#define NAMT_DEFAULT_READOUT_DURATION	10000	/* ms */
#define NANM_DEFAULT_READOUT_DURATION	60000	/* ms */
#define DEFAULT_WRITEOUT_RECS		30
#define DEFAULT_TRIGGER_DURATION	1500	/* us */
#define DEFAULT_VOLTAGE			0x80

#define SNT_DEFAULT_WRITE_SPECTRUM			180
#define SNT_DEFAULT_WRITE_EVENTS			1
#define SNT_DEFAULT_SOFTWARE_THRESHOLD			1
#define SNT_DEFAULT_READ_CONTROL			0
#define SNT_DEFAULT_WRITE_CONTROL			0
#define SNT_DEFAULT_CONTROL_READOUT_REQUIRED_TIME	2000 /* ms */
#define SNT_DEFAULT_HIGH_READ_PAUSE			100 /* ms */

#define CALLIBRATION_TIME 		10

//#define REAL_TIME_READER	

#ifdef _WIN32 
# define lrint(x) ((long int)(x))
# define attribute_packed 
#else /* _WIN32 */
# define attribute_packed __attribute__ ((__packed__))
#endif /* _WIN32 */


#if 2 * MAX_NAMT_DETECTORS < MAX_NAMT_SENSORS
# error "Error. MAX_NAMT_SENSORS should be greater than 2 * MAX_NAMT_DETECTORS"
#endif


enum DataTypeT {
    DATA_TYPE_UNKNOWN = 0,
    DATA_TYPE_UINT8,		/* x86 byte order */
    DATA_TYPE_UINT16,		/* x86 byte order */
    DATA_TYPE_UINT32,		/* x86 byte order */
    DATA_TYPE_UINT64,		/* x86 byte order */
    DATA_TYPE_DOUBLE,		/* x86 byte order */
    DATA_TYPE_MIXED		/* double, x86 byte order */
};

typedef enum DataTypeT DataType;

#endif /* _DEFAULTS_H */

