#define _DEVICE_C
#define _DS_INCLUDE_XML_HEADERS

#include <stdio.h>
#include <string.h>

#include <dsassert.h>
#include <dsdebug.h>
#include <dserror.h>

#include "defaults.h"

#include "device.h"

static DeviceControlRequest DeviceControlRequestCreate(ADASCommand cmd, uint32_t reqid, const char *req, DSDatagramServerContext dsc, DSConnectionAddress *addr);
static void DeviceControlRequestDestroy(DeviceControlRequest cr);
static int DeviceControlRequestProcess(Device dev);
static int DeviceControlRequestAdd(Device dev, ADASCommand cmd, uint32_t reqid, const char *req, DSDatagramServerContext dsc, DSConnectionAddress *request_addr);
static int DeviceControlLoop(Device dev);

Device DeviceCreate(DSConfigContext config, DSConfigReference ref, Connection con, Writer wr, Buffer buf) {
    int err = 0;
    DSConfigReference cfgref;
    Device dev = NULL;
    DSThreadContext rthread = NULL, cthread = NULL;
    xmlNodePtr cfgnode, readcfg, writecfg, chipcfg, proccfg;
    xmlNodeSetPtr cconnections, connections, writers;
    DSLockContext lock = NULL, cqlock = NULL;
    DSConditionContext cqcond = NULL;
    Connection *mycon = NULL, *myccon = NULL;
    Writer *mywr = NULL;
    DSTime omf = NULL;
    char *id = NULL;
    const char *tmp;
    int i, device_type = -1;
    
    ds_assert_null(config);
    ds_assert_null(ref);
    ds_assert_null(con);
    ds_assert_null(wr);
    ds_assert_null(buf);
    
    cfgnode = dsConfigGetConfigByReference(config, ref);
    if (!cfgnode) return NULL;

    for (i=0; device_configuration[i].id; i++) {
	if (!xmlStrcmp(cfgnode->name, (unsigned char*)device_configuration[i].id)) break;
    }

    if (device_configuration[i].id) device_type = i;
    else {
	dsConfigReturnConfig(config, cfgnode);
	return NULL;
    }

    readcfg = dsXmlFindSubNode(cfgnode, "Readout");
    writecfg = dsXmlFindSubNode(cfgnode, "Writeout");
    chipcfg = dsXmlFindSubNode(cfgnode, "Configuration");
    proccfg = dsXmlFindSubNode(cfgnode, "Processor");
    
    if ((!readcfg)||(!writecfg)||(!chipcfg)||(!proccfg)) {
	dsConfigReturnConfig(config,cfgnode);
	return NULL;
    }

    connections = dsXmlFindSubNodes(readcfg, "Connections", "Connection");
    cconnections = dsXmlFindSubNodes(readcfg, "ControlConnections", "Connection");
    writers = dsXmlFindSubNodes(writecfg, "Writers", "Writer");

/*        
    if ((!connections)||(!writers)) {
	dsConfigReturnConfig(config,cfgnode);
	if (cconnections) dsXmlFreeSubNodes(cconnections);
	if (connections) dsXmlFreeSubNodes(connections);
	if (writers) dsXmlFreeSubNodes(writers);
	return NULL;
    }
*/

    mycon = (Connection*)malloc((connections?(connections->nodeNr + 1):1)*sizeof(Connection));
    if (mycon) {
	for (i=0;i<(connections?connections->nodeNr:0);i++) {
	    mycon[i] = ConnectionFind(con, dsXmlGetNodeString(connections->nodeTab[i], NULL, NULL));
	    if (!mycon[i]) {
		err = 1;
		break;
	    }
	}
	mycon[i] = NULL;
    } else err = DS_ERR_MEM;
    
    if ((cconnections)&&(cconnections->nodeNr>0)) {
	myccon = (Connection*)malloc((cconnections->nodeNr + 1)*sizeof(Connection));
	if (myccon) {
    	    for (i=0;i<cconnections->nodeNr;i++) {
		myccon[i] = ConnectionFind(con, dsXmlGetNodeString(cconnections->nodeTab[i], NULL, NULL));
		if (!myccon[i]) {
		    err = 1;
		    break;
		}
	    }
	    myccon[i] = NULL;
	} else err = 1;
    } else myccon = NULL;
    
    mywr = (Writer*)malloc((writers?(writers->nodeNr + 1):1)*sizeof(Writer));
    if (mywr) {
	for (i=0;i<(writers?writers->nodeNr:0);i++) {
	    mywr[i] = WriterFind(wr, dsXmlGetNodeString(writers->nodeTab[i], NULL, NULL));
	    if (!mywr[i]) {
		err = 1;
		break;
	    }
	}
	mywr[i] = NULL;
    } else err = DS_ERR_MEM;

    if (cconnections) dsXmlFreeSubNodes(cconnections);
    if (connections) dsXmlFreeSubNodes(connections);
    if (writers) dsXmlFreeSubNodes(writers);

    tmp = dsXmlGetPropString(cfgnode, "id", NULL);
    if (tmp) err = dsExchangeString(NULL, &id, tmp);
    else err = DS_ERR_CONFIG;

    dsConfigReturnConfig(config, cfgnode);

    omf = dsTimeCreate(NULL);
    if (!omf) err = DS_ERR_MEM;
    
    if (!err) {    
	lock = dsLockCreateContext(NULL);
	if (lock) err = dsLockInitContext(lock);
	else err = DS_ERR_MEM;
    }

    if (!err) {    
	cqlock = dsLockCreateContext(NULL);
	if (cqlock) err = dsLockInitContext(cqlock);
	else err = DS_ERR_MEM;
    }

    if (!err) {    
	cqcond = dsConditionCreateContext(NULL);
	if (cqcond) err = dsConditionInitContext(cqcond);
	else err = DS_ERR_MEM;
    }
    
    if (!err) {
	cthread = dsThreadCreateContext(NULL);
	if (cthread) err = dsThreadInitContext(cthread);
	else err = DS_ERR_MEM;
    }

    if (!err) {
	rthread = dsThreadCreateContext(NULL);
	if (rthread) err = dsThreadInitContext(rthread);
	else err = DS_ERR_MEM;
    }

#ifdef REAL_TIME_READER	
    if (!err) {
	dsThreadSetScheduler((DSBaseThreadContext)rthread, DS_THREAD_SCHEDULER_REALTIME, NULL);
	dsThreadSetPriority((DSBaseThreadContext)rthread, dsThreadGetMaximalPriority((DSBaseThreadContext)rthread));
    }
#endif /* REAL_TIME_READER */

    if (!err) {
	cfgref = dsConfigCopyReference(config, ref);
	if (!cfgref) err = DS_ERR_MEM;
    }

    if (!err) {
	dev = device_configuration[device_type].create(config, cfgref);
	if (dev) dev->type = device_configuration[device_type].type;
	else err = DS_ERR_MEM;
    }
    
    if (err) {
	if (omf) dsTimeDestroy(omf);
	if (cfgref) dsConfigDestroyReference(config, cfgref);
	if (lock) dsLockDestroyContext(lock);
	if (cqlock) dsLockDestroyContext(cqlock);
	if (cqcond) dsConditionDestroyContext(cqcond);
	if (cthread) dsThreadDestroyContext(cthread);
	if (rthread) dsThreadDestroyContext(rthread);
	if (myccon) free(myccon);
	if (mycon) free(mycon);
	if (mywr) free(mywr);
	if (id) dsExchangeString(NULL, &id, NULL);
	return NULL;
    }

    dev->operation_mode_finishing = omf;
    dsTimeZeroTime(dev->operation_mode_finishing);

    dev->lock = lock;
    dev->id = id;
    dev->reader_thread = rthread;
    dev->control_thread = cthread;
    dev->ccon = myccon;
    dev->con = mycon;
    dev->wr = mywr;
    dev->buffer = buf;
    dev->cfg = config;
    dev->cfgref = cfgref;
    dev->sf = 1;

    dev->cqlock = cqlock;
    dev->cqcond = cqcond;
    dev->cq = dev->cqr = NULL;
    dev->cr = NULL;
    dev->cqsize = 0;

    dev->operation_mode = DEVICE_OPERATION_STANDARD_MODE;
    dev->saved_operation_mode = DEVICE_OPERATION_STANDARD_MODE;

    dev->curcon = NULL;
    dev->curccon = NULL;
    
    dev->dsc = NULL;
    dev->multimessage_delay = NULL;
    
    return dev;
}

int DeviceSetDSC(Device dev, DSDatagramServerContext dsc) {
    ds_assert_param(dev);

    dev->dsc = dsc;
    return 0;
}

int DeviceSetDebugDSC(Device dev, DSDatagramServerContext dsc) {
    ds_assert_param(dev);

    dev->debug_dsc = dsc;
    return 0;
}

int DeviceSetControlMessageDelay(Device dev, DSDuration mmdelay) {
    dev->multimessage_delay = mmdelay;
}

void DeviceDestroy(Device dev) {
    DeviceControlRequest cr;
    Connection *con, *ccon;
    Writer *wr;

    ds_assert_return(dev);

    while (dev->cqr) {
	cr = dev->cqr;
	dev->cqr = cr->next;
	DeviceControlRequestDestroy(cr);
    }

    dsTimeDestroy(dev->operation_mode_finishing);

    dsThreadDestroyContext(dev->reader_thread);
    dsThreadDestroyContext(dev->control_thread);
    dsConfigDestroyReference(dev->cfg, dev->cfgref);
    
    dsLockDestroyContext(dev->lock);
    dsLockDestroyContext(dev->cqlock);
    dsConditionDestroyContext(dev->cqcond);

    ccon = dev->ccon;
    con = dev->con;
    wr = dev->wr;

    dsExchangeString(NULL, &dev->id, NULL);
    dsExchangeString(NULL, &dev->text_separator, NULL);

    if (dev->destroy) dev->destroy(dev);
    else free(dev);

    if (ccon) free(ccon);
    free(con);
    free(wr);

}

void DeviceSetDefaults(Device dev) {
    ds_assert_return(dev);
    
    memset(dev, 0, sizeof(DeviceS));
    dev->status = DEVICE_STATUS_NOT_CONNECTED;
}

int DeviceLock(Device dev) {
    ds_assert_param(dev);
    return dsLockWrite(dev->lock);
}

void DeviceUnlock(Device dev) {
    ds_assert_return(dev);
    dsLockReturn(dev->lock);
}

int DeviceStart(Device dev) {
    int err;
    
    ds_assert_param(dev);

    DeviceConfigure(dev);
        
    if (dev->reader) {
	err = dsThreadRun(dev->control_thread, (DSThreadFunction)DeviceControlLoop, (void*)dev);
	if (err) return err;	
	return dsThreadRun(dev->reader_thread, (DSThreadFunction)dev->reader, (void*)dev);
    }
    
    return 0;
}

void DeviceStop(Device dev) {
    dsThreadTerminate((DSBaseThreadContext)dev->reader_thread);
    dsThreadTerminate((DSBaseThreadContext)dev->control_thread);

    dsConditionBroadcast(dev->cqcond);    
}

void DeviceWaitTermination(Device dev) {
    dsThreadWaitTerminate(dev->reader_thread);
    dsThreadWaitTerminate(dev->control_thread);
}


void DeviceReset(Device dev) {
    ds_assert_return(dev);
    
    dsDebugServer(DS_DEBUG_WARNING, "Reseting device: %s", dev->id);
    
    if (dev->reset) dev->reset(dev);
}

int DeviceDebug(Device dev) {
    ds_assert_param(dev);
    
    if (dev->debug) return dev->debug(dev);
    return 0;
}

char *DeviceControl(Device dev, const char *request) {
    ds_assert_null(dev);
    
    if (dev->control) return dev->control(dev, request);
    return NULL;
}

int DeviceQueueControl(Device dev, ADASCommand cmd, uint32_t reqid, const char *req, DSDatagramServerContext dsc, DSConnectionAddress *request_addr) {
    ds_assert_param(dev);
    ds_assert_param(req);

    return DeviceControlRequestAdd(dev, cmd, reqid, req, dsc, request_addr);
}



int DeviceSetConfiguration(Device dev, const char *config) {
    int err = 0;
    
    xmlDocPtr doc;
    xmlNodePtr rnode, cnode, pnode, lnode, node;
    
    ds_assert_param(dev);

    dsDebugServer(DS_DEBUG_INFO, "Device(%s). Configuring Device", dev->id);
    
    doc = xmlReadMemory(config, strlen(config), "NewConfiguration", "UTF-8", 0);
    if (!doc) return DS_ERR_CONFIG_INVALID_CONFIG;
    
    rnode = xmlDocGetRootElement(doc);
    if (rnode) {
	cnode = dsXmlFindSubNode(rnode, "Configuration");
	pnode = dsXmlFindSubNode(rnode, "Processor");
	lnode = dsXmlFindSubNode(rnode, "Layout");
    } else {
	xmlFreeDoc(doc);
	return DS_ERR_CONFIG_INVALID_CONFIG;
    }
    
    node = dsConfigGetConfigByReference(dev->cfg, dev->cfgref);
    if (node) {
	if (cnode) {
	    err = dsConfigSetNode(dev->cfg, cnode, node, "Configuration");
	    if (err) {
		dsConfigReturnConfig(dev->cfg, node);
		xmlFreeDoc(doc);
		return err;
	    }
	}

	if (pnode) {
	    err = dsConfigSetNode(dev->cfg, pnode, node, "Processor");
	    if (err) {
		dsConfigReturnConfig(dev->cfg, node);
		xmlFreeDoc(doc);
		return err;
	    }
	}

	if (lnode) {
	    err = dsConfigSetNode(dev->cfg, lnode, node, "Layout");
	    if (err) {
		dsConfigReturnConfig(dev->cfg, node);
		xmlFreeDoc(doc);
		return err;
	    }
	}
    
	dsConfigReturnConfig(dev->cfg, node);
    } else {
	xmlFreeDoc(doc);
	return DS_ERR_CONIFG_NOTFOUND;
    }

    if (dev->setconfig) err = dev->setconfig(dev, doc);

    xmlFreeDoc(doc);
    
    if (!err) {
	DeviceReset(dev);
	dsConfigSave(dev->cfg);
    }

    return err;
}

int DeviceContinueLoop(Device dev) {
    return dsThreadCheckStatusBitsOff((DSBaseThreadContext)dev->reader_thread,DS_THREAD_TERMINATE_FLAG);
}

Connection DeviceInitializeConnection(Device dev) {
    int err = 0;
    int i;

    xmlNodePtr cfgnode, readcfg;
    
    ds_assert_null(dev);

    cfgnode = dsConfigGetConfigByReference(dev->cfg, dev->cfgref);
    if (cfgnode) {
	readcfg = dsXmlFindSubNode(cfgnode, "Readout");
	if (readcfg) {
	    dev->max_text_size = dsXmlGetPropLong(readcfg,"MaxStringSize", TEXT_READER_MAX_STRING_SIZE);
	    if (dsExchangeString(NULL, &dev->text_separator, dsXmlGetNodeString(readcfg, "RecordSeparator", TEXT_READER_RECORD_SEPARATOR))) err = DS_ERR_MEM;
	} else err = DS_ERR_CONFIG;
	dsConfigReturnConfig(dev->cfg, cfgnode);
    } else err = DS_ERR_CONFIG;

    if (err) return NULL;

    DeviceLock(dev);
    if (dev->curcon) {
	DeviceUnlock(dev);
	return NULL;
    }
        
    for (i=0;dev->con[i];i++) {
	err = ConnectionOpen(dev->con[i]);
	if (!err) {
	    dev->curcon = dev->con[i];
	    DeviceUnlock(dev);
	    return dev->curcon;
	}
    }
    DeviceUnlock(dev);
    return NULL;
}

Connection DeviceGetConnection(Device dev) {
    ds_assert_null(dev);

    DeviceLock(dev);
    if ((dev->curcon)&&(!ConnectionLock(dev->curcon))) {
	DeviceUnlock(dev);
	return dev->curcon;
    }
    DeviceUnlock(dev);
    return NULL;
}

Connection DeviceTryConnection(Device dev) {
    ds_assert_null(dev);

    DeviceLock(dev);
    if ((dev->curcon)&&(!ConnectionTryLock(dev->curcon))) {
	DeviceUnlock(dev);
	return dev->curcon;
    }
    DeviceUnlock(dev);
    return NULL;
}

void DeviceReturnConnection(Device dev) {
    ConnectionUnlock(dev->curcon);
}

void DeviceCloseConnection(Device dev) {
    ds_assert_return(dev);
    
    DeviceLock(dev);
    if (dev->curcon) {
	ConnectionClose(dev->curcon);
	dev->curcon = NULL;
    }
    DeviceUnlock(dev);
}

char *DeviceReadText(Device dev, Connection con, const char *separator) {
    int err = 0;
    char *buf;
    size_t pos = 0;
    size_t readed;
    size_t separator_size;

    ds_assert_null(dev);
    
    if (!separator) separator = dev->text_separator;
    separator_size = strlen(separator);

    buf = dsMalloc(NULL, (dev->max_text_size+1)*sizeof(char));
    if (!buf) return NULL;

//    memset(buf, 0, dev->max_text_size);
    while ((DeviceContinueLoop(dev))&&(!err)) {
	err = ConnectionVariableRead(con, buf + pos, dev->max_text_size - pos, &readed);
	if ((!err)&&(readed>0)) {
	    buf[pos+readed] = 0;
	    if ((!separator)||(!*separator)) {
		if (pos+readed==dev->max_text_size) return buf;
		pos+=readed;
		continue;	    
	    } else if (!strncmp(buf+pos+readed-separator_size, separator, separator_size)) {
		return buf;
	    } else if (pos+readed==dev->max_text_size) {
		free(buf);
		return NULL;
	    } else {
		pos+=readed;
		continue;
	    }
	} else if (err == DS_ERR_TIMEOUT) {
	    if (((!separator)||(!*separator))&&(pos>0)) {
		buf[pos] = 0;
		return buf;
	    }
	    break;
	} else break;	        
    }
    dsFree(NULL, buf);
    return NULL;
}


Connection DeviceInitializeControlConnection(Device dev) {
    int err = 0;
    int i;

    ds_assert_null(dev);

    DeviceLock(dev);
    if (dev->curccon) {
	DeviceUnlock(dev);
	return NULL;
    }
        
    for (i=0;dev->ccon[i];i++) {
	err = ConnectionOpen(dev->ccon[i]);
	if (!err) {
	    dev->curccon = dev->ccon[i];
	    DeviceUnlock(dev);
	    return dev->curccon;
	}
    }
    DeviceUnlock(dev);
    return NULL;
}

Connection DeviceGetControlConnection(Device dev) {
    ds_assert_null(dev);

    DeviceLock(dev);
    if ((dev->curccon)&&(!ConnectionLock(dev->curccon))) {
	DeviceUnlock(dev);
	return dev->curccon;
    }
    DeviceUnlock(dev);
    return NULL;
}

Connection DeviceTryControlConnection(Device dev) {
    ds_assert_null(dev);

    DeviceLock(dev);
    if ((dev->curccon)&&(!ConnectionTryLock(dev->curccon))) {
	DeviceUnlock(dev);
	return dev->curccon;
    }
    DeviceUnlock(dev);
    return NULL;
}

void DeviceReturnControlConnection(Device dev) {
    DeviceLock(dev);
    ConnectionUnlock(dev->curccon);
    DeviceUnlock(dev);
}

void DeviceCloseControlConnection(Device dev) {
    ds_assert_return(dev);
    
    DeviceLock(dev);
    if (dev->curccon) {
	ConnectionClose(dev->curccon);
	dev->curccon = NULL;
    }
    DeviceUnlock(dev);
}


int DeviceConfigure(Device dev) {
    ds_assert_param(dev);
    
    if (dev->configure) return dev->configure(dev);
    return 0;
}

DeviceWriter DeviceInitWriters(Device dev, const char *instid) {
    int i, prefixlen;
    const char *prefix = NULL;
    char *id;
    DeviceWriter dw;
    
    xmlNodePtr cfgnode, writecfg;
    
    ds_assert_null(dev);
    
    cfgnode = dsConfigGetConfigByReference(dev->cfg, dev->cfgref);
    if (cfgnode) {
	writecfg = dsXmlFindSubNode(cfgnode, "Writeout");
	if (writecfg) {
	    prefix = dsXmlGetNodeString(writecfg, "DataPath", NULL);
	}
    }
    if (!prefix) prefix = dev->id;
    
    if (instid) {
	prefixlen = strlen(prefix);
	while ((prefixlen)&&(prefix[prefixlen-1]=='/')) prefixlen--;

	id = malloc((prefixlen + strlen(instid) + 1)*sizeof(char));
	if (id) {
	    memcpy(id, prefix, prefixlen);
	    strcpy(id + prefixlen, instid);
	}
    } else {
	prefixlen = strlen(prefix);
	while ((prefixlen)&&(prefix[prefixlen-1]=='/')) prefixlen--;

	id = malloc((prefixlen + 1)*sizeof(char));
	if (id) {
	    memcpy(id, prefix, prefixlen);
	    id[prefixlen] = 0;
	}
    }
    
    if (cfgnode) dsConfigReturnConfig(dev->cfg, cfgnode);

    if (!id) return NULL;

    for (i=0;dev->wr[i];i++);

    dw = (DeviceWriter)malloc(i*sizeof(WriterInstance));
    if (!dw) {
	free(id);
	return dw;
    }
    
    for (i=0;dev->wr[i];i++) {
	dw[i] = WriterCreateInstance(dev->wr[i], dev, id);
	if (!dw[i]) {
	    dsDebugServer(DS_DEBUG_ERROR, "Device(%s:%s). Initializing Writer Instance (%s) Failed", dev->id, id, dev->wr[i]->id);
	}
    }
    
    free(id);
    
    return dw;
}

void DeviceFreeWriters(Device dev, DeviceWriter dw) {
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) WriterDestroyInstance(dev->wr[i], dw[i]);
    }
    free(dw);
}

int DeviceWritersReset(Device dev, DeviceWriter dw) {
    int err;
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterResetInstance(dev->wr[i], dw[i]);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Reset failed with Error: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }
    return 0;
}


int DeviceOpenWriters(Device dev, DeviceWriter dw, const char *suffix, DSTime time) {
    int err;
    int i;
    
    ds_assert_param(dev);
    ds_assert_param(dw);
    ds_assert_param(time);

    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterOpen(dev->wr[i], dw[i], suffix, time);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Open Failed with Error: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }

    
    return 0;
}

void DeviceCloseWriters(Device dev, DeviceWriter dw) {
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);

    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    WriterClose(dev->wr[i], dw[i]);
	}
    }

}

int DeviceWritersSetQuality(Device dev, DeviceWriter dw, double quality) {
    int err;
    int i;
    
    ds_assert_param(dev);
    ds_assert_param(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterSetQuality(dev->wr[i], dw[i], quality);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Error Setting Quality: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }
    
    return 0;
}

int DeviceWritersSetDuration(Device dev, DeviceWriter dw, DSDuration duration) {
    int err;
    int i;
    
    ds_assert_param(dev);
    ds_assert_param(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterSetDuration(dev->wr[i], dw[i], duration);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Error Setting Duration: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }
    
    return 0;
}

void DeviceWritersStartBlock(Device dev, DeviceWriter dw) {
    int err;
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) WriterStartBlock(dev->wr[i], dw[i]);
    }
}

void DeviceWritersFinishBlock(Device dev, DeviceWriter dw) {
    int err;
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) WriterFinishBlock(dev->wr[i], dw[i]);
    }
}

void DeviceWritersEnableFixedConfigOptimization(Device dev, DeviceWriter dw) {
    int err;
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) WriterEnableFixedConfigOptimization(dev->wr[i], dw[i]);
    }
}

void DeviceWritersEnableFixedLayoutOptimization(Device dev, DeviceWriter dw, const char *suffix) {
    int err;
    int i;
    
    ds_assert_return(dev);
    ds_assert_return(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) WriterEnableFixedLayoutOptimization(dev->wr[i], dw[i], suffix);
    }
}


int DeviceWritersSetDurationMilliseconds(Device dev, DeviceWriter dw, unsigned long duration) {
    int err;
    int i;
    DSDuration dur;
    
    ds_assert_param(dev);
    ds_assert_param(dw);

    dur = dsDurationCreate(NULL);
    if (!dur) return DS_ERR_MEM;
    
    dsDurationImportMilliseconds(dur, duration);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterSetDuration(dev->wr[i], dw[i], dur);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Error Setting Duration: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }
    
    dsDurationDestroy(dur);
    
    return 0;
}


	

int DeviceWrite(Device dev, DeviceWriter dw, WriterDataType type, WriterDataOptions options, WriterLayerID layerid, WriterDataID n, WriterDataID *id, WriterData data) {
    int err;
    int i;
    
    ds_assert_param(dev);
    ds_assert_param(dw);
    
    for (i=0;dev->wr[i];i++) {
	if (dw[i]) {
	    err = WriterWrite(dev->wr[i], dw[i], type, options, layerid, n, id, data);
	    if (err) dsDebugServer(DS_DEBUG_ERROR, "Writer(%s). Data Writeout Failed with Error: %X(%s)", dev->wr[i]->id, err, dsError(err, NULL));
	}
    }
    
    return 0;
}

xmlNodePtr DeviceGetDriverInfo(Device dev) {
    int i;
    xmlNodePtr res;
    char stmp[32];
    
    res = xmlNewNode(NULL, "DriverInfo");
    if (res) {
	xmlSetProp(res, "id", dev->id);

	for (i=0;device_configuration[i].id;i++) {
	    if (device_configuration[i].type == dev->type) {
		xmlNewChild(res, NULL, "Driver", device_configuration[i].id);
		break;
	    }
	}
	
	sprintf(stmp, "%u", dev->version);
	xmlNewChild(res, NULL, "DriverVersion", stmp);

	if (dev->getdrvinfo) dev->getdrvinfo(dev, res);
    }	
    
    return res;	
}

xmlNodePtr DeviceGetConfig(Device dev, DeviceConfgurationSection sections) {
    xmlNodePtr res, cfgnode, node, section;

    cfgnode = dsConfigGetConfigByReference(dev->cfg, dev->cfgref);
    if (cfgnode) {
	res = xmlNewNode(NULL, "DeviceConfiguration");
	if (res) {
	    xmlSetProp(res, "id", dev->id);
	    if (sections&DEVICE_CONFIGURATION_CONFIGURATION) {
	        section = dsXmlFindSubNode(cfgnode, "Configuration");
		if (section) {
		    node = xmlCopyNode(section, 1);
		    if (node) xmlAddChild(res, node);
		}
	    }

	    if (sections&DEVICE_CONFIGURATION_PROCESSOR) {
		section = dsXmlFindSubNode(cfgnode, "Processor");
		if (section) {
		    node = xmlCopyNode(section, 1);
		    if (node) xmlAddChild(res, node);
		}
	    }

	    if (sections&DEVICE_CONFIGURATION_WRITEOUT) {
		section = dsXmlFindSubNode(cfgnode, "Writeout");
		if (section) {
		    node = xmlCopyNode(section, 1);
		    if (node) xmlAddChild(res, node);
		}
	    }

	    if (sections&DEVICE_CONFIGURATION_READOUT) {
		section = dsXmlFindSubNode(cfgnode, "Readout");
		if (section) {
		    node = xmlCopyNode(section, 1);
		    if (node) xmlAddChild(res, node);
		}
	    }

	    if (sections&DEVICE_CONFIGURATION_LAYOUT) {
		section = dsXmlFindSubNode(cfgnode, "Layout");
		if (section) {
		    node = xmlCopyNode(section, 1);
		    if (node) xmlAddChild(res, node);
		}
	    }
	}
	dsConfigReturnConfig(dev->cfg, cfgnode);
    } else res = NULL;

    return res;
}

const char *DeviceGetDeviceID(Device dev) {
    ds_assert_null(dev);
    return dev->id;
}

const char *DeviceGetDeviceName(Device dev) {
    int i;
    
    ds_assert_null(dev);
    
    for (i=0;device_configuration[i].id;i++) {
	if (dev->type == device_configuration[i].type) {
	    return device_configuration[i].id;
	}
    }
    return NULL;
}

    // Add Locking to synchronise (Currently from single thread)
int DeviceSetOperationMode(Device dev, DeviceOperationMode mode, DSDuration duration) {
    int err = 0;

    ds_assert_param(dev);
    
    if (duration) {
	dsTimeCurrentTime(dev->operation_mode_finishing);
	dsTimeAdd(dev->operation_mode_finishing, duration);
    } else dsTimeZeroTime(dev->operation_mode_finishing);

    if (dev->operation_mode != mode) {
	if (duration) dev->saved_operation_mode = dev->operation_mode;

	dev->operation_mode = mode;
	if (dev->applymode) err = dev->applymode(dev);
	else DeviceReset(dev);
    } 

    return err;
}

int DeviceManageOperationMode(Device dev) {
    int err = 0;
    
    DSTime current_time;
    
    if ((dev->operation_mode != DEVICE_OPERATION_STANDARD_MODE)&&(!dsTimeIsZero(dev->operation_mode_finishing))&&(dsTimeCurrentCmp(dev->operation_mode_finishing)<0)) {
	dev->operation_mode = dev->saved_operation_mode;
	dev->saved_operation_mode = DEVICE_OPERATION_STANDARD_MODE;
	dsTimeZeroTime(dev->operation_mode_finishing);
	
	if (dev->applymode) err = dev->applymode(dev);
	else DeviceReset(dev);
    }
    return err;
}

static DeviceControlRequest DeviceControlRequestCreate(ADASCommand cmd, uint32_t reqid, const char *req, DSDatagramServerContext dsc, DSConnectionAddress *addr) {
    DeviceControlRequest cr;
    
    cr = dsMalloc(NULL, sizeof(DeviceControlRequestS));
    if (!cr) return NULL;
    
    if (dsSetString(NULL, &cr->req, req)) {
	dsFree(NULL, cr);
	return NULL;
    }

    cr->cmd = cmd;
    cr->reqid = reqid;
    cr->dsc = dsc;
    dsConnectionAddressCpy(&cr->addr, addr);
    cr->next = NULL;
    
    return cr;
}

static void DeviceControlRequestDestroy(DeviceControlRequest cr) {
    dsExchangeString(NULL, &cr->req, NULL);
    dsFree(NULL, cr);
}

static int DeviceControlRequestProcess(Device dev) {
    int i,j;
    char *membuf, *membufptr;
    size_t id_size, membufsize;

    unsigned char request_buffer[DS_MAX_UDP_MESSAGE_SIZE];

    DeviceControlRequest cr;
    
    while (dev->cqr) {
	dsLockWrite(dev->cqlock);
	cr = dev->cqr;
	dev->cqr = cr->next;
	dev->cqsize--;
	
	dev->cr = cr;
	dsLockReturn(dev->cqlock);
    
	dsDebugServer(DS_DEBUG_DEBUG, "Device(%s). Processing control request: %s", dev->id, cr->req);

	request_buffer[0] = cr->cmd;
	*(uint32_t*)(request_buffer+4) = cr->reqid;

	membuf = DeviceControl(dev, cr->req);
	if (membuf) {
	    id_size = strlen(membuf)+1;
	    membufptr = membuf;
	
	    for (i=0;((id_size+8)>DS_MAX_UDP_MESSAGE_SIZE);i++) {
		request_buffer[1] = 1;
		*(uint16_t*)(request_buffer+2) = i;
		memcpy(request_buffer+8, membufptr, DS_MAX_UDP_MESSAGE_SIZE-4);
		dsDatagramServerWrite(cr->dsc, &cr->addr, request_buffer, DS_MAX_UDP_MESSAGE_SIZE, 0, NULL);
		id_size -= (DS_MAX_UDP_MESSAGE_SIZE-8);
		membufptr += (DS_MAX_UDP_MESSAGE_SIZE-8);

		if (dev->multimessage_delay) dsWait(dev->multimessage_delay);
	    }
	    request_buffer[1] = 0;
	    *(uint16_t*)(request_buffer+2) = i;
	    memcpy(request_buffer+8, membufptr, id_size);
	    free(membuf);

	    dsDatagramServerWrite(cr->dsc, &cr->addr, request_buffer, id_size + 8, 0, NULL);
	}
/*
	 else {
	    memset(request_buffer+1,0,3);
	    strcpy(request_buffer+8, "No response");
	    id_size = strlen("No response");
	}
	dsDatagramServerWrite(cr->dsc, &cr->addr, request_buffer, id_size + 8, 0, NULL);
*/

	dsLockWrite(dev->cqlock);
	dev->cr = NULL;
	dsLockReturn(dev->cqlock);
    
	DeviceControlRequestDestroy(cr);
    }

    return 0;    
}


static int DeviceControlRequestAdd(Device dev, ADASCommand cmd, uint32_t reqid, const char *req, DSDatagramServerContext dsc, DSConnectionAddress *request_addr) {
    DeviceControlRequest cr;

    if (dev->cqsize > CONTROL_QUEUE_SIZE) return DS_ERR_BUSY;

    dsLockWrite(dev->cqlock);
    if ((dev->cr)&&(!strcmp(dev->cr->req, req))) {
	dsLockReturn(dev->cqlock);
	return 0;
    }
    
    for (cr=dev->cqr;cr;cr=cr->next) {
	if (!strcmp(req,cr->req)) {
	    dsLockReturn(dev->cqlock);
	    return 0;
	}
    }
    dsLockReturn(dev->cqlock);
    
    cr = DeviceControlRequestCreate(cmd, reqid, req, dsc, request_addr);
    if (!cr) return DS_ERR_MEM;

    dsDebugServer(DS_DEBUG_DEBUG, "Device(%s). Queueing control request: %s", dev->id, req);
    
    dsLockWrite(dev->cqlock);
    if (dev->cqr) {
	dev->cq->next = cr;
	dev->cq = cr;
    } else {
	dev->cqr = cr;
	dev->cq = cr;
    }
    dev->cqsize++;
    dsLockReturn(dev->cqlock);

    dsConditionBroadcast(dev->cqcond);    
    
    return 0;    
}


static int DeviceControlLoop(Device dev) {
    DeviceControlQueue req;
    
    while (dsThreadCheckStatusBitsOff((DSBaseThreadContext)dev->control_thread,DS_THREAD_TERMINATE_FLAG)) {
	dsConditionPreLock(dev->cqcond);
	if (dev->cqr) dsConditionUnLock(dev->cqcond);
	else dsConditionWait(dev->cqcond);
	
	DeviceControlRequestProcess(dev);
    }

    return 0;
}
