#include <stdio.h>

#include <dsdebug.h>
#include <dsassert.h>

#include "buffer.h"

#include "namtbuffer.h"

static void namtBufferDestroy(NAMTReaderBuf buf);

int namtBufferAdd(NAMTDevice dev, NAMTData *data, NAMTData *cdata, NAMTEvents events, NAMTSpectrum *spectrum, uint16_t errors) {
    int err;
    int i;
    
    Buffer buf;
    DSTime tm;
    NAMTReaderBuf rb;
    
    unsigned short *detectors;

    ds_assert_param(dev);
    
    buf = dev->device.buffer;
    tm = dev->datatime;
    
    ds_assert_param(buf);
    ds_assert_param(tm);
    
    rb = (NAMTReaderBuf)malloc(sizeof(NAMTReaderBufS));
    detectors = (unsigned short*)malloc(dev->devices * sizeof(unsigned short));
    if ((!rb)||(!detectors)) err = DS_ERR_MEM;
    else err = ReaderBufInit((ReaderBuf)rb, (Device)dev, tm);
    if (err) {
	if (rb) free(rb);
	if (detectors) free(detectors);
	return err;
    }

    rb->data = data;
    rb->cdata = cdata;
    rb->spectrum = spectrum;
    rb->events = events;
    
    rb->spectrum_iteration = dev->write_spectrum_last_iteration;
    rb->current_iteration = dev->current_iteration;
    
    rb->devices = dev->devices;
    rb->data_size = dev->data_size;
    rb->detectors = detectors;
    for (i=0;i<dev->devices;i++) detectors[i] = dev->detectors[i];

    rb->rt_time = dev->rt_time;
    rb->recs = dev->recs;
    rb->desync = dev->desync;
    rb->errors = errors;
    
    rb->device_type = dev->device_type;
    rb->device_version = dev->device_version;
    
    rb->sf = dev->sf;
    dev->sf = 0;

    rb->plain = dev->plain;
    if (dev->plain) {
	rb->coincidences = dev->coincidences;
	rb->spectrums = dev->spectrums;
    }
    
    rb->buf.destroy = (ReaderBufDestroyFunction)namtBufferDestroy;
    
    BufferAdd(buf, (ReaderBuf)rb);
    dsDebugServer(DS_DEBUG_DEBUG, "Buffer: New NAMT event");

    return 0;
}

static void namtBufferDestroy(NAMTReaderBuf buf) {
    int i;
    
    ds_assert_return(buf);
    
    for (i=0;i<buf->devices;i++) {
	if ((buf->data)&&(buf->data[i])) free(buf->data[i]);
	if ((buf->cdata)&&(buf->cdata[i])) free(buf->cdata[i]);
	if ((buf->spectrum)&&(buf->spectrum[i])) free(buf->spectrum[i]);
	if (buf->events) namtEventsFree(buf->events+i);
    }
    if (buf->data) free(buf->data);
    if (buf->cdata) free(buf->cdata);
    if (buf->spectrum) free(buf->spectrum);
    if (buf->events) free(buf->events);
    if (buf->detectors) free(buf->detectors);

    ReaderBufFree((ReaderBuf)buf);
    free(buf);
}

