#include <stdio.h>

#include <dsdebug.h>
#include <dserror.h>


#include "../snt/commands.h"
#include "../snt/error.h"

#include "lowlevel.h"

#include "namterror.h"

int namtProcessError(NAMTDevice dev, Connection con,  int req, int resp, DSDebugPriority mdl) {
    int err;
    
    if (resp<0) {
	switch(resp) {
	    case LL_ERR_TIMEOUT:
		err = DS_ERR_TIMEOUT;
	    break;
	    case LL_ERR_READ:
		err = DS_ERR_CONNECTION_READ;
	    break;
	    case LL_ERR_WRITE:
		err = DS_ERR_CONNECTION_WRITE;
	    break;
	    case LL_ERR_MEM:
		err = DS_ERR_MEM;
	    break;
	    default:
		err = DS_ERR_INTERNAL;
	}
	dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Communication error: %X(%s)", dev->device.id, err, dsError(err, NULL));
    } else if ((resp==COMMAND_RESPONSE_ERROR)||(resp==COMMAND_RESPONSE_RETRY)) {
	resp = con->llerr; /*ReadByte(con);*/
	switch(resp) {
	    case COMMAND_ERROR_INVALID_REQUEST:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Invalid request", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_NOT_SUPPORTED:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Operation not supported", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_INVALID_PARAMETER:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Invalid parameter", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_INVALID_INSTALLATION_ID:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Invalid sub-device", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_NOT_INITIALIZED:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Microcode is not initialized", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_NOT_ENOUGH_TIME:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Not enough time", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_UART:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Communication error", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_CONTROL_RS485:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: RS485 communication error", dev->device.id);
		err = DS_ERR_INTERNAL;
	    break;
	    case COMMAND_ERROR_NO_DATA:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: No data available", dev->device.id);
		err = ADAS_ERR_EXCHANGE_TIME_IS_OVER;
	    break;
	    case COMMAND_ERROR_EXCHANGE_TIME_IS_OVER:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Data exchange time is over", dev->device.id);
		err = ADAS_ERR_EXCHANGE_TIME_IS_OVER;
	    break;
	    case -1:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Timeout reading error code", dev->device.id);
		err = DS_ERR_INTERNAL;
	    default:
		dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Unknown error: %i", dev->device.id, resp);
		err = DS_ERR_INTERNAL;
	}
    } else {
	err = ConnectionSkipAll(con);
	dsDebugServer((mdl>DS_DEBUG_ERROR)?mdl:DS_DEBUG_ERROR, "NAMT(%s). Microcode error: Invalid answer. Request: %lx, Answer: %lx. Skipped Bytes: %lu", dev->device.id, req, resp, err);
	err = DS_ERR_INTERNAL;
    }

    return err;
} 

void namtErrorLogMicrocodeErrors(NAMTDevice dev, uint16_t errors) {
    if (errors&MCU_ERROR_BUFFER_FULL)
	dsDebugServer(DS_DEBUG_CRITICAL, "NAMT(%s). Microcode Error: Buffer Overrun!", dev->device.id);
    if (errors&MCU_ERROR_GATE_TOO_LONG)
	dsDebugServer(DS_DEBUG_CRITICAL, "NAMT(%s). Microcode Error: Gate is open too long!", dev->device.id);
    if (errors&MCU_ERROR_EVENT_BUFFER_FULL)
	dsDebugServer(DS_DEBUG_CRITICAL, "NAMT(%s). Microcode Error: Event Buffer Overrun!", dev->device.id);
}
