#ifndef _NAMT_READER_H
#define _NAMT_READER_H

#include <dslib.h>

typedef struct NAMTDataT *NAMTData;
typedef struct NAMTDetectorDataT *NAMTDetectorData;
typedef struct NAMTDetectorCDataT *NAMTDetectorCData;

#include "defaults.h"

struct NAMTDetectorDataT {
    union {
	struct {
	    uint32_t up[MAX_NAMT_DETECTORS];
	    uint32_t down[MAX_NAMT_DETECTORS];
	} namt;
	struct {
	    uint32_t sensor[MAX_NAMT_SENSORS];
	} plain;	
    };
};
typedef struct NAMTDetectorDataT NAMTDetectorDataS;

struct NAMTDetectorCDataT {
    union {
	struct {
	    uint32_t coincidences[MAX_NAMT_DETECTORS * MAX_NAMT_DETECTORS];
	    uint32_t many_many;
	    uint32_t many_zero;
    	    uint32_t zero_many;
	    uint32_t zero_one;
	    uint32_t one_zero;
	    uint32_t many_one;
	    uint32_t one_many;
	    uint32_t total;
	} namt;
	struct {
	    uint32_t coincidences[MAX_NAMT_COINCIDENCES];
	} plain;
    };
};
typedef struct NAMTDetectorCDataT NAMTDetectorCDataS;

typedef void *NAMTSpectrum;


typedef struct NAMTEventT *NAMTEvent;

struct NAMTEventsT {
    unsigned char subdevice;

    unsigned short int mmevents;
    NAMTEvent *mmevent;
};
typedef struct NAMTEventsT *NAMTEvents;
typedef struct NAMTEventsT NAMTEventsS;

/*
#define NAMTDetectorPackedDataT(type, size) \
	struct { \
	    uint32_t iter; \
	    uint16_t duration; \
	    type up[size]; \
	    type down[size]; \
	} __attribute__ ((__packed__))


#define namtDetectorPackedDataSize(data_size, detectors) \
    ((data_size == 16)?sizeof(NAMTDetectorPackedDataT(uint16_t, detectors)):sizeof(NAMTDetectorPackedDataT(uint32_t, detectors)))

#define namtDataInternal(ptr, type, detectors, item) \
    (&((namtDetectorPackedDataT(type,detectors)*)ptr)->item)
*/

#define namtDetectorPackedDataSize(data_size, detectors) \
    (sizeof(NAMTDataS) + detectors * data_size / 4)

#define namtDetectorPackedCDataSize(data_size, detectors) \
    (sizeof(NAMTDataS) + (detectors * detectors + 8) * data_size / 8)

#define namtDetectorPackedSpectrumSize(data_size, detectors) \
    (2 * detectors * 127 * data_size / 8)

#define namtDetectorPackedEventSize(data_size, detectors) \
    (8 + 2 * detectors)
    
#define namtDetectorPackedPlainDataSize(data_size, detectors) \
    (sizeof(NAMTDataS) + detectors * data_size / 8)

#define namtDetectorPackedPlainCDataSize(data_size, detectors, coincidences) \
    (sizeof(NAMTDataS) + coincidences * data_size / 8)

#define namtDetectorPackedPlainSpectrumSize(data_size, detectors, spectrums) \
    ((detectors + spectrums) * 127 * data_size / 8)

#define namtDetectorPackedPlainEventSize(data_size, detectors) \
    (8 + detectors)

#define namtData_up(ptr, type, size) \
    ((void*)(((char*)(ptr)) + sizeof(NAMTDataS)))

#define namtData_down(ptr, type, size) \
    ((void*)(((char*)(ptr)) + sizeof(NAMTDataS) + size*sizeof(type)))

#define namtCData_coincidences(ptr, type, size) \
    ((void*)(((char*)(ptr)) + sizeof(NAMTDataS)))

#define namtCData_many_many(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + size*size*sizeof(type)))

#define namtCData_many_zero(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 1)*sizeof(type)))

#define namtCData_zero_many(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 2)*sizeof(type)))

#define namtCData_zero_one(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 3)*sizeof(type)))

#define namtCData_one_zero(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 4)*sizeof(type)))

#define namtCData_many_one(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 5)*sizeof(type)))

#define namtCData_one_many(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 6)*sizeof(type)))

#define namtCData_total(ptr, type, size) \
    (*(type*)(((char*)(ptr)) + sizeof(NAMTDataS) + (size*size + 7)*sizeof(type)))

#define namtInternalFunc(ptr, type, detectors, func) \
    func(ptr, type, detectors)

#define namtDataSize(dev, i) \
    (dev->plain?namtDetectorPackedPlainDataSize(dev->data_size, dev->detectors[i]):namtDetectorPackedDataSize(dev->data_size, dev->detectors[i]))

#define namtCDataSize(dev, i) \
    (dev->plain?namtDetectorPackedPlainCDataSize(dev->data_size, dev->detectors[i], dev->coincidences):namtDetectorPackedCDataSize(dev->data_size, dev->detectors[i]))

#define namtSpectrumSize(dev, i) \
    (dev->plain?namtDetectorPackedPlainSpectrumSize(dev->data_size, dev->detectors[i], dev->spectrums):namtDetectorPackedSpectrumSize(dev->data_size, dev->detectors[i]))

#define namtEventSize(dev, i) \
    (dev->plain?namtDetectorPackedPlainEventSize(dev->data_size, dev->detectors[i]):namtDetectorPackedEventSize(dev->data_size, dev->detectors[i]))

#define namtData(ptr, data_size, detectors, item) \
    ((data_size == 16)?(namtInternalFunc(ptr, uint16_t, detectors, item)):(namtInternalFunc(ptr, uint32_t, detectors, item)))

#define namtDataElement(ptr, data_size, detectors, item, element) \
    ((data_size == 16)?((uint16_t*)(namtInternalFunc(ptr, uint16_t, detectors, item)))[element]:((uint32_t*)(namtInternalFunc(ptr, uint32_t, detectors, item)))[element])

#define namtBufferData(buf, i, item) \
    namtData(buf->data[i], buf->data_size, buf->detectors[i], namtData_##item)

#define namtBufferDataElement(buf, i, item, element) \
    namtDataElement(buf->data[i], buf->data_size, buf->detectors[i], namtData_##item, element)

#define namtBufferCData(buf, i, item) \
    namtData(buf->cdata[i], buf->data_size, buf->detectors[i], namtCData_##item)

#define namtBufferCDataElement(buf, i, item, element) \
    namtDataElement(buf->cdata[i], buf->data_size, buf->detectors[i], namtCData_##item, element)

#define NAMT_BUFFER_DATA_DECLARE \
    const char *namt_buffer_pos;

#define namtBufferDataInitData(buf, i) \
    namt_buffer_pos = ((char*)(buf->data[i])) + sizeof(NAMTDataS)

#define namtBufferDataInitCData(buf, i) \
    namt_buffer_pos = ((char*)(buf->cdata[i])) + sizeof(NAMTDataS)
    
#define namtBufferDataSkip(buf, elements) \
    namt_buffer_pos += elements * buf->data_size / 8

#define namtBufferSpectrumData(buf, i, sensor) \
    (buf->spectrum[i] + sensor * 127 * (buf->data_size / 8))

#define namtBufferMMEvent(buf, i, ev) \
    ((NAMTEvent)(((char*)(buf->events[i].mmevent))+ev*(buf->plain?namtDetectorPackedPlainEventSize(buf->data_size, buf->detectors[i]):namtDetectorPackedEventSize(buf->data_size, buf->detectors[i]))))

#ifdef _NAMT_READER_C
inline uint32_t namtBufferDataNext16(uint16_t **pos) {
    return *((*pos)++);
}

inline uint32_t namtBufferDataNext32(uint32_t **pos) {
    return *((*pos)++);
}
#else
inline uint32_t namtBufferDataNext16(uint16_t **pos);
inline uint32_t namtBufferDataNext32(uint32_t **pos);
#endif

#define namtBufferDataNext(buf) \
    ((buf->data_size == 16)?namtBufferDataNext16((uint16_t**)&namt_buffer_pos):namtBufferDataNext32((uint32_t**)&namt_buffer_pos))

#define NAMT_POINTER_DATA_DECLARE \
    const char *namt_pointer_pos;

#define namtPointerDataInit(ptr) \
    namt_pointer_pos = ((char*)(ptr)) + sizeof(NAMTDataS)

#define namtPointerDataSkip(data_size, elements) \
    namt_pointer_pos += elements * data_size / 8

#define namtPointerDataNext(data_size) \
    ((data_size == 16)?namtBufferDataNext16((uint16_t**)&namt_pointer_pos):namtBufferDataNext32((uint32_t**)&namt_pointer_pos))

/*
#define namtBufferDataNext(buf) \
    ((buf->data_size == 16)?*(((uint16_t*)namt_buffer_pos)++):*(((uint32_t*)namt_buffer_pos)++))
*/

/*
#define NAMTDetectorPackedCDataT(type, size) \
    struct  { \
	uint32_t iter; \
	uint16_t duration; \
	type coincidences[(size) * (size)]; \
	type many_many; \
	type many_zero; \
	type zero_many; \
	type zero_one; \
	type one_zero; \
	type many_one; \
	type one_many; \
	type total; \
    }  __attribute__ ((__packed__))

#define namtDetectorPackedCDataSize(data_size, ndetectors) \
    ((data_size == 16)?(sizeof(NAMTDetectorPackedCDataT(uint16_t, ndetectors))):(sizeof(NAMTDetectorPackedCDataT(uint32_t, ndetectors))))

#define namtCDataSize(dev, i) \
    namtDetectorPackedCDataSize(dev->data_size, dev->detectors[i])

#define namtCDataInternal(ptr, type, detectors, item) \
    ((NAMTDetectorPackedCDataT(type,detectors)*)ptr)->item

#define namtCData(ptr, data_size, detectors, item) \
    ((data_size == 16)?(namtCDataInternal(ptr, uint16_t, detectors, item)):(namtCDataInternal(ptr, uint32_t, detectors, item)))

#define namtBufferCData(buf, i, item) \
    namtCData(buf->cdata[i], buf->data_size, buf->detectors[i], item)
*/

#ifdef _WIN32
# pragma pack(1)
#endif /* _WIN32 */
struct VersionT {
    uint16_t version;
    uint8_t data_size;    
} attribute_packed;
typedef struct VersionT VersionS;
typedef struct VersionT *Version;

struct Version1T {
    uint16_t version;
    uint8_t data_size;    

    uint16_t clock;
    uint8_t r_divider, ext_divider;
    uint16_t detectors;
} attribute_packed;
typedef struct Version1T Version1S;
typedef struct Version1T *Version1;

#define NAMTDEVICE_TYPE_NAMT 		1
#define NAMTDEVICE_TYPE_SNT 		2
#define NAMTDEVICE_TYPE_ANALYZER	3

struct Device6T {
    uint16_t detectors;
} attribute_packed;
typedef struct Device6T *Device6;
typedef struct Device6T Device6S;

struct Version6T {
    uint16_t version;
    uint8_t data_size;    
    uint16_t clock;
    
    uint16_t device_type;
    uint16_t device_version;

    uint8_t devices;
    Device6S device[];
} attribute_packed;
typedef struct Version6T Version6S;
typedef struct Version6T *Version6;

struct Version7T {
    uint16_t version;
    uint8_t data_size;    
    uint16_t clock;
    
    uint16_t device_type;
    uint16_t device_version;
    uint32_t handled_events;

    uint8_t devices;
    Device6S device[];
} attribute_packed;
typedef struct Version7T Version7S;
typedef struct Version7T *Version7;

struct LayoutInfoT {
    uint8_t flags;
} attribute_packed;
typedef struct LayoutInfoT LayoutInfoS;
typedef struct LayoutInfoT *LayoutInfo;

struct LayoutInfo7T {
    uint8_t flags;
    
    uint16_t sensors;
    uint16_t rulesets;
    uint16_t spectrums;
    uint16_t rules;
} attribute_packed;
typedef struct LayoutInfo7T LayoutInfo7S;
typedef struct LayoutInfo7T *LayoutInfo7;


struct SyncAttr {
    uint16_t rtm;
    uint32_t iter;
} attribute_packed;

struct NAMTDataT {
    uint32_t iter;
    uint16_t duration;
} attribute_packed;
typedef struct NAMTDataT NAMTDataS;

struct SpectrumPropertiesT {
    uint8_t sensor;
    uint8_t flags;
    uint8_t cursor;
} attribute_packed;
typedef struct SpectrumPropertiesT *SpectrumProperties;
typedef struct SpectrumPropertiesT SpectrumPropertiesS;

struct NAMTEventT {
    uint32_t iter;
    uint32_t us;
    uint32_t d[2 * MAX_NAMT_DETECTORS];
} attribute_packed;
typedef struct NAMTEventT NAMTEventS;

struct NAMTRuleSetT {
    uint16_t flags;
    int16_t success;
} attribute_packed;
typedef struct NAMTRuleSetT NAMTRuleSetS;
typedef struct NAMTRuleSetT *NAMTRuleSet;

struct NAMTRuleT {
    int16_t target;
    int16_t threshold;
} attribute_packed;
typedef struct NAMTRuleT NAMTRuleS;
typedef struct NAMTRuleT *NAMTRule;

struct NAMTRuleSpecT {
    uint16_t ruleset;
    uint16_t sensor;
} attribute_packed;
typedef struct NAMTRuleSpecT NAMTRuleSpecS;
typedef struct NAMTRuleSpecT *NAMTRuleSpec;

#define SPECTRUM_LOGARITHM_FLAG 1

#ifdef _WIN32
# pragma pack()
#endif /* _WIN32 */



#include "namt.h"

int namtReader(NAMTDevice dev);
void namtZero(NAMTDevice dev);
void namtFree(NAMTDevice dev);

int namtAllocate(NAMTDevice dev, unsigned char devices);
int namtAllocateNextDevice(NAMTDevice dev, unsigned char detectors);

void namtReset(Device dev);
int namtDebug(NAMTDevice dev);
char* namtControl(NAMTDevice dev, const char *request);
int namtSetConfig(NAMTDevice dev, xmlDocPtr config);
int namtGetDriverInfo(NAMTDevice dev, xmlNodePtr node);

void namtFreeEvents(NAMTEvents event);

#endif /* _NAMT_READER_H */
