#include <stdio.h>

#include <dsdebug.h>
#include <dsassert.h>

#include "buffer.h"

#include "nanm.h"
#include "nanmbuffer.h"

NANMReaderBuf nanmBufferCreate(NANMDevice dev, unsigned char hosts, unsigned char *host) {
    int err;
    int i;
    
    DSTime tm;
    NANMReaderBuf rb;
    
    WriterDataID *id;
    uint32_t *value;

    ds_assert_null(dev);
    
    tm = dev->datatime;
    
    ds_assert_null(tm);
    
    rb = (NANMReaderBuf)malloc(sizeof(NANMReaderBufS));
    id = (WriterDataID*)malloc(hosts * sizeof(WriterDataID));
    value = (uint32_t*)malloc(hosts * sizeof(uint32_t));

    if ((!rb)||(!id)||(!value)) err = DS_ERR_MEM;
    else err = ReaderBufInit((ReaderBuf)rb, (Device)dev, tm);

    if (err) {
	if (rb) free(rb);
	if (id) free(id);
	if (value) free(value);
	return NULL;
    }
    
    rb->ids = hosts;

    for (i=0;i<hosts;i++) id[i] = host[i];
    rb->id = id;

    for (i=0;i<hosts;i++) value[i] = NANM_BAD_VALUE;
    rb->value = value;
    
    rb->buf.destroy = (ReaderBufDestroyFunction)nanmBufferDestroy;

    rb->trigger.triggers = 0;
    
    return rb;
    
}

int nanmBufferSet(NANMReaderBuf buffer, unsigned int rt_time) {
    ds_assert_param(buffer);
    
    buffer->rt_time = rt_time;
    return 0;
}


int nanmBufferAdd(NANMReaderBuf buffer, unsigned char id, uint16_t value) {
    ds_assert_param(buffer);
    ds_assert_param(id < buffer->ids);
    
    buffer->value[id] = value;
    return 0;
}


int nanmBufferRegister(NANMDevice dev, NANMReaderBuf rb) {
    ds_assert_param(dev);
    ds_assert_param(rb);

    BufferAdd(dev->device.buffer, (ReaderBuf)rb);

    dsDebugServer(DS_DEBUG_DEBUG, "Buffer: New NANM event");

    return 0;
}

void nanmBufferDestroy(NANMReaderBuf buffer) {
    unsigned int i;
    
    ds_assert_return(buffer);

    if (buffer->trigger.triggers) {
	if (buffer->trigger.trigger_time) {
	    free(buffer->trigger.trigger_time);
	}
	if (buffer->trigger.trigger_count) {
	    free(buffer->trigger.trigger_count);
	}
    }    
    free(buffer->id);
    free(buffer->value);
    ReaderBufFree((ReaderBuf)buffer);
    free(buffer);
}

int nanmBufferTriggerInit(nanmTriggerData trigger, unsigned int triggers) {
    ds_assert_param(trigger);

    if (triggers) {
	trigger->trigger_time = (uint16_t*)malloc(triggers*sizeof(trigger->trigger_time));
	trigger->trigger_count = (nanmTriggerCount*)malloc(triggers*sizeof(nanmTriggerCount));
	if ((!trigger->trigger_time)||(!trigger->trigger_count)) {
	    if (trigger->trigger_time) free(trigger->trigger_time);
	    if (trigger->trigger_count) free(trigger->trigger_count);
	    return DS_ERR_MEM;
	}
	trigger->triggers = triggers;
    }
    return 0;
}
