#include <stdio.h>

#include <dsdebug.h>
#include <dsassert.h>

#include "buffer.h"

#include "nmv.h"
#include "nmvbuffer.h"

int nmvBufferAdd(NMVDevice dev, const DSTime time, unsigned int channels, uint16_t *channel) {
    int err;
    uint16_t *mychannel;
    DSTime mytime;
        
    NMVReaderBuf rb;
    
    ds_assert_param(dev);
    ds_assert_param(channels);
    
    rb = (NMVReaderBuf)malloc(sizeof(NMVReaderBufS));
    mychannel = (uint16_t*)malloc(channels*sizeof(uint16_t));

    if ((!rb)||(!mychannel)||(!mytime)) err = DS_ERR_MEM;
    else err = ReaderBufInit((ReaderBuf)rb, (Device)dev, time);

    if (err) {
	if (rb) free(rb);
	if (mychannel) free(mychannel);
	return err;
    }
    
    
    rb->channels = channels;
    rb->channel = mychannel;
    memcpy(mychannel, channel, channels*sizeof(uint16_t));

    rb->buf.destroy = (ReaderBufDestroyFunction)nmvBufferDestroy;
    
    BufferAdd(dev->device.buffer, (ReaderBuf)rb);
    dsDebugServer(DS_DEBUG_DEBUG, "Buffer: New NMV event");
    
    return 0;
}


void nmvBufferDestroy(NMVReaderBuf buffer) {
    ds_assert_return(buffer);
    free(buffer->channel);
    ReaderBufFree((ReaderBuf)buffer);
    free(buffer);
}
