# include <unistd.h>
# include <sys/io.h>

static unsigned short int ppd=0x378;	// Paralell Port Data Register
static unsigned short int pps=0x379;	// Paralell Port Status Register
static unsigned short int ppc=0x37A;	// Paralell Port Command Register

void Pause(unsigned short int n)
{
    unsigned long PauseExit,ms;
    struct timeval tv;
    gettimeofday(&tv,NULL);
    PauseExit=tv.tv_usec+n*2;
    ms=tv.tv_sec;
    if (PauseExit>1000000) 
    {
	PauseExit-=1000000;
	ms++;
    }

    do {
	gettimeofday(&tv,NULL);
    } while ((tv.tv_sec<ms)||((tv.tv_usec<PauseExit)&&(tv.tv_sec==ms)));
}

//#define Pause(n) dsWaitMicrosecondsRT(2*n) DS: Not working for some reason

void InitialisePorts(unsigned short int portaddr)
{
    if (portaddr) {
	ppd = portaddr;
	pps = ppd + 1;
	ppc = ppd + 2;
    }

	// Getting permissions on required ports
    ioperm(ppd,3,1);
}

void WrTim(unsigned char tn, unsigned char sa, unsigned char data)
{
	  if (tn < 10) outb(0x80 + 4*tn + sa,ppd);
	  else outb(0x40 + 4*(tn - 10) + sa,ppd);
	  outb(2,ppc);   Pause(1);        /* PreWrAddress */
          outb(3,ppc);   Pause(1);        /* WrAddress Strobe */
          outb(2,ppc);   Pause(1);        /* Reset Strobe */
          outb(data,ppd);                 /* Data for Timer */
          outb(0,ppc);   Pause(1);        /* PreWrTimers */
          outb(1,ppc);   Pause(1);        /* WrTimers Strobe */
          outb(0,ppc);   Pause(1);        /* Reset Strobe */
          outb(0xC0,ppd);Pause(1);        /* End CS */
          outb(2,ppc);   Pause(1);        /* PreWrAddress */
          outb(3,ppc);   Pause(1);        /* WrAddress Strobe */
          outb(2,ppc);   Pause(1);        /* Reset Strobe */
}

unsigned char RdTim(unsigned char tn, unsigned char sa)
{
    unsigned char dl,dh;
    if (tn < 10) outb(0x80 + 4*tn + sa,ppd);
    else outb(0x40 + 4*(tn - 10) + sa,ppd);

	  outb(2,ppc);   Pause(1);        /* PreWrAddress */
          outb(3,ppc);   Pause(1);        /* WrAddress Strobe */
          outb(2,ppc);   Pause(1);        /* Reset Strobe */
          outb(0x0A,ppc);Pause(1);        /* PreRdTimersLo */
          outb(0x0B,ppc);Pause(1);        /* RdTimersLo Strobe */
          dl=inb(pps)>>3;		  /* Low half byte */
          outb(9,ppc);   Pause(1);        /* RdTimersHi Strobe */
          dh=inb(pps)<<1;                 /* High half Strobe */
          outb(8,ppc);			  /* Reset Strobe */
          outb(0xC0,ppd);Pause(1);        /* End CS */
          outb(2,ppc);   Pause(1);        /* PreWrAddress */
          outb(3,ppc);   Pause(1);        /* WrAddress Strobe */
          outb(2,ppc);   Pause(1);        /* Reset Strobe */
	  return ((dl&0x0F)|(dh&0xF0));
}

unsigned char GateReady()
{
    return (inb(pps)&0x80==0x80);
}

void Strobe()
{
          outb(4,ppc); Pause(1);
          outb(5,ppc); Pause(1);
          outb(4,ppc);
}


unsigned char Gateon()
{
    return ((inb(pps)&0x80)==0);
}

unsigned char Gateof()
{
    return ((inb(pps)&0x80));
}

void StartGate()
{
          outb(6,ppc); Pause(1);
          outb(7,ppc); Pause(1);
          outb(6,ppc);
	  while (Gateof());
}

void InitCounter(unsigned char cnt, unsigned char mode)
{
    WrTim(cnt/3, 3, 64*(cnt%3) + 0x30 + 2*mode);
}

void LoadCounter(unsigned short int cnt, unsigned short int data)
{
    WrTim(cnt/3, cnt%3, (unsigned char)(data&0x00FF));
    WrTim(cnt/3, cnt%3, (unsigned char)((data&0xFF00)>>8));
}

void ResetCounters(unsigned short int cn)
{
    unsigned short int i;
    for (i=1;i<=cn;i++) LoadCounter(i, 0xFFFF);
}

void Capture(unsigned char cnt)
{
    WrTim(cnt/3, 3, 64*(cnt%3));
}

unsigned short int DoubleCount(unsigned char cnt)
{
	  unsigned char alo,bhi;
	  Capture(cnt);
          alo = ~RdTim(cnt/3, cnt%3);
          bhi = ~RdTim(cnt/3, cnt%3);
	  return alo+(((unsigned short int)bhi)<<8);
}

int ProcTIMERS(int nch, unsigned short int *dbuf, DSTime nextread) {
    unsigned char NewGateFlag=0; 
    unsigned char OldGateFlag;
    unsigned int j;		// Counter
    char *dattim;		// Current time
    unsigned short int tmp;

    DSTime curtime;


    curtime = dsTimeCreate(NULL);
    
    if (!curtime) {
	if (curtime) dsTimeDestroy(curtime);
	return DS_ERR_MEM;
    }
        
    StartGate();
    NewGateFlag=0;

	// Waiting data ready
    do {
	OldGateFlag = NewGateFlag;
	NewGateFlag = Gateon();
	if ((!OldGateFlag)&&(NewGateFlag)) Strobe();

	dsTimeCurrentTime(curtime);
	if (dsTimeCmp(curtime, nextread)>0) {
	    dsTimeDestroy(curtime);
	    return DS_ERR_TIMEOUT;
	}
    } while ((!OldGateFlag)||(NewGateFlag));

    dsTimeDestroy(curtime);
    dsWaitRTUntil(nextread);
    
	// Reading data
    for (j=0;j<nch;j++)
	dbuf[j]=DoubleCount(j+1);

    ResetCounters(nch);
    
    return 0;
}
