#include <stdio.h>

#include <dsdebug.h>
#include <dsassert.h>

#include "buffer.h"

#include "sevan.h"
#include "sevanbuffer.h"

int sevanBufferAdd(SEVANDevice dev, const DSTime time, unsigned int channels, double *channel, unsigned int sensors, int rcd_flags, double *control_data) {
    int err;
    double *mychannel, *mycontrol;
    DSTime mytime;
        
    SEVANReaderBuf rb;
    
    ds_assert_param(dev);
    ds_assert_param(channels);
    
    rb = (SEVANReaderBuf)malloc(sizeof(SEVANReaderBufS));
    mychannel = (double*)malloc(channels*sizeof(double));
    if ((rcd_flags)&&(control_data)) {
	mycontrol = (double*)malloc((sensors+SEVAN_WEATHER_DATA_SIZE)*sizeof(double));
	if (!mycontrol) err = DS_ERR_MEM;
    } else mycontrol = NULL;

    if ((!rb)||(!mychannel)||(!mytime)) err = DS_ERR_MEM;
    else err = ReaderBufInit((ReaderBuf)rb, (Device)dev, time);

    if (err) {
	if (rb) free(rb);
	if (mychannel) free(mychannel);
	if (mycontrol) free(mycontrol);
	return err;
    }
    
    
    rb->channels = channels;
    rb->channel = mychannel;
    memcpy(mychannel, channel, channels*sizeof(double));
    rb->sensors = sensors;
    rb->control_data = mycontrol;
    if (mycontrol) memcpy(mycontrol, control_data, (sensors+SEVAN_WEATHER_DATA_SIZE)*sizeof(double));
    rb->rcd_flags = rcd_flags;

    rb->buf.destroy = (ReaderBufDestroyFunction)sevanBufferDestroy;
    
    BufferAdd(dev->device.buffer, (ReaderBuf)rb);
    dsDebugServer(DS_DEBUG_DEBUG, "Buffer: New SEVAN event");
    
    return 0;
}


void sevanBufferDestroy(SEVANReaderBuf buffer) {
    ds_assert_return(buffer);
    if (buffer->control_data) free(buffer->control_data);
    free(buffer->channel);
    ReaderBufFree((ReaderBuf)buffer);
    free(buffer);
}
