#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>

#define _DEVICE_H
#define _NEED_DEVICE_NAMES
#include <device.h>
#include <defaults.h>
#include <commands.h>

#include "display.h"

char instid[129];
char Server[129]="127.0.0.1";
unsigned int port = 13011;
unsigned char subdevice = 1;

#define ReAnnounceTimeout 60
#define Timeout 600

#define BufferSize 32768
#define TIMEOUT 50000000 /* ns */
#define VC_TIMEOUT 3000000 /* us */
#define CR_TIMEOUT 3000000 /* us */
#define SC_TIMEOUT 3000000 /* us */

#define RETRIES 2
#define DELAY 250000 /* us */

#define MAX_MULTI_ID 64

unsigned char ExitFlag = 1;

void Stop(int n) {
    ExitFlag = 0;
}

void Usage(char *name) {
    printf("Usage:\n");
    printf("  Shutdown       : %s -s    [server ip[:port]]\n",name);
    printf("  List           : %s -l    [server ip[:port]]\n",name);
    printf("  Hardware Info  : %s -hi   [server ip[:port]]/device\n",name);
    printf("  GetConfig      : %s -c    [server ip[:port]][/device]\n",name);
    printf("  Reset          : %s -r    [server ip[:port]][/device]\n",name);
    printf("  View Channels  : %s -vc   [server ip[:port]][/device[-subdev]]\n",name);
    printf("  Montior        : %s [-d#] [server ip[:port]][/device]\n", name);
    printf("  Control Request: %s -cr   [server ip[:port]]/device  request\n", name);
    printf("  Control Message: %s -crm  [server ip[:port]]/device  request\n", name);
    printf("  SetConfig      : %s -sc   [server ip[:port]]/device\n", name);
    printf("  ConfigMode     : %s -cm   [server ip[:port]]/device\n", name);
    printf("\n");
    printf("Options:\n");
    printf("  --timeout <ms> : ADAS Server response timeout (in milliseconds)\n");
    printf("  --retries <n>  : Number of retries\n");
    printf("  --unique-reqid : Generate unique ReqIDs for control requests\n");

    printf("\n"); 
/*    printf(" * Use subdev=0 to request the data channels not belonging to any subdevice\n");
    printf("\n");*/
}

void List() {
    
}

static unsigned char parsed = 0;
int Parse(char *app, char *param) {
    int size;
    char *ch, *pch;
    
    if (parsed) {
	printf("ERROR. ADAS device specified twice.\n\n");
	return 1;
    }
    
    parsed = 1;
    
    if (param) ch = strchr(param, '/');
    else ch = NULL;
    
    if (ch) {
	pch = strchr(ch + 1, '-');
	if (pch) {
	    subdevice = atoi(pch+1);
	    if (subdevice<1) {
		printf("ERROR. Invalid subdevice specified: %s.\n", pch+1);
		return 2;
	    }
	    size = pch - ch - 1;
	    if (size>128) size = 127;
	    strncpy(instid, ch + 1, size);
	} else {
	    strncpy(instid, ch + 1, 128);
	    instid[127] = 0;
	}
    } else {
	if (strstr(app, "namt")) strcpy(instid, "namt");
	else if (strstr(app, "nanm")) strcpy(instid, "nanm");
	else strcpy(instid, "");
    }
    
    if (!param) return 0;
    
    pch = strchr(param, ':');
    if (pch) {
	if (ch) {
	    size = ch - pch;
	    if (size > 128) {
		printf("ERROR. Invalid server string.\n\n");
		return 2;
	    }
	    strncpy(Server, pch + 1, size);
	    Server[size] = 0;
	    port = atoi(Server);
	} else port = atoi(pch + 1);
    }
    
    if ((ch)||(pch)) {
	size = pch?pch-param:ch-param;
	if (size) {
	    strncpy(Server,param,size);
	    Server[size] = 0;
	} else if (pch) {
	    printf("ERROR. Invalid server string.\n\n");
	    return 2;
	}
    } else {
	strncpy(Server,param,127);
	Server[127] = 0;
    }
    
    return 0;    
}

int print_value(unsigned char type, unsigned long duration, void *data) {
    double dbl_value;
    long int_value;
    
    switch (type) {
	case DATA_TYPE_UINT8:
	    if (duration) printf("%6u  ", *(uint8_t*)(data)*1000/duration);
	    else printf("%6u  ", *(uint8_t*)(data));
	    return sizeof(uint8_t);
	case DATA_TYPE_UINT16:
	    if (duration) printf("%6u  ", *(uint16_t*)(data)*1000/duration);
	    else printf("%6u  ", *(uint16_t*)(data));
	    return sizeof(uint16_t);
	case DATA_TYPE_UINT32:
	    if (duration) printf("%6u  ", *(uint32_t*)(data)*1000/duration);
	    else printf("%6u  ", *(uint32_t*)(data));
	    return sizeof(uint32_t);
	case DATA_TYPE_DOUBLE:
	    if (duration) printf("%6.2lf  ", *(double*)(data)*1000/duration);
	    else printf("%6.2lf  ", *(double*)(data));
	    return sizeof(double);
	case DATA_TYPE_MIXED:
	    dbl_value = *(double*)(data);
	    int_value = lround(dbl_value);
	    if (fabs(dbl_value-int_value)<1E-04) {
	        if (duration) printf("%6li  ", int_value*1000/duration);
		else printf("%6li  ", int_value);
	    } else {
	        if (duration) printf("%6.2lf  ", dbl_value*1000/duration);
		else printf("%6.2lf  ", dbl_value);
	    }
	    return sizeof(double);
    }
    return -1;
}

uint32_t generate_reqid() {
    struct timeval tv;
    
    gettimeofday(&tv, NULL);
    return (tv.tv_usec<<12)|((int)getpid())&0xFFFF;
}

#include "hash-string.h"
uint32_t hash_reqid(const char *str) {
    return hash_string(str);
}
    
main(int argc, char *argv[])
{
    int err = 0;
    int dsock;
    struct sockaddr_in servaddr, clientaddr;
    unsigned int clientaddrsize=sizeof(struct sockaddr_in);
    int idsize, propsize;
    unsigned char idmsg[131];
    unsigned char idcurmsg[129];
    unsigned char buf[BufferSize];
    time_t LastPacketTime,LastAnnounceTime,CurTime,CurReqTime = 0;
    unsigned char debug = 0, debug_level = 0;

    unsigned char list = 0;
    uint16_t device, version, prop_size;
    uint16_t device_type, device_version;
    unsigned int status;
    char device_name[257];
    unsigned long i, idev, pos, idpos;

    unsigned char reset = 0;
    unsigned char shutdown = 0;
    unsigned char view_channels = 0;
    unsigned char get_config = 0;
    unsigned char set_config = 0;
    unsigned char control_request = 0;
    unsigned char just_send = 0;
    unsigned char device_info = 0;
    unsigned char config_mode = 0;
    unsigned char unique_reqid = 0;
    const char *request;
    
    struct hostent *hent;
    struct timespec tm = {0, TIMEOUT};
    fd_set rd;

    unsigned long duration;
    unsigned long nd;
    unsigned char type;
    unsigned char perline;
    unsigned long timeout = 0;
    double quality;
    
    unsigned int retries = RETRIES;
    unsigned int retry;
    size_t readed;
    
    uint32_t reqid;
    uint32_t flags;
    uint32_t layout_sensors, layout_channels, layout_rules, layout_spectrums;
    
    char* multibuf[MAX_MULTI_ID];
    int multisize[MAX_MULTI_ID];
    
    int do_write;
    int msg_flags;
    int msg_id, max_id;
    
    unsigned short hw_type, hw_version;
    unsigned char printed;
    
    int devices;

    for (i=1;i<argc;i++) {
	if (!strncmp(argv[i],"--",2)) {
	    if (!strcmp(argv[i],"--timeout")) {
		if ((++i)<argc) timeout = atol(argv[i]);
		else {
		    printf("ERROR. Option --timeout requires attribute\n\n");
		    Usage(argv[0]);
		    exit(0);
		}
	    } else if (!strcmp(argv[i],"--retries")) {
		if ((++i)<argc) retries = atol(argv[i]);
		else {
		    printf("ERROR. Option --retries requires attribute\n\n");
		    Usage(argv[0]);
		    exit(0);
		}
	    } else if (!strcmp(argv[i],"--unique-reqid")) {
		unique_reqid = 1;
	    } else {
		if (strcmp(argv[i],"--help")) 
		    printf("ERROR. Unknown option: %s\n\n", argv[i]);
		Usage(argv[0]);
    		exit(0);
	    }
	} else if (!strcmp(argv[i], "-h")) {
	    Usage(argv[0]);
    	    exit(0);
	} else if (!strcmp(argv[i],"-l")) {
	    list = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strcmp(argv[i],"-hi")) {
	    device_info = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strcmp(argv[i],"-r")) {
	    reset = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strcmp(argv[i],"-s")) {
	    shutdown = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strncmp(argv[i],"-d", 2)) {
	    debug = 1;
	    if (argv[i][2]) debug_level = atoi(argv[i]+2);
	    else debug_level = 0;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strncmp(argv[i],"-vc", 3)) {
	    view_channels = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (!strncmp(argv[i],"-cm", 3)) {
	    if ((i+1)<argc) {
		config_mode = 1;
		err = Parse(argv[0], argv[++i]);
	    } else {
		err = 2;
		printf("ERROR. Device is not specified.\n\n");
	    }
	} else if (!strncmp(argv[i],"-cr", 3)) {
	    if ((i+2)<argc) {
	        control_request = 1;
		if (!strncmp(argv[i],"-crs", 4)) just_send = 1;
		err = Parse(argv[0], argv[(++i)]);
		request = argv[++i];
	    } else {
		err = 2;
		printf("ERROR. Control Request is missing.\n\n");
	    }
	} else if (!strncmp(argv[i],"-sc", 3)) {
	    if ((i+1)<argc) {
	        set_config = 1;
		err = Parse(argv[0], argv[++i]);
	    } else {
		err = 2;
		printf("ERROR. Device is not specified.\n\n");
	    }
	} else if (!strncmp(argv[i],"-c", 2)) {
	    get_config = 1;
	    if (((i+1)<argc)&&(argv[i+1][0]!='-')) err = Parse(argv[0], argv[++i]);
	    else err = Parse(argv[0], NULL);
	} else if (argv[i][0] != '-') {
	    err = Parse(argv[0], argv[i]);
	} else {
	    printf("ERROR. Unknown option: %s\n\n", argv[i]);
	    Usage(argv[0]);
    	    exit(0);
	}

	if (err) break;
    }

    if ((!parsed)&&(!err)) err = Parse(argv[0], NULL);
    
    if (err) {
	Usage(argv[0]);
	exit(0);
    }
    
    if (timeout) {
	tm.tv_sec = timeout / 1000;
	tm.tv_nsec = (timeout%1000)*1000000;
    } else {
	if (set_config) {
	    tm.tv_sec = SC_TIMEOUT / 1000000;
	    tm.tv_nsec = (SC_TIMEOUT % 1000000)*1000;
	} else if (control_request) {
	    tm.tv_sec = CR_TIMEOUT / 1000000;
	    tm.tv_nsec = (CR_TIMEOUT % 1000000)*1000;
	} else if (view_channels) {
	    tm.tv_sec = VC_TIMEOUT / 1000000;
	    tm.tv_nsec = (VC_TIMEOUT % 1000000)*1000;
	}
    }

/*    printf("%s.\n",Server);
    printf("%lu\n", port);
    printf("%s.\n",instid);*/
    hent = gethostbyname(Server);
    if (!hent) {
	fprintf(stderr, "Can't resolve server: %s!\n", Server);
	exit(1);
    }

    if (hent->h_length > sizeof(servaddr.sin_addr)) {
	fprintf(stderr, "Invalid address type: %s!\n", Server);
	exit(1);
    }

    dsock=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP);
    if (dsock<0)
    {
	fprintf(stderr,"Can't open socket!\n");
	exit(1);
    }
    
	// Configuring client port ( Parsing configuration )
    servaddr.sin_family=AF_INET;
    servaddr.sin_port=htons(0);
    servaddr.sin_addr.s_addr=htonl(INADDR_ANY);
    
    if (bind(dsock,(struct sockaddr*)&servaddr,sizeof(struct sockaddr_in)))
    {
	fprintf(stderr, "Can't bind socket\n");
	exit(1);
    }
    
	// Configuring server port ( Parsing configuration )
    servaddr.sin_family=AF_INET;
    servaddr.sin_port=htons(port);
//    servaddr.sin_addr.s_addr=inet_addr("127.0.0.1");
//    servaddr.sin_addr.s_addr=inet_addr(hent->h_name);
    memcpy(&servaddr.sin_addr, hent->h_addr, hent->h_length);

    if (list) {
	idmsg[0] = ADAS_COMMAND_GET_INFO;

	retry = retries;
send_again:
	sendto(dsock,&idmsg,1,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
	
	FD_ZERO(&rd);

read_again:
	FD_SET(dsock,&rd);
	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	if (err<1) {
	    if ((--retry)>0) {
		usleep(DELAY);
		goto send_again;
	    }
	    printf("Error: timeout connecting ADAS server\n");
	    exit(0);
	}

	err = recvfrom(dsock,(void*)buf,BufferSize,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	if ((err<1)||(buf[0] != ADAS_COMMAND_GET_INFO)) goto read_again;
	printf("ADAS Server: %s\nVersion: %u.%u.%u, Status: %u\nServer Time: %s\n\nNumber of working devices: %u\n", Server, buf[2], buf[3], buf[4], buf[5], buf+6, buf[1]);
	for (idev = 0, pos = 6 + strlen(buf+6)+1; idev < buf[1]; idev++) {
	    if (pos + 2 > err) break;
	    
	    idsize = buf[pos]; pos++;
	    propsize = buf[pos]; pos++;
	    if (pos + idsize + propsize + sizeof(uint32_t) + 2 * sizeof(uint16_t) > err) break;
	    
	    device = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    version = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    status = ntohl(*(uint32_t*)(buf+pos)); pos += sizeof(uint32_t);

	    if (idsize) memcpy(device_name, buf + pos, idsize);
	    device_name[idsize] = 0; pos += idsize;
	    
	    if ((device == DEVICE_TYPE_NAMT)&&(version>2)) {
		if (ntohs(*(unsigned short*)(buf+pos)) == 3) {
		    printf(" %s ", "SA");
		    printed = 1;
		} else printed = 0;
	    } else printed = 0;
	    
	    if (!printed) {
		for (i=0;device_configuration_name[i].id;i++) {
		    if (device_configuration_name[i].type == device) {
			printf(" %s ", device_configuration_name[i].id);
			break;
		    }
		}
		if (!device_configuration_name[i].id)
		    printf(" Unknown ");
	    }

	    printf("Device (ID: %u, Version: %u). %s (Status: %u, Mode: %u). ", device, version, device_name, status&0xFFFF, status>>16);
	    switch (device) {
		case DEVICE_TYPE_NAMT:
		    if (((version == 1)||(version == 2))&&(propsize)) {
			printf("SubDevices: %hhu(", buf[pos]);
			if (buf[pos] + 1 == propsize) {
			    for (i = 0; i<buf[pos]; i++) {
				if (i) printf(", ");
				printf("%hhu", buf[pos + i + 1]);
			    }
			}
			printf(")");
		    } else if ((version == 3)&&(propsize>4)) {
			hw_type = ntohs(*(unsigned short*)(buf+pos));
			hw_version = ntohs(*(unsigned short*)(buf+pos+2));
			
			if (hw_type == 3) {
			    if (propsize == 7) printf("Sensors: %hu", ntohs(*(unsigned short*)(buf + pos + 5)));
			    else printf("Sensors: 0");
			} else {
			    printf("SubDevices: %hhu(", buf[pos+4]);
			    if (2*buf[pos+4] + 5 == propsize) {
				for (i = 0; i<buf[pos+4]; i++) {
				    if (i) printf(", ");
				    printf("%hu", ntohs(*(unsigned short*)(buf + pos + 2*i + 5)));
				}
			    }
			    printf(")");
			}
		    } 
		    pos += propsize;
		break;
		case DEVICE_TYPE_NANM:
		    if ((version == 1)&&(propsize))
			printf("Hosts: %u.", buf[pos]);
		    pos += propsize;
		break;
		case DEVICE_TYPE_ASCII:
		case DEVICE_TYPE_NMV:
		    if ((version == 1)&&(propsize))
			printf("Channels: %u.", *(uint16_t*)(buf+pos));
		    pos += propsize;
		break;
		case DEVICE_TYPE_SEVAN:
		    if ((version == 1)&&(propsize)) {
			printf("Sensors: %u, ", *(uint16_t*)(buf+pos));
			printf("Channels: %u.", *(uint16_t*)(buf+pos+sizeof(uint16_t)));
		    }
		    pos += propsize;
		break;
		default:
		    pos += propsize;
		
	    }
	    
	    printf("\n");
	}
	return 0;
    }

    if (shutdown) {
	idmsg[0] = ADAS_COMMAND_SHUTDOWN;
	sendto(dsock,&idmsg,1,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
	exit(0);
    }
    
    idsize = strlen(instid) + 1;
    idmsg[0]=ADAS_COMMAND_SUBSCRIBE_BASE;	// New client arrived message
    strcpy((char*)idmsg + 1, instid);
    idcurmsg[0]=ADAS_COMMAND_DEBUG_BASE;	// Get Current
    strcpy((char*)idcurmsg + 1, instid);

    if (device_info) {
	idmsg[0] = ADAS_COMMAND_GET_MCU_INFO;

	retry = retries;
device_info_send_again:
	sendto(dsock,&idmsg,idsize,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
	
	FD_ZERO(&rd);

device_info_read_again:
	FD_SET(dsock,&rd);
	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	if (err<1) {
	    if ((--retry)>0) {
		usleep(DELAY);
		goto device_info_send_again;
	    }
	    printf("Error: timeout connecting ADAS server\n");
	    exit(0);
	}

	err = recvfrom(dsock,(void*)buf,BufferSize,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	if ((err<1)||(buf[0] != ADAS_COMMAND_GET_MCU_INFO)) goto device_info_read_again;

	pos = 1;
	device = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	version = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	status = ntohl(*(uint32_t*)(buf+pos)); pos += sizeof(uint32_t);
	device_type = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	device_version = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	devices = *(uint8_t*)(buf+pos); pos += sizeof(uint8_t);

	if ((device == DEVICE_TYPE_NAMT)&&(device_type == 3)) {
	    printf("SA ");
	} else {
	    for (i=0;device_configuration_name[i].id;i++) {
		if (device_configuration_name[i].type == device) {
		    printf("%s ", device_configuration_name[i].id);
		    break;
		}
	    }
	    if (!device_configuration_name[i].id) printf("Unknown ");
	}

	switch (device) {
	    case DEVICE_TYPE_NAMT:
		if (device_type == 1) {
		    printf("Device %u. Type: NAMT, Version: %u. Status: %u, Mode: %u, SubDevices: %u.\n", version, device_version, status&0xFFFF, status>>16, devices);
		} else if (device_type == 2) {
		    printf("Device %u. Type: SNT, Version: %u. Status: %u, Mode: %u, SubDevices: %u.\n", version, device_version, status&0xFFFF, status>>16, devices);
		} else if (device_type == 3) {
		    printf("Device %u. Type: SA, Version: %u. Status: %u, Mode: %u, SubDevices: %u.\n", version, device_version, status&0xFFFF, status>>16, devices);
		} else 
		    printf("Device %u. Type: %u, Version: %u. Status: %u, Mode: %u, SubDevices: %u.\n", version, device_type, device_version, status&0xFFFF, status>>16, devices);
	    break;
	    default:
		printf("Device %u. Type: %u, Version: %u. Status: %u, Mode: %u, SubDevices: %u.\n", version, device_type, device_version, status&0xFFFF, status>>16, devices);
	}

	for (i=0;i<devices;i++) {
	    layout_sensors = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    layout_channels = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    layout_spectrums = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    
	    printf(" SubDevice %u. Sensors: %u, Coincidences: %u, Spectrums: %u.\n", i+1, layout_sensors, layout_channels, layout_spectrums);
	}

	flags = ntohl(*(uint32_t*)(buf+pos)); pos += sizeof(uint32_t);
	
	if (flags&ADAS_HARDWARE_FLAG_CONFIGURABLE_LAYOUT) {
	    layout_sensors = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    layout_channels = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    layout_rules = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    layout_spectrums = ntohs(*(uint16_t*)(buf+pos)); pos += sizeof(uint16_t);
	    
	    printf("Configurable Layout.");
	    printf(" Sensors: %lu,", layout_sensors);
	    printf(" Channels: %lu, Rules: %lu,", layout_channels, layout_rules);
	    printf(" Spectrums: %lu\n", layout_spectrums);
	}
	
	return 0;
    }

    if (reset) {
	printf("Reseting: %s\n", instid);
	idmsg[0] = ADAS_COMMAND_RESET;
	err = sendto(dsock,idmsg,idsize,0,(struct sockaddr*)&servaddr,sizeof(struct sockaddr_in));
    
	exit(0);
    }

    if (config_mode) {
	printf("Setting Configuration Mode: %s\n", instid);
	idmsg[0] = ADAS_COMMAND_CONFIGURATION_MODE;
	err = sendto(dsock,idmsg,idsize,0,(struct sockaddr*)&servaddr,sizeof(struct sockaddr_in));
    
	exit(0);
    }

    if (get_config) {
	idmsg[0] = ADAS_COMMAND_GET_CONFIG;
	    /* If the device unspecified (directly or by the application name),
	    the idmsg="" */

	retry = retries;
cfg_again:
	sendto(dsock,idmsg,idsize,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
	
	FD_ZERO(&rd);

cfg_read_again:
	FD_SET(dsock,&rd);
	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	if (err<1) {
	    if ((--retry)>0) {
		usleep(DELAY);
		goto cfg_again;
	    }
	    printf("Error: timeout connecting ADAS server\n");
	    return 0;
	}

	err = recvfrom(dsock,(void*)buf,BufferSize,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	if ((err<1)||(buf[0] != ADAS_COMMAND_GET_CONFIG)) goto cfg_read_again;
	
	fwrite(buf+1,1,err-1,stdout);
	
	return 0;
    }
    
    if (set_config) {
	idmsg[0] = ADAS_COMMAND_CONFIGURE;
	memcpy(buf, idmsg, idsize); buf[idsize++] = 0;
	while (!feof(stdin)) {
	    readed = fread(buf+idsize, 1, BufferSize-idsize, stdin);
	    if (readed>0) idsize+=readed;
	}
	buf[idsize++] = 0;
	
	sendto(dsock,buf,idsize,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
	
	FD_ZERO(&rd);
	

cfg_set_read_again:
	FD_SET(dsock,&rd);
	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	if (err<1) {
	    printf("Error: timeout connecting ADAS server\n");
	    return 1;
	}

	err = recvfrom(dsock,(void*)idcurmsg,130,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	if ((err<1)||(idcurmsg[0] != ADAS_COMMAND_CONFIGURE)) goto cfg_set_read_again;
	
	if (err!=1+sizeof(int)) {
	    printf("Error: invalid answer from ADAS server\n");
	    return 1;
	}
	
	err = *((int*)(idcurmsg+1));
	if (err) {
	    printf("Error: error setting configuration %lx\n", err);
	    return err;
	} else {
	    printf("Done\n");
	}

	return 0;
    }
    
    if (control_request) {
	idmsg[0] = ADAS_COMMAND_CONTROL;

	if (unique_reqid) reqid = generate_reqid();
	else reqid = hash_reqid(request);
	
	idmsg[idsize] = 0;
	*(uint32_t*)(idmsg + idsize + 1) = reqid;
	strncpy(idmsg + idsize + 5, request, 131 - 6 - idsize);

	retry = retries;

control_request_send:
	sendto(dsock, idmsg, idsize + strlen(idmsg+idsize+5) + 6,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));

	if (just_send) {
	    printf("Control message is sent");
	    return 0;
	}
	
	FD_ZERO(&rd);

	for (i=0;i<MAX_MULTI_ID;i++) multibuf[i] = NULL;
	
	msg_flags = 1; max_id = 0; do_write = 1;
	for (i=0;msg_flags&1;i++) {
control_request_again:
	    FD_SET(dsock,&rd);
	    
	    err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	    if (err<1) {
		if ((--retry)>0) {
		    usleep(DELAY);

		    for (i=0;i<max_id;i++) {
			if (multibuf[i]) {
			    free(multibuf[i]);
			    multibuf[i] = NULL;
			}
		    }

		    sendto(dsock, idmsg, idsize + strlen(idmsg+idsize+1) + 2,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
		    FD_ZERO(&rd);
		    
		    i = 0; max_id = 0;
		    goto control_request_again;
		}

		do_write = 0;
		printf("Error: timeout connecting ADAS server\n");
		break;
	    }
	
	    err = recvfrom(dsock,(void*)buf,BufferSize,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	    if ((err<8)||(buf[0] != ADAS_COMMAND_CONTROL)||(*(uint32_t*)(buf+4) != reqid)) goto control_request_again;
	    
	    msg_flags = buf[1];
	    msg_id = *(uint16_t*)(buf+2);
	    
//	    printf("%u %u %u\n", msg_flags, msg_id, err);
	    
	    if ((msg_flags&1)==0) {
		if (!i) {
		    if (((err-1)>18)&&(!strncmp(buf+8,"No response", 11))) {
			if ((--retry)>0) {
			    usleep(DELAY);
			    goto control_request_send;
			}
		    }
		    if (buf[err-1]) fwrite(buf+8,1,err-8,stdout);
		    else  fwrite(buf+8,1,err-9,stdout);
		    fprintf(stdout,"\n\n");
		    do_write = 0;
		    break;
		}
		err--; /* Skipping last zero */
	    }

	    if (msg_id > MAX_MULTI_ID) {
		fprintf(stdout, "CLIENT ERROR. Response too big!\n\n");
		do_write = 0;
		break;
	    }

	    if (multibuf[msg_id]) {
		fprintf(stdout, "CLIENT ERROR. Duplicate frame received!\n\n");
		do_write = 0;
		break;
	    }

	    multibuf[msg_id] = malloc(err-8);
	    if (!multibuf[msg_id]) {
		fprintf(stdout, "CLIENT ERROR. Memory allocation error!\n\n");
		do_write = 0;
		break;
	    }

	    memcpy(multibuf[msg_id], buf+8, err-8);
	    multisize[msg_id] = err-8;

	    if ((msg_id + 1) > max_id) max_id = msg_id + 1;
	}
	
	for (i=0;i<max_id;i++)  {
	    if (!multibuf[i]) {
		fprintf(stdout, "CLIENT ERROR. Not all frames received!\n\n");
		do_write = 0;
		break;
	    }
	}

	for (i=0;i<max_id;i++) {
	    if (do_write) {
		if (((i+1)==max_id)&&(!multibuf[i][multisize[i]-1]))
		    fwrite(multibuf[i],1,multisize[i]-1,stdout);
		else 
		    fwrite(multibuf[i],1,multisize[i],stdout);
	    }
	    if (multibuf[i]) free(multibuf[i]);
	}
	
	return 0;
    }


    if (view_channels) {
	err=sendto(dsock,idcurmsg,idsize,0,(struct sockaddr*)&servaddr,sizeof(struct sockaddr_in));

	FD_ZERO(&rd);
	
vc_read_again:

	FD_SET(dsock,&rd);

	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);
	if (err<1) {
	    printf("Error: timeout connecting ADAS server\n");
	    return 0;
	}

	err = recvfrom(dsock,(void*)buf,BufferSize,0,(struct sockaddr*)&clientaddr,&clientaddrsize);
	if ((err<6)||(buf[0] != ADAS_COMMAND_DEBUG_BASE+subdevice)) goto vc_read_again;
	
	pos = 1;
	device = *(uint16_t*)(buf+pos);	pos += sizeof(uint16_t);
	version = *(uint16_t*)(buf+pos);pos += sizeof(uint16_t);

	idpos = pos;
	if ((instid[0])&&(strcmp(instid, buf+idpos))) goto vc_read_again;

	while (buf[pos]) pos++;
	pos++;
	
	pos+=sizeof(time_t);
	duration=*(uint32_t*)(buf+pos);		pos+=sizeof(uint32_t);
	nd=*(uint32_t*)(buf+pos); 		pos+=sizeof(uint32_t);
	type=*(uint8_t*)(buf+pos);		pos+=sizeof(uint8_t);
	quality=*(double*)(buf+pos);		pos+=sizeof(double);
	prop_size = *(uint16_t*)(buf+pos);	pos+=sizeof(uint16_t)+prop_size;
	
	if (device == DEVICE_TYPE_NAMT) perline = nd/2;
	else {
	    switch (type) {
		case DATA_TYPE_UINT8:
		case DATA_TYPE_UINT16:
		case DATA_TYPE_UINT32:
		case DATA_TYPE_UINT64:
		    perline = 8;
		break;
	        case DATA_TYPE_DOUBLE:
		case DATA_TYPE_MIXED:
		    perline = 8;
		break;
		default: 
		    perline = 8;
	    }
	}
	
	    
	for (i=0;i<nd;i++,pos+=err) {
	    if (i%perline==0) printf("\n  ");
	    err = print_value(type, duration, buf+pos);
	    if (err<0) {
		printf("Error: Unknown data type\n");
		return 0;
	    }
	}
	printf("\n");
	
	return 0;
    }    


    err=sendto(dsock,idmsg,idsize,0,(struct sockaddr*)&servaddr,
		sizeof(struct sockaddr_in));
    
    LastPacketTime=time(NULL);
    LastAnnounceTime=LastPacketTime;


    FD_ZERO(&rd);
    signal(SIGINT,Stop);

    InitDisplay();
    while (ExitFlag) {

	tm.tv_nsec = TIMEOUT;
	FD_SET(dsock,&rd);
	err = pselect(dsock+1,&rd,NULL,NULL,&tm,NULL);

	if (err<=0) {
	    CurTime=time(NULL);
	    if ((CurTime-LastPacketTime>Timeout)||
		(CurTime-LastAnnounceTime>ReAnnounceTimeout))
	    {
		err=sendto(dsock,idmsg,idsize,0,(struct sockaddr*)&servaddr,
		    sizeof(struct sockaddr_in));
		LastAnnounceTime = CurTime;
		LastPacketTime = CurTime;
	    }
	    
	    if ((debug)&&(CurTime - CurReqTime > 0)) {
	        CurReqTime = CurTime;
		err=sendto(dsock,idcurmsg,idsize,0,(struct sockaddr*)&servaddr,
		    sizeof(struct sockaddr_in));
	    }

	    Clear();
	    continue;
	}

	err = recvfrom(dsock,(void*)buf,BufferSize,0,
    	    (struct sockaddr*)&clientaddr,&clientaddrsize);
	
	if (err>5) {
	    if ((buf[0]>=ADAS_COMMAND_SUBSCRIBE_BASE)&&(buf[0]<(ADAS_COMMAND_SUBSCRIBE_BASE+0x10))) {
		pos = 1;
		device = *(uint16_t*)(buf+pos);	pos += sizeof(uint16_t);	// Second byte is HW device
		version = *(uint16_t*)(buf+pos);pos += sizeof(uint16_t);	// Second byte is HW device

		if ((!instid[0])||(!strcmp(instid, buf+pos))) {
		
		    if ((device&0xFF) == DEVICE_TYPE_NAMT)
			status = DisplayNAMT(buf[0] - ADAS_COMMAND_SUBSCRIBE_BASE, device>>8, (char*)buf+pos+strlen(buf+pos)+1);
		    else 
			status = DisplayRaw(0, buf[0] - ADAS_COMMAND_SUBSCRIBE_BASE, (char*)buf+1);

		    if (!status) LastPacketTime = time(NULL);
		}
	    }
	    else if ((debug)&&(buf[0]>=ADAS_COMMAND_DEBUG_BASE)&&(buf[0]<(ADAS_COMMAND_DEBUG_BASE+0x10))) {
		pos = 1;
		device = *(uint16_t*)(buf+pos);	pos += sizeof(uint16_t);
		version = *(uint16_t*)(buf+pos);pos += sizeof(uint16_t);
		if ((!instid[0])||(!strcmp(instid, buf+pos))) {
		    if ((device&0xFF) == DEVICE_TYPE_NAMT)
			status = DisplayNAMTCurrent(buf[0]-ADAS_COMMAND_DEBUG_BASE,device>>8,(char*)buf+1);
		    else 
			status = DisplayRaw(1, buf[0]-ADAS_COMMAND_DEBUG_BASE, (char*)buf+1);
	
		    if ((!status)&&(buf[0]==ADAS_COMMAND_DEBUG_BASE+subdevice)&&(debug_level)) {
	    		CurReqTime = CurTime;
			err=sendto(dsock,idcurmsg,idsize,0,(struct sockaddr*)&servaddr, sizeof(struct sockaddr_in));
		    }
		}
	    }
	}

    }
    CloseDisplay();
}
