#include <string.h>
#include <time.h>
#include <curses.h>
#include <math.h>
#include <netinet/in.h>

#include "tools.h"
#include "display.h"

extern unsigned char subdevice;

time_t rawtime = 0, time1 = 0, time2 = 0;
static WINDOW *win1,*rwin1,*twin1,*win2,*rwin2,*twin2,*win3,*rwin3, *rwin3_1, *rwin3_2;
static WINDOW *rawwin, *win12;

void InitDisplay() {
    initscr();
    cbreak();
    noecho();
    curs_set(0);
    
    win1 = newwin(7,52,1,0);
    rwin1 = newwin(6,28,1,52);
    twin1 = newwin(1,80,0,0);

    win2 = newwin(8,52,9,0);
    rwin2 = newwin(6,28,9,52);
    twin2 = newwin(1,80,8,0);
    
    win12 = newwin(15,52,1,0);

    win3 = newwin(8,52,16,0);
    rwin3 = newwin(9,13,16,52);
    rwin3_1 = newwin(9,6,16,66);
    rwin3_2 = newwin(9,6,16,74);

    rawwin = newwin(23,80,1,0);
    
    refresh();
}

void CloseDisplay() {
    endwin();
}

void Clear() {
    if ((time1)&&(time(NULL)>time1)) {
	wclear(rwin1);
	wmove(rwin1,3,15);
	wprintw(rwin1, "TIMEOUT");
	wrefresh(rwin1);
	time1 = 0;
    }
    if ((time2)&&(time(NULL)>time2)) {
	wclear(rwin2);
	wmove(rwin2,3,15);
	wprintw(rwin2, "TIMEOUT");
	wrefresh(rwin2);
	time2 = 0;
    }
    
    if ((rawtime)&&(time(NULL)>rawtime)) {
	wmove(twin1,0,52);
	wprintw(twin1, "TIMEOUT");
	wrefresh(twin1);
    }
}	

/* Displaying events per second */
int DisplayNAMT(unsigned char id, unsigned char hwdevice, char *buf) {
	double dtmp, maxdisp;
	struct tm date;
	time_t itm;
	char ct[32],fn[32];
	unsigned long ti;
	unsigned long Dti;
	unsigned long nd;

	unsigned int i, pos, spos;
	int perline;
	
	WINDOW *win,*rwin,*twin;
	char dispfmt[12];
	char *title;
	
	switch(id) {
	    case 0x1:
		win = win1;
		rwin = rwin1;
		twin = twin1;
		title = "Detector1";
		if (!time1) wclear(rwin);
		time1 = time(NULL);
    	    break;
	    case 0x2:
		win = win2;
		rwin = rwin2;
		twin = twin2;
		title = "Detctor2";
		if (!time2) wclear(rwin);
		time2 = time(NULL);
	    break;
	    default:
		return 1;
	}
	
	itm=*(time_t*)buf;
	pos=sizeof(time_t); 	ti=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t); 	Dti=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);	nd=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);
	
	if (nd > 12) {
	    win = win12;
	    if (nd > 24) perline = nd / 5;
	    else perline = nd / 4;
	} else perline = nd / 2;
	
	switch(id) {
	    case 0x1:
		time1 += 2 + ti / 500;
    	    break;
	    case 0x2:
		time2 += 2 + ti / 500;
    	    break;
	}
	
	gmtime_r(&itm,&date);
	sprintf(ct,"%02u:%02u:%02u",date.tm_hour,date.tm_min,date.tm_sec);

	wmove(twin,0,0);
	wprintw(twin,"%s. %s (Data: %'9ums, Disp: %'9ums",title,ct,ti,Dti);

	wmove(win,0,0);
	for (i=0;i<nd;i++,pos+=sizeof(uint32_t)) {
		if (i%perline==0) {
			wprintw(win,"\n  ");
		}
		if ((i%perline)>5) continue;
		wprintw(win, "%6u  ", lrint(*(uint32_t*)(buf+pos)*1000./ti));
	} 	
	wprintw(win,"\n");

	maxdisp = 0;
	for (i=0,spos=pos;i<nd;i++,spos+=sizeof(double)) {
		dtmp = *(double*)(buf+spos);
		if (dtmp > maxdisp) maxdisp = dtmp;
	}
	if (maxdisp<1) strcpy(dispfmt, "% 6.3lf  ");
	else if (maxdisp<10) strcpy(dispfmt, "% 6.2lf  ");
	else if (maxdisp<100) strcpy(dispfmt, "% 6.1lf  ");
	else strcpy(dispfmt, "% 6.0lf  ");

	for (i=0;i<nd;i++,pos+=sizeof(double)) {
		if (i%perline==0) {
			wprintw(win,"\n  ");
		}
		wprintw(win,dispfmt, *(double*)(buf+pos));
	}

	if (hwdevice < 3) {
	    wmove(rwin,0,0);
	    wprintw(rwin, "Total       : %6u\n", lrint(*(uint32_t*)(buf+pos)*1000./ti)); pos+=sizeof(uint32_t);
	    wprintw(rwin, "Coincidences: %6u\n", lrint(*(uint32_t*)(buf+pos)*1000./ti)); pos+=sizeof(uint32_t);
	    wprintw(rwin, "Many - Many : %6u\n", lrint(*(uint32_t*)(buf+pos)*1000./ti)); pos+=sizeof(uint32_t);
	    wprintw(rwin, "Many - Zero : %6u, %6u", lrint(*(uint32_t*)(buf+pos)*1000./ti),lrint(*(uint32_t*)(buf+pos+sizeof(uint32_t))*1000./ti)); pos+=2*sizeof(uint32_t);
	    wprintw(rwin, "Zero - One  : %6u, %6u", lrint(*(uint32_t*)(buf+pos)*1000./ti),lrint(*(uint32_t*)(buf+pos+sizeof(uint32_t))*1000./ti)); pos+=2*sizeof(uint32_t);
	    wprintw(rwin, "Many - One  : %6u, %6u", lrint(*(uint32_t*)(buf+pos)*1000./ti),lrint(*(uint32_t*)(buf+pos+sizeof(uint32_t))*1000./ti)); pos+=2*sizeof(uint32_t);
	}
	
	wprintw(twin, ", Confidence: % 3.2lf%)", *(double*)(buf+pos)*100);
	
	wrefresh(twin);
	wrefresh(win);
	wrefresh(rwin);	
	
	return 0;
}

int DisplayNAMTCurrent(unsigned char id, unsigned char hwdevice, char *buf) {
	char ct[32],fn[32];
	unsigned long duration;
	unsigned long cduration;
	unsigned long nd;
	
	unsigned int i, pos;
	
	WINDOW *win,*rwin;
	unsigned int y;
	char *title;

	time_t itm;
	DeviceType device;
	uint16_t version, prop_size;
	DataType type;
	unsigned char datasize;
	unsigned char have_float;
	double quality;
	
	int perline;

	pos=0; 				device=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t);		version=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t);		// id

	wmove(win3, 0, 0);	
	wprintw(win3, "Current Data. Duration:");
	wmove(win3, 0, 33);
	wprintw(win3, ",");

	switch(id) {
		case 1:
		    win = win3;
		    rwin = rwin3_1;
		    title = "Detector1";
		    wmove(win, 0, 25);
		    y = 1;
    		break;
		case 2:
		    win = win3;
		    rwin = rwin3_2;
		    title = "Detctor2";
		    wmove(win, 0, 37);
		    y = 4;
		break;
		default:
		    return 1;
	}

	while (buf[pos]) pos++; pos++;	itm=*(time_t*)buf;
	pos+=sizeof(time_t);  		duration=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);		nd=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);  	type=*(uint8_t*)(buf+pos);
	pos+=sizeof(uint8_t);		quality=*(double*)(buf+pos);
	pos+=sizeof(double);		prop_size=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t)+prop_size;


	perline = nd/2;

	wprintw(win, "%6lums", duration);
	wmove(win, y, 0);

	for (i=0;i<nd;i++,pos+=sizeof(uint32_t)) {
		if (i%perline==0) wprintw(win,"\n  ");
		if ((i%perline)>5) continue;
		wprintw(win, "%6u  ", *(uint32_t*)(buf+pos)*1000/duration);
	} 	
	wprintw(win,"\n");
	wrefresh(win3);

	if (hwdevice < 3) {
	    wmove(rwin3,0,0);
	    wprintw(rwin3, "Total       :");
	    wprintw(rwin3, "Coincidences:");
	    wprintw(rwin3, "Many - Many :");
	    wprintw(rwin3, "Many - Zero :");
	    wprintw(rwin3, "Zero - Many :");
	    wprintw(rwin3, "Zero - One  :");
	    wprintw(rwin3, "One  - Zero :");
	    wprintw(rwin3, "Many - One  :");
	    wprintw(rwin3, "One  - Many :");
	    wrefresh(rwin3);	
	}

	wmove(rwin,0,0);

	cduration=*(uint32_t*)(buf+pos); 	pos+=sizeof(uint32_t);
	nd=*(uint32_t*)(buf+pos);		pos+=sizeof(uint32_t);	
	type=*(uint8_t*)(buf+pos);		pos+=sizeof(uint8_t);	

	if (hwdevice < 3) {
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
    	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	    wprintw(rwin, "%6u", *(uint32_t*)(buf+pos)*1000/cduration); pos+=sizeof(uint32_t);
	}
	
	wrefresh(win);
	wrefresh(rwin);	
	
	return 0;
}


int DisplayRaw(unsigned char debug, unsigned char id, char *buf) {
	WINDOW *win;
	unsigned long perrow;
	struct tm date;
	time_t itm;
	char ct[32],fn[32];
	uint32_t ti;
	uint32_t nd;

	DeviceType device;
	uint16_t version, prop_size;
	DataType type;
	unsigned char datasize;
	double quality;

	size_t pos;
	
	char dispfmt[16], dispffmt[16];
	
	
	if (subdevice != id) return 1;	
	
	pos=0; 				device=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t);		version=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t);		// id
	while (buf[pos]) pos++; pos++;	itm=*(time_t*)buf;
	pos+=sizeof(time_t);  		ti=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);		nd=*(uint32_t*)(buf+pos);
	pos+=sizeof(uint32_t);  	type=*(uint8_t*)(buf+pos);
	pos+=sizeof(uint8_t);		quality=*(double*)(buf+pos);
	pos+=sizeof(double);		prop_size=*(uint16_t*)(buf+pos);
	pos+=sizeof(uint16_t)+prop_size;

	datasize = GetDataSize(type);	

	if (debug) {
	    wmove(win3, 0, 0);
	    wprintw(win3, "Current Data. Duration: %6lums\n", ti);
	    win = win3;
	} else {	
	    rawtime = time(NULL) + 2 + ti / 500;
	
	    gmtime_r(&itm,&date);
	    sprintf(ct,"%02u:%02u:%02u",date.tm_hour,date.tm_min,date.tm_sec);

	    wmove(twin1,0,0);
	    wprintw(twin1,"%s. %s (Data: %'9ums","Detector",ct,ti);
	    wprintw(twin1, ", Confidence: % 6.2lf%)", quality*100);
	    wrefresh(twin1);

	    win = rawwin;
	    wmove(rawwin,0,0);
	}

	perrow = GetFormat(type, buf+pos, nd, datasize, ti, dispfmt, dispffmt);

	PrintValues(type, buf+pos, nd, datasize, ti, perrow, dispfmt, dispffmt, win);
	wprintw(rawwin,"\n");
	wrefresh(rawwin);
	
	if (debug) {
	    wrefresh(win3);
	}
	
	return 0;
}
