#include <stdio.h>
#include <string.h>
#include <curses.h>
#include <math.h>

#include "tools.h"

unsigned char GetDataSize(DataType type) {
	unsigned char datasize;
	
	switch(type) {
	    case DATA_TYPE_UINT8:
		datasize = sizeof(uint8_t);
	    break;
	    case DATA_TYPE_UINT16:
		datasize = sizeof(uint16_t);
	    break;
	    case DATA_TYPE_UINT32:
		datasize = sizeof(uint32_t);
	    break;
	    case DATA_TYPE_UINT64:
		datasize = sizeof(uint64_t);
	    break;
	    case DATA_TYPE_DOUBLE:
		datasize = sizeof(double);
	    break;
	    case DATA_TYPE_MIXED:
		datasize = sizeof(double);
	    break;
	    default:
		return -1;
	}	
	
	return datasize;
}

unsigned char GetValue(DataType type, char *buf, uint32_t ti, uint32_t *value, double *dvalue) {
    unsigned char have_float = 0;
    
    switch (type) {
	    case DATA_TYPE_UINT8:
		*value = *(uint8_t*)(buf);
	    break;
	    case DATA_TYPE_UINT16:
		*value = *(uint16_t*)(buf);
	    break;
	    case DATA_TYPE_UINT32:
	    	*value = *(uint32_t*)(buf);
	    break;
	    case DATA_TYPE_UINT64:
		*value = *(uint64_t*)(buf);
	    break;
	    case DATA_TYPE_DOUBLE:
		*dvalue = *(double*)(buf);
		have_float = 1;
	    break;
	    case DATA_TYPE_MIXED:
		*dvalue = *(double*)(buf);
		*value = lround(*dvalue);
		if (fabs(*dvalue-*value)>1E-4) have_float = 1;
	    break;
    }
    
    if (ti) *value = lrint(*value * 1000./ti);
    
    return have_float;
}

long GetMaxValue(DataType type, char *buf, unsigned long nd, unsigned char datasize, uint32_t ti) {
	size_t spos;
	long i, maxvalue = 0;
	uint32_t value;
	double dvalue;
	
	if (type == DATA_TYPE_DOUBLE) return -1;
    
        for (i=0,spos=0,maxvalue=0;i<nd;i++,spos+=datasize) {
		if (GetValue(type, buf+spos, ti, &value, &dvalue)) return -1;
		if (labs(value) > maxvalue) maxvalue = labs(value);
	}
	return maxvalue;
}

unsigned char GetFormat(DataType type, char *buf, unsigned long nd, unsigned char datasize, uint32_t ti, char *dispfmt, char *dispffmt) {
	unsigned char perrow = 8;
	uint32_t maxvalue;

	maxvalue = GetMaxValue(type, buf, nd, datasize, ti);
	
	strcpy(dispffmt, "%7.2lf  ");
	
	if (maxvalue<0) {
	    strcpy(dispfmt, "%7lu  ");
	    perrow = 8;
	} else if (maxvalue<10) {
	    strcpy(dispfmt, "%1lu  ");
	    perrow = 24;
	} else if (maxvalue<100) {
	    strcpy(dispfmt, "%2lu  ");
	    perrow = 16;
	}
	else if (maxvalue<1000) {
	    strcpy(dispfmt, "%3lu  ");
	    perrow = 12;
	}
	else if (maxvalue<10000) {
	    strcpy(dispfmt, "%4lu  ");
	    perrow = 12;
	}
	else {
	    strcpy(dispfmt, "%7lu  ");
	    perrow = 8;
	}
	
	if ((nd/22)<8) perrow = 8;
	else if (((nd/22)<12)&&(nd>12)) perrow = 12;
	else if (((nd/22)<16)&&(nd>16)) perrow = 16;
	
	return perrow;
}

long PrintValues(DataType type, char *buf, unsigned long nd, unsigned char datasize, uint32_t ti, int perrow, const char *dispfmt, const char *dispffmt, void *win) {
	WINDOW *rawwin = (WINDOW*)win;
	size_t spos;
	long i;
	
	uint32_t value;
	double dvalue;
	
	unsigned char have_float;

	for (i=0,spos=0;i<nd;i++,spos+=datasize) {
	    if (i%perrow==0) {
		wprintw(rawwin,"\n  ");
	    }

	    if (GetValue(type, buf+spos, ti, &value, &dvalue))
		    wprintw(rawwin, dispffmt, dvalue);
	    else 
	    	    wprintw(rawwin, dispfmt, value);
	}
}
