/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/*****************************************************************************
 *  UART0.C
 *
 *  Routines for UART0 communication of MB91360-devices
 *
 *  V1.1
 *  (C) Fujitsu Mikroelektronik GmbH
 *****************************************************************************/

/*

    ????????	Fujitsu Mikroelektronik GmbH
		V 1.1

    20050120	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Word & DWORD read / write
		Timeout support

    20050217	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Catch Frame Errors, Potentional Problem Fixes

*/


#include "global.h"
#include "mb91364.h"
#include "UART0.h"

#define PORT_DEVICE "/dev/tts/USB0"
#define PORT_SPEED 38400

#include <dsconnection.h>
DSConnectionContext con = NULL;
/*****************************************************************************
 *  DESCRIPTION:    Initializes UART0 for asyncronous point-to-point 
 *				communication mode
 *
 *  PARAMETERS:     baudr(1200..115200),datab(7,8),par(0..2),stopb(1,2)
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0init(DWORD baudr, BYTE datab, BYTE par, BYTE stopb)
{
    int err;
    DSConnectionCOMOptions options;
    DSDuration dur;
    
    options = dsConnectionCOMGetDefaultOptions();
    if (!options) return;

    con = dsConnectionCreateContext(NULL);
    if (!con) {
	dsConnectionDestroyContext(con);
	return;
    }
    
    dsConnectionCOMOptionsSetSpeed(options, PORT_SPEED);//baudr);
    err = dsConnectionConnectCOM(con, NULL, PORT_DEVICE, options);    
    dsConnectionCOMDestroyOptions(options);

    if (err) {
	dsConnectionDestroyContext(con);
	con = NULL;
    }

    dsConnectionSetDefaultFlags(con, DS_CONNECTION_FLAG_EXACTREAD);

    dur = dsDurationCreate(NULL);
    if (dur) {
	dsDurationImportMilliseconds(dur, UART_TIMEOUT);
	dsConnectionSetTimeout(con, dur);
	dsDurationDestroy(dur);
    }
}

// V.D. Function
int UART0received()
{
    return !dsConnectionCheckRead(con, NULL, 0, 0, NULL);
}

extern unsigned long mls;
int UART0skip() {
    char buffer[64];
    return dsConnectionTryRead(con, buffer, 64, 0, NULL);
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0getch (void)   
{
    unsigned char a;
    if (dsConnectionRead(con, &a, 1, 0, NULL)) return -1;
    return a;
}


/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0Tgetch (void)   
{
    unsigned char a;
    if (dsConnectionRead(con, &a, 1, 0, NULL)) return -1;
    return a;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming word (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0getchW (void)   
{
    unsigned short int a;
    if (dsConnectionRead(con, &a, 2, 0, NULL)) return -1;
    return a;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming word (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0TgetchW (void)   
{
    unsigned short int a;
    if (dsConnectionRead(con, &a, 2, 0, NULL)) return -1;
    return a;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming double word (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        "0" or "-1" (Error)
 *****************************************************************************/
int  UART0getchDW (unsigned long *ret)   
{
    unsigned int a;
    if (dsConnectionRead(con, &a, 4, 0, NULL)) return -1;
    return a;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming double word (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        "0" or "-1" (Error)
 *****************************************************************************/
int  UART0TgetchDW (unsigned long *ret)   
{
    unsigned int a;
    if (dsConnectionRead(con, &a, 4, 0, NULL)) return -1;
    return a;
}


/*****************************************************************************
 *  DESCRIPTION:    sends a single character  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putch (int ch)         	
{
    unsigned char a = ch;
    dsConnectionWrite(con, &a, 1, 0, NULL);
}

/*****************************************************************************
 *  DESCRIPTION:    sends a word  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putchW (int ch)         	
{
    unsigned short int a = ch;
    dsConnectionWrite(con, &a, 2, 0, NULL);
}

/*****************************************************************************
 *  DESCRIPTION:    sends a double word  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putchDW (int ch)         	
{
    unsigned int a = ch;
    dsConnectionWrite(con, &a, 4, 0, NULL);
}

/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putS(const char *buf)		
{
    dsConnectionWrite(con, buf, strlen(buf), 0, NULL);
}

