/*
    20040221	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Initial Version.

    20040226	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Switched to 4 byte send

    20041217	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Two way communication protocol. Initial Change.

    20050120	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Trigger signal in case of many-many

    20050121	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Trigger handling moved to IRQ handler

    20050125	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		GetCurrent, GetCurrentCoincidence
		Sync correction moved to PC code
		Send calculation duration along with detector data

    20050413	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Fixed bug in ddHandleEvent loop

*/

#include <stdio.h>
#include <stdlib.h>

#include "defs.h"
#include "mb91364.h"

#include "debug.h"
#include "detector.h"

extern unsigned long iter;
extern unsigned long mls;
extern unsigned char rest_flag;
extern unsigned short int counting_duration;
extern unsigned short int rest_counting_duration;
extern short int sync_flag;

int ddInitDetector(DetectorData dd, unsigned char detectors_number) {
    dd->detectors_number=detectors_number;
    ddResetData(dd);
    return 0;
}

void ddFreeDetector(DetectorData dd) {
}

void ddResetData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number, detectors_number2;
    DataType *detectors1, *detectors2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    coincidences=dd->coincidences;
    
    for (i=0;i<detectors_number;i++) {
	detectors1[i]=0;
	detectors2[i]=0;
    }
    
    for (i=0;i<detectors_number2;i++) {
	coincidences[i]=0;
    }
    
    dd->many_many = 0;
    dd->many_zero = 0;
    dd->zero_many = 0;
    dd->zero_one = 0;
    dd->one_zero = 0;
    dd->many_one = 0;
    dd->one_many = 0;
    dd->total = 0;
}

/*
void ddSyncData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number, detectors_number2;
    DataType *detectors1, *detectors2;
    DataType *coincidences;
    double multiplayer;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    coincidences=dd->coincidences;
    
    multiplayer = 1. * (counting_duration + rest_counting_duration) / (counting_duration + rest_counting_duration + sync_flag);
    
    
    for (i=0;i<detectors_number;i++) {
	detectors1[i]*=multiplayer;
	detectors2[i]*=multiplayer;
    }
    
    for (i=0;i<detectors_number2;i++) {
	coincidences[i]*=multiplayer;
    }
    
    dd->many_many *= multiplayer;
    dd->many_zero *= multiplayer;
    dd->zero_many *= multiplayer;
    dd->zero_one *= multiplayer;
    dd->one_zero *= multiplayer;
    dd->many_one *= multiplayer;
    dd->one_many *= multiplayer;
    dd->total *= multiplayer;
}
*/

int ddSendData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number;
    DataType *detectors1, *detectors2;

    detectors_number = dd->detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    
    UART0putchW(6+2*detectors_number*sizeof(DataType));
    UART0putchDW(iter-1);
    UART0putchW(counting_duration + rest_counting_duration + sync_flag);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors1[i]);

    for (i=0;i<detectors_number;i++)
	uart_function(detectors2[i]);
    
    return 0;
}


int ddSendCurrentData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number;
    DataType *detectors1, *detectors2;

    detectors_number = dd->detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    
    UART0putchW(6+2*detectors_number*sizeof(DataType));
    UART0putchDW(iter);
    if (rest_flag) UART0putchW(counting_duration + rest_counting_duration + sync_flag - mls);
    else UART0putchW(counting_duration - mls);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors1[i]);

    for (i=0;i<detectors_number;i++)
	uart_function(detectors2[i]);
    
    return 0;
}

int ddSendCoincidenceData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number, detectors_number2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    coincidences=dd->coincidences;

    UART0putchW(6+detectors_number2*sizeof(DataType)+8*sizeof(DataType));
    UART0putchDW(iter-1);
    UART0putchW(counting_duration + rest_counting_duration + sync_flag);

    for (i=0;i<detectors_number2;i++)
	uart_function(coincidences[i]);
    
    uart_function(dd->many_many);
    uart_function(dd->many_zero);
    uart_function(dd->zero_many);
    uart_function(dd->zero_one);
    uart_function(dd->one_zero);
    uart_function(dd->many_one);
    uart_function(dd->one_many);
    uart_function(dd->total);
    
    return 0;
}

int ddSendCurrentCoincidenceData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number, detectors_number2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    coincidences=dd->coincidences;

    UART0putchW(6+detectors_number2*sizeof(DataType)+8*sizeof(DataType));
    UART0putchDW(iter);
    if (rest_flag) UART0putchW(counting_duration + rest_counting_duration + sync_flag - mls);
    else UART0putchW(counting_duration - mls);

    for (i=0;i<detectors_number2;i++)
	uart_function(coincidences[i]);
    
    uart_function(dd->many_many);
    uart_function(dd->many_zero);
    uart_function(dd->zero_many);
    uart_function(dd->zero_one);
    uart_function(dd->one_zero);
    uart_function(dd->many_one);
    uart_function(dd->one_many);
    uart_function(dd->total);

    return 0;
}
