/*
    20040220	Varuzhan Danielyan <dan@xter.net>
		Initial Version.
		 
    20040221	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Timer0 configured to 1 millisecond
    
    20041101	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Applied patch from Varuzhan Danielyan
		    - PDRH is output port

    20050117	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Timer clock calibration
    
    20050119	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Timer clock fix
    
    20050120	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		DACR switched to 8 bit mode ( 3 -> 7 )
    
    20050121	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Second reload timer for trigger
		Watchdog timer
    
    20050514	Varuzhan Danielyan <dan@xter.net>
		IRQ is generated on High Level instead of Rising Edge
		(fixes random haltig issue)
*/

#include "MB91364.h"	// include all IO-Register definitions
#include "defs.h"

#include "config.h"

static unsigned long clock;
static unsigned short int timer_counter;

void Init()
{
  // Port G
  // To prevent floating, set as output if not used
  PDRG = 0;
  DDRG = 0x0F;
  
  // Port H
  // To prevent floating, unused pins set as outputs
  // PDRH_PDH0 set as output
  // PDRH_PDH0 = 1 - open gate, PDRH_PDH0 = 0 - close gate
  PDRH = 0;
  DDRH = 0xC1;
  
  // Port I
  // To prevent floating, set as output if not used
  PDRI = 0;
  DDRI = 0x08;
  
  // Port J
  PDRJ = 0x3A;	// All output signals are Active Low,
  		// bits 6, 7 zero to reset Latches
  PFRJ = 0xFF;	// High current outputs
  DDRJ = 0xFF;	// All outputs
  
  // Port K
  // To prevent floating, unused pins set as outputs
  PDRK = 0,
  PFRK = 0x03;
  DDRK = 0x3C;
  
  // Port L
  // To prevent floating, unused pins set as outputs
  PDRL = 0;
  DDRL = 0xC0;
  
  // Port M
  // To prevent floating, set as output if not used
  PDRM = 0;
  DDRM = 0x0C;
  
  // Port N
  // To prevent floating, set as output if not used
  PDRN = 0;
  DDRN = 0x38;
  
  // Port O
  PDRO = 0xFF;	// All output signals are Active Low
  DDRO = 0x0F;

  // Port P
  // External A0, A1
  PDRP = 0;
  DDRP = 0x03;

  // Port Q
  // UART0
  PDRQ = 2;
  PFRQ = 0x03;
  
  // Port R
  // All Inputs As Data Bus

  // Port T
  // UART 5
  // To prevent floating, unused pins set as outputs
  PDRT = 4;
  PFRT = 0x05;
  DDRT = 0xFE;


  // Enable DAC0 and DAC1 with 8 bit reslution;
  DACR = 7;
  /* DS: Should be adjustable from PC. Set DADR0,1 to 0-200 */
  DADR0 = 0x80;		// Bits 0..9 are setting output voltage of D/A converter 0
  DADR1 = 0x80;		// Bits 0..9 are setting output voltage of D/A converter 1

  /* 
     DIVR0_P contains PLL divisor for CLKP, 
     instead of 64000.0, PLL clock must be placed
  */  
  clock = 1000.0 * PLLclock / (DIVR0_P + 1);


     // Reload Timer 0
  ICR08 = 6;			// High priority Timer
  TMCSR0_INTE = 1;		// Enabling Interrupts
  TMCSR0_CNTE = 1;		// Enabling Counting
  TMCSR0_UF = 0;		// Reseting IRQ Flag
  TMCSR0_RELD = 1;		// Enabling Reload Operation
  TMRLR0 = clock / 32 - 1;	// IRQ generated once per 1ms
  TMCSR0_CSL = 2;		// Seting Timer clock divider to 32 (2 - 32, 1 - 8, 0 - 2)
//TMR0 = TMRLR0 = 0xFFFF;
//TMCSR0 = 0x81B;
    
    // Reload Timer 1
  ICR09 = 5;
  TMCSR1_INTE = 1;
  TMCSR1_CNTE = 1;
  TMCSR1_UF = 0;
  TMCSR1_RELD = 0;
  TMCSR1_CSL = 1;
  TMRLR1 = 2000;		// 1ms


  // External Interruption
  //ELVR = 0x000A;	// Both on Rising Edge
  ELVR = 0x0005;      //Both on High level

  /* DS: Shouldn't it have higher priority than timer? */
  ICR00 = 7;		// Interrupt Level ( smaller higher)
  ICR01 = 7;		// Interrupt Level ( smaller higher)
  ENIR = 0x03;		// Int0 & Int1

  __set_il(31);		// Set ILM to 31
  __EI();		// enable interrupts at all
  
    // Setting WatchDog
  RSRR_WT = 3;
}

int SetTimerClock(unsigned char clock_divider, unsigned short int counter) {
    if (clock_divider > 2) return -1;
    timer_counter = counter;
    TMCSR0_CSL = clock_divider;
    TMRLR0 = counter;
    TMCSR0_TRG = 1;    
    return 0;
}

int CorrectTimerClock(short int counter) {
    if (timer_counter + counter <= 0) return -1;
    timer_counter += counter;
    TMRLR0 = timer_counter;
    return 0;
}
