/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/*****************************************************************************
 *  UART0.C
 *
 *  Routines for UART0 communication of MB91360-devices
 *
 *  V1.1
 *  (C) Fujitsu Mikroelektronik GmbH
 *****************************************************************************/

/*

    ????????	Fujitsu Mikroelektronik GmbH
		V 1.1

    20050120	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Word & DWORD read / write
		Timeout support

    20050217	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Catch Frame Errors, Potentional Problem Fixes

*/


#include "global.h"
#include "mb91364.h"
#include "UART0.h"

/*****************************************************************************
 *  DESCRIPTION:    Initializes UART0 for asyncronous point-to-point 
 *				communication mode
 *
 *  PARAMETERS:     baudr(1200..115200),datab(7,8),par(0..2),stopb(1,2)
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0init(DWORD baudr, BYTE datab, BYTE par, BYTE stopb)
{
  /* Baudrate setting : */
  switch (baudr)
  {
    case 1200   :UTIMC0_UCC1 = 1;UTIMR0 = 415; break;	/* 1200 Baud */
    case 2400   :UTIMC0_UCC1 = 0;UTIMR0 = 207; break;	/* 2400 Baud */
    case 4800   :UTIMC0_UCC1 = 0;UTIMR0 = 103; break;	/* 4800 Baud */
    case 9600   :UTIMC0_UCC1 = 0;UTIMR0 = 51; break;	/* 9600 Baud*/
    case 19200  :UTIMC0_UCC1 = 0;UTIMR0 = 25; break;	/* 19200 Baud */
    case 38400  :UTIMC0_UCC1 = 0;UTIMR0 = 12; break;	/* 38400 Baud */
    case 57600  :UTIMC0_UCC1 = 1;UTIMR0 = 7; break;	/* 57600 Baud */
    case 115200 :UTIMC0_UCC1 = 1;UTIMR0 = 3; break;	/* 115200 Baud */
    default 	 :UTIMC0_UCC1 = 0;UTIMR0 = 51; break;	/* Default 9600 Baud*/
  }
  /* Number of Databits : */
  switch (datab)
  {
    case 7 :	SCR0_CL  = 0; break; /* 7 Databits */
    case 8 :	SCR0_CL  = 1; break; /* 8 Databits */
    default :	SCR0_CL  = 1; break; /* Default 8 Databits */
  }
  /* Number of Stopbits : */
  switch (stopb)
  {
    case 1 :	SCR0_SBL = 0; break; /* 1 Stopbit */
    case 2 :	SCR0_SBL = 1; break; /* 2 Stopbits */
    default :	SCR0_SBL = 0; break; /* Default 1 Stopbit */
  }
  /* Parity Select : */
  switch (par)
  {
    case 0 :	SCR0_PEN = 0; break; /* no parity */
    case 1 :	SCR0_PEN = 1; SCR0_P = 1; break; /* odd parity */
    case 2 :	SCR0_PEN = 1; SCR0_P = 0; break; /* even parity */
    default :	SCR0_PEN = 0; break; /* Default no parity */
  }

    PFRQ_PFQ0 = 1;		/* Pin PQ0 acts as Serial Input Pin */
    PFRQ_PFQ1 = 1;		/* Pin PQ1 acts as Serial Output Pin */
    SMR0_MD0 = 0;		/* async normal mode */
    SMR0_MD1 = 0;
    SMR0_CS0 = 0;		/* select U-timer (USE THIS ORDER FOR SOE,CS0)*/
    SCR0_RXE = 1;		/* Receive enable */
    SCR0_TXE = 1;		/* Transmit enable */
    UTIMC0_UTST = 1;	/* Start U-Timer */
  
}

// V.D. Function
int UART0received()
{
  return SSR0&UART_INPUT_MASK;
}

extern unsigned long mls;
int UART0skip() {
    unsigned char read = 0;
    unsigned long tmp = mls, passed = 0;
	
    while(passed < UART_TIMEOUT) {
	while (SSR0&UART_INPUT_MASK) {
	    if (SSR0&UART_ERROR_MASK) SCR0_REC = 0;
	    else if (SIDR0) read++;
	}
	if (mls != tmp) {
	    tmp = mls;
	    passed++;
	}
    }

    while (SSR0&UART_INPUT_MASK) {
        if (SSR0&UART_ERROR_MASK) SCR0_REC = 0;
        else if (SIDR0) read++;
    }

    return read;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0getch (void)   
{
	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		return (SIDR0);		/* return char */
}


/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0Tgetch (void)   
{
	unsigned long tmp = mls, passed = 0;
	
	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) {
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}

	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		return (SIDR0);		/* return char */
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming word (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0getchW (void)   
{
	int res;
	
	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res = SIDR0;		/* return char */

	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<8;	/* return char */

	return res;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming word (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART0TgetchW (void)   
{
	int res;
	unsigned long tmp = mls, passed = 0;
	
	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res = SIDR0;		/* return char */

	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<8;	/* return char */

	return res;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming double word (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        "0" or "-1" (Error)
 *****************************************************************************/
int  UART0getchDW (unsigned long *ret)   
{
	int res;
	
	if (!ret) return -1;
	
	while(!(SSR0&UART_INPUT_MASK));  /* wait for data in buffer */
	
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res = SIDR0;		/* return char */

	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<8;	/* return char */

	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<16;	/* return char */

	while(!(SSR0&UART_INPUT_MASK)); /* wait for data in buffer */
	if (SSR0&UART_ERROR_MASK) {      /* overrun or parity error */
		SCR0_REC = 0;           /* reset error flags */
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<24;	/* return char */

	*ret = res;
	return 0;
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming double word (with timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        "0" or "-1" (Error)
 *****************************************************************************/
int  UART0TgetchDW (unsigned long *ret)   
{
	int res;
	unsigned long tmp = mls, passed = 0;
	
	if (!ret) return -1;
	
	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res = SIDR0;		/* return char */

	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<8;	/* return char */

	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<16;	/* return char */

	while(!(SSR0&UART_INPUT_MASK) && passed < UART_TIMEOUT) { /* wait for data in buffer */
	    if (mls != tmp) {
		tmp = mls;
		passed++;
	    }
	}
	if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else if (!SSR0_RDRF) {
		return (-1);
	} else if (SSR0&UART_ERROR_MASK) {
		SCR0_REC = 0;
		UART0skip();
		return (-1);
	} else
		res += SIDR0<<24;	/* return char */

	*ret = res;
	return 0;
}


/*****************************************************************************
 *  DESCRIPTION:    sends a single character  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putch (int ch)         	
{

	while (!SSR0_TDRE);     /* wait for empty buffer */
	SODR0 = (unsigned char)ch;	/* fill register */

}

/*****************************************************************************
 *  DESCRIPTION:    sends a word  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putchW (int ch)         	
{

	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)ch;		/* fill register */
	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)(ch>>8);		/* fill register */
}

/*****************************************************************************
 *  DESCRIPTION:    sends a double word  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putchDW (int ch)         	
{
	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)ch;		/* fill register */
	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)(ch>>8);		/* fill register */
	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)(ch>>16);	/* fill register */
	while (!SSR0_TDRE);     		/* wait for empty buffer */
	SODR0 = (unsigned char)(ch>>24);	/* fill register */
}

/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART0putS(const char *buf)		
{
	while (*buf != '\0')
    { 
     if(*buf == 10){  UART0putch(13);  } 	/* correct CR LF seq */
     UART0putch(*buf++);		/* send every char of string */
    }
}

