// UART5.C
// Version 1.0
 
#include "global.H"
#include "MB91364.H"
#include "UART5.h"


void UART5init(void)
{
  // Initialize UART3 asynchronous mode
  BGR5 = 1666; // 1666 => 9600Baud @ 16MHz
  SCR5 = 0x17; // 8N1
  SMR5 = 0x0d; // enable SOT3, Reset, normal mode
  SSR5 = 0x00; // LSB first

  PFRT_PFT2 = 1;		/* Pin PT5 acts as Serial Input Pin */
  PFRT_PFT0 = 1;		/* Pin PT3 acts as Serial Output Pin */
}


/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/
int  UART6getch (void)   
{
	while(!SSR6_RDRF && !SSR6_ORE); /* wait for data in buffer */
	if (SSR6_ORE || SSR6_PE) {      /* overrun or parity error */
		SCR6_CRE = 0;           /* reset error flags */
		return (-1);
	} else
		return (RDR6);		/* return char */
}

/*****************************************************************************
 *  DESCRIPTION:    sends a single character  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART6putch (int ch)         	
{
	while (!SSR6_TDRE);     /* wait for empty buffer */
	TDR6 = (unsigned char)ch;	/* fill register */

}


/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART6putS(const char *buf)		
{
	while (*buf != '\0')
    { 
     if(*buf == 10){  UART6putch(13);  } 	/* correct CR LF seq */
     UART6putch(*buf++);		/* send every char of string */
    }
}

