/*
    20040220	Varuzhan Danielyan <dan@xter.net>
		Initial Version: Blank with Interrupt handler, per event communication
		 
    20040221	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Blank for error handling (debug.c)
		Implementation of DetectorData (detector.c)
		Separation of supporting tools (support.c)
		Buffering of incoming events
		Run 1 millisecond timer
		Write data every COUNTING_DURATION seconds
		Configuration in (config.h)

    20041129	Varuzhan Danielyan <dan@xter.net>
		Interupt handling optimization
    
    20041217	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Two way communication protocol. Initial Change.
    
    20050117	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Synchronization and calibration stuff

    20050119	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Runtime sync autocorrection
    
    20050120	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Configuration of output voltage of D/A converters
		Configuration of duration of trigger signal
		Configuration of detector numbers

    20050121	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		More preciese trigger handling
		Fixed errors with Detector2 handling
		Support for Watchdog Timer
    
    20050125	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		GetCurrent, GetCurrentCoincidence
		Sync correction moved to PC code
		Ultimate Fixes: Watchdog, Error codes

    20050127	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Error handling improvenment (COMMAND_RESPONSE_RETRY)
		VersionInfo mangled

    20050216	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		RESET, WATCHDOG, SYNC_DIVIDER commands
		Improved sync fixes, can be adjusted by master software
		Change all reads on supporting timeout modifycation
    
    20050219	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Disable LED3 to overcome board bug

    20050223	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Fix for first iteration counting

    20050313	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Freeze Fix (I Hope) 
		    Enable EIRR before C_SEL

    20050413	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Removed fix from 20050313
		Fixed bug in ihTriggerTimer16

    20050413	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		Some mixed breaks are fixed
		Light indication altered (3rd light represents sync status)

    20051216	Suren Chilingaryan <csa@crdlx5.yerphi.am>
		ERROR_* are renamed to COMMAND_ERROR_*
*/


#include <stdio.h>
#include <stdlib.h>

#include "mb91364.h"	// include all IO-Register definitions
#include "defs.h"

#define _MAIN_C
#include "config.h"
#include "debug.h"


unsigned char Event1 = 0, Event2 = 0;
unsigned char Data11[BUFFER_SIZE], Data12[BUFFER_SIZE];
unsigned char Data21[BUFFER_SIZE], Data22[BUFFER_SIZE];
unsigned int buffer1_start=0, buffer1_end=0;
unsigned int buffer2_start=0, buffer2_end=0;

unsigned long mls = DEFAULT_COUNTING_DURATION;		// Counter of milliseconds
unsigned long iter = 0;					// Current Iteration
unsigned char insync = 0;				// Flag: If Time is Synchronized
short int sync = 0;
short int sync_step = DEFAULT_COUNTING_DURATION / DEFAULT_SYNC_DIVIDER;
short int sync_divider = DEFAULT_SYNC_DIVIDER;

unsigned char calibration = 0;
unsigned char write_flag = 0;
unsigned char reset_flag = 0;
unsigned char rest_flag = 1;
unsigned char switch_flag = 0;
short int sync_flag = 0;

unsigned short int rest_counting_duration = DEFAULT_COUNTING_DURATION / COUNTING_DIVIDER;
unsigned short int counting_duration = DEFAULT_COUNTING_DURATION - (DEFAULT_COUNTING_DURATION / COUNTING_DIVIDER);

#include "detector.h"

struct DetectorDataT dd11, dd12;
struct DetectorDataT dd21, dd22;
DetectorData dd1, dd2;
DetectorData ddp1, ddp2;
DetectorData ddt1, ddt2;

void __interrupt ihTriggerTimer16() {
    PDRH_PDH0 = 0;
    TMCSR1_UF = 0;
}

void __interrupt ihTimer16()
{
  if (--mls==0) {
    if (rest_flag) {
	mls=counting_duration;
	ddt1=dd1; dd1=ddp1; ddp1=ddt1;
	ddt2=dd2; dd2=ddp2; ddp2=ddt2;
	if (insync) {
	    if (!(++iter)) iter = 2;
	}
	rest_flag = 0;
    } else {
	if (sync) {
	    if (sync>0) {
		if (sync>sync_step) {
		    mls = rest_counting_duration - sync_step;
		    sync_flag = -sync_step;
	    	    sync -= sync_step;
		} else {
		    mls = rest_counting_duration - sync;
		    sync_flag = -sync;
	    	    sync = 0;
		}
	    } else {
		if ((-sync)>sync_step) {
		    mls = rest_counting_duration + sync_step;
		    sync_flag = sync_step;
		    sync += sync_step;
		} else {
		    mls = rest_counting_duration - sync;
		    sync_flag = -sync;
		    sync = 0;
		}
	    }
	} else {
	    mls = rest_counting_duration;
	    sync_flag = 0;
	}
	reset_flag = 1;
	rest_flag = 1;
    }
  }

  if ((mls % LIGHT_SWITCH_INTERVAL) == 0) {
    if (switch_flag&1) LED0 = 1 - LED0;
    else LED0 = 1;
    
    if (switch_flag&2) LED1 = 1 - LED1;
    else LED1 = 1;

    if (insync) {
	if (switch_flag&4) LED2 = 1 - LED2;
	else LED2 = 0;
    } else {
	if ((mls % (LIGHT_SWITCH_INTERVAL*LIGTH_SWITCH_MULTIPLAYER)) == 0) LED2 = 1 - LED2;
    }

/*  Setting LED3 on causes unpredictible board behaviour 
    (still simple loops like in debug.c are still working okey)

    if (switch_flag&8) LED3 = 1 - LED3;
    else LED3 = 1;
*/
    
    switch_flag = 0;
  }

  TMCSR0_UF = 0;
}

void __interrupt ExtInt0()
{
    register unsigned char tmp1, tmp2;
    
	// Read Data
    PDRO = Select1;

    //  C_SEL1_L = 0;
    tmp1 = PDRR;
    //  C_SEL1_L = 1;
    PDRO = Select2;
    //  C_SEL2_L = 0;
    tmp2 = PDRR;
    //  C_SEL2_L = 1;

    C_RES1_L = 0;

	/* DS: optimize assembler code, extra LDI operator */
    if ((tmp1==0)||(tmp2==0)||(tmp1==1)||(tmp2==1)||(tmp1==2)||(tmp2==2)||
	(tmp1==4)||(tmp2==4)||(tmp1==8)||(tmp2==8)||(tmp1==16)||(tmp2==16)||
	(tmp1==32)||(tmp2==32)||(tmp1==64)||(tmp2==64)||(tmp1==128)||(tmp2==128)) goto skip;

    PDRH_PDH0 = 1;
    TMCSR1_TRG = 1;

skip:


    Data11[buffer1_end] = tmp1;
    Data12[buffer1_end] = tmp2;
    
    if ((++buffer1_end)==BUFFER_SIZE) buffer1_end = 0;

    if (buffer1_end == buffer1_start) Event1 = 1;
    else C_RES1_L = 1;

    EIRR_ER0 = 0;
}

void __interrupt ExtInt1()
{
    register unsigned char tmp1, tmp2;
    
    // Read Data
    PDRO = Select3;
    //  C_SEL3_L = 0;
    tmp1 = PDRR;
    //  C_SEL3_L = 1;
    PDRO = Select4;
    //  C_SEL4_L = 0;
    tmp2 = PDRR;
    //  C_SEL4_L = 1;

    C_RES2_L = 0;

	/* DS: optimize assembler code, extra LDI operator */
    if ((tmp1==0)||(tmp2==0)||(tmp1==1)||(tmp2==1)||(tmp1==2)||(tmp2==2)||
	(tmp1==4)||(tmp2==4)||(tmp1==8)||(tmp2==8)||(tmp1==16)||(tmp2==16)||
	(tmp1==32)||(tmp2==32)||(tmp1==64)||(tmp2==64)||(tmp1==128)||(tmp2==128)) goto skip;

    PDRH_PDH0 = 1;
    TMCSR1_TRG = 1;

skip:


    Data21[buffer2_end] = tmp1;
    Data22[buffer2_end] = tmp2;
  
    if ((++buffer2_end)==BUFFER_SIZE) buffer2_end=0;
    
    if (buffer2_end == buffer2_start) Event2 = 1;
    else C_RES2_L = 1;
    EIRR_ER1 = 0;
}

static void WatchDog() {
    WPR = 0xA5;
    WPR = 0x5A;
}
#pragma inline WatchDog

void Process1Events() {
    while ((buffer1_start!=buffer1_end)||(Event1)) {
	WatchDog();
	if (Event1) {
	    debug(DEBUG_ERROR,ERR_BUFFER_FULL);

	    if (buffer1_start!=buffer1_end) {
		Event1 = 0;
		C_RES1_L = 1;
	    }
	}
	ddHandleEvent(dd1,Data11[buffer1_start],Data12[buffer1_start]);
	if ((++buffer1_start)==BUFFER_SIZE) buffer1_start=0;
	switch_flag |= 1;
    }
}

void Process2Events() {
    while ((buffer2_start!=buffer2_end)||(Event2)) {
	WatchDog();
	if (Event2) {
	    debug(DEBUG_ERROR,ERR_BUFFER_FULL);

	    if (buffer2_start!=buffer2_end) {
		Event2 = 0;
		C_RES2_L = 1;
	    }
	}
	ddHandleEvent(dd2,Data21[buffer2_start],Data22[buffer2_start]);
	if ((++buffer2_start)==BUFFER_SIZE) buffer2_start=0;	
	switch_flag |= 2;
    }
}
 

static void DropSync() {
    LED2 = 1;
    insync = 0;
    calibration = 0;
    iter = 0;
}
#pragma inline DropSync

void main()
{
    int err;
    short int cmd;
    long int tmp, tmp_param;
  
    Event1 = Event2 = 0;

	/* Initialize Defaults */
    

	/* Self Initialization */

    Init();
    LED0 = LED1 = LED2 = LED3 = 1;

    C_RES1_L = C_RES2_L = 0;	// Reset Detectors 1 and 2

	/* Initialize  Uart0 */
    UART0init(BAUD_RATE,8,0,1);

  //  Init Detectors
  err=ddInitDetector(&dd11,MAX_DETECTORS_NUMBER1);
  if (err) debug(DEBUG_FATAL,err);
  err=ddInitDetector(&dd12,MAX_DETECTORS_NUMBER1);
  if (err) debug(DEBUG_FATAL,err);
  err=ddInitDetector(&dd21,MAX_DETECTORS_NUMBER2);
  if (err) debug(DEBUG_FATAL,err);
  err=ddInitDetector(&dd22,MAX_DETECTORS_NUMBER2);
  if (err) debug(DEBUG_FATAL,err);
  dd1=&dd11;ddp1=&dd12;
  dd2=&dd21;ddp2=&dd22;

  TMCSR0_TRG = 1;		// Running counter
  C_RES1_L = C_RES2_L = 1;	// Enable Detectors 1 and 2
  
  while (1) {
    WatchDog();
    
    do {
	Process1Events();
	Process2Events();
    } while ((rest_flag)&&(mls<DEAD_COUNTING_DURATION));
    
    
    if (reset_flag) {
	ddResetData(ddp1);
	ddResetData(ddp2);
	reset_flag = 0;
    }

    while (UART0received()) {
	cmd = UART0getch();
	
	WatchDog();
	switch (cmd) {
	    case COMMAND_REQUEST_SYNC:
		mls = UART0TgetchW();

		if (mls == (unsigned long)-1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    DropSync();
		    break;
		}
		
		if (mls < rest_counting_duration) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_ENOUGH_TIME);
		    DropSync();
		    break;
		}

		UART0putch(COMMAND_RESPONSE_SYNC);

		if (!rest_flag) {
		    ddResetData(ddp1);
		    ddResetData(ddp2);
		    rest_flag = 1;
		}

		reset_flag = 0;
		
		iter = 0;
		calibration = 0;
		sync = 0;

		buffer1_start = buffer1_end = 0;
		buffer2_start = buffer2_end = 0;

		if (Event1) {
		    Event1 = 0;
		    C_RES1_L = 1;
		}
		
		if (Event2) {
		    Event2 = 0;
		    C_RES2_L = 1;
		}
		
		insync = 1;

		LED2 = 0;
	    break;
	    case COMMAND_REQUEST_GET_SYNC:
		if (!insync) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_GET_SYNC);
		UART0putchW(6);
		if (rest_flag) UART0putchW(mls);
		else UART0putchW(mls+rest_counting_duration);
		UART0putchDW(iter);
	    break;
	    case COMMAND_REQUEST_CALIBRATION_START:
		mls = 0xFFFF;
		UART0putch(COMMAND_RESPONSE_CALIBRATION_START);
		DropSync();
	        while (!UART0received()) WatchDog();
	    break;
	    case COMMAND_REQUEST_CALIBRATION:
		tmp = mls;
		if (!calibration) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_CALIBRATION);
		UART0putchW(0xFFFF - tmp);
	        while (!UART0received()) WatchDog();
	    break;
	    case COMMAND_REQUEST_CALIBRATION_FINISH:
		tmp = mls;
		if (!calibration) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_CALIBRATION_FINISH);
		UART0putchW(0xFFFF - tmp);
		calibration = 0;
	    break;
	    case COMMAND_REQUEST_GET_CURRENT:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		if (!iter) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    UART0putch(COMMAND_RESPONSE_GET_CURRENT);
		    ddSendCurrentData(dd1);
		} else if (tmp==1) {
		    UART0putch(COMMAND_RESPONSE_GET_CURRENT);
		    ddSendCurrentData(dd2);
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_CURRENT_COINCIDENCE:
		tmp = UART0TgetchW();

		if (tmp==-1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		if (!iter) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    UART0putch(COMMAND_RESPONSE_GET_CURRENT_COINCIDENCE);
		    ddSendCurrentCoincidenceData(dd1);
		} else if (tmp==1) {
		    UART0putch(COMMAND_RESPONSE_GET_CURRENT_COINCIDENCE);
		    ddSendCurrentCoincidenceData(dd2);
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET:
		tmp = UART0TgetchW();
		
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		if ((rest_flag)||(iter<2)) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    if (!insync) UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    else if (iter) UART0putch(COMMAND_ERROR_EXCHANGE_TIME_IS_OVER);
		    else UART0putch(COMMAND_ERROR_NO_DATA);
		    break;
		} 

		if (tmp==0) {
		    UART0putch(COMMAND_RESPONSE_GET);
		    ddSendData(ddp1);
		} else if (tmp==1) {
		    UART0putch(COMMAND_RESPONSE_GET);
		    ddSendData(ddp2);
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_COINCIDENCE:
		tmp = UART0TgetchW();
		
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		if ((rest_flag)||(iter<2)) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    if (!insync) UART0putch(COMMAND_ERROR_NOT_INITIALIZED);
		    else if (iter) UART0putch(COMMAND_ERROR_EXCHANGE_TIME_IS_OVER);
		    else UART0putch(COMMAND_ERROR_NO_DATA);
		    break;
		} 

		if (tmp==0) {
		    UART0putch(COMMAND_RESPONSE_GET_COINCIDENCE);
		    ddSendCoincidenceData(ddp1);
		} else if (tmp==1) {
		    UART0putch(COMMAND_RESPONSE_GET_COINCIDENCE);
		    ddSendCoincidenceData(ddp2);
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_INFO:
		UART0putch(COMMAND_RESPONSE_INFO);
		UART0putchW(9);
		UART0putchW(VERSION);
		UART0putch(sizeof(DataType));

		UART0putchW(PLLclock);
		UART0putch(DIVR0_P + 1); UART0putch(DIVR1_T + 1);
		UART0putchW(MAX_DETECTORS_NUMBER);
	    break;
	    case COMMAND_REQUEST_SET_COUNTING_DURATION:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		counting_duration = tmp;
		rest_counting_duration = counting_duration / COUNTING_DIVIDER;

		sync_step = counting_duration / sync_divider;
		if (!sync_step) sync_step = 1;
		else if (sync_step*2 > rest_counting_duration) sync_step = rest_counting_duration / 2;
		counting_duration -= rest_counting_duration;
		UART0putch(COMMAND_RESPONSE_SET_COUNTING_DURATION);
		DropSync();
	    break;
	    case COMMAND_REQUEST_SET_SYNC_DIVIDER:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		    
		}
		sync_divider = tmp;
		sync_step = (counting_duration + rest_counting_duration) / sync_divider;
		if (!sync_step) sync_step = 1;
		else if (sync_step*2 > rest_counting_duration) sync_step = rest_counting_duration / 2;
		UART0putch(COMMAND_RESPONSE_SET_SYNC_DIVIDER);
	    break;
	    case COMMAND_REQUEST_SET_TIMER_CLOCK:
	    	tmp = UART0Tgetch();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		tmp_param = UART0TgetchW();
		if (tmp_param == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		if (SetTimerClock(tmp, tmp_param)) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_SET_TIMER_CLOCK);
		DropSync();
	    break;
	    case COMMAND_REQUEST_CORRECT_TIMER_CLOCK:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		if (CorrectTimerClock(tmp)) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_CORRECT_TIMER_CLOCK);
		DropSync();
	    break;
	    case COMMAND_REQUEST_SET_SYNC:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		sync = tmp;
		UART0putch(COMMAND_RESPONSE_SET_SYNC);
	    break;
	    case COMMAND_REQUEST_PREPARE_SYNC:
		UART0putch(COMMAND_RESPONSE_PREPARE_SYNC);
	        while (!UART0received()) WatchDog();
	    break;
	    case COMMAND_REQUEST_SET_TRIGGER_DURATION:
		tmp = UART0TgetchW();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}
		
		if (tmp > 32767) {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		TMRLR1 = tmp * 2;
		UART0putch(COMMAND_RESPONSE_SET_TRIGGER_DURATION);
	    break;
	    case COMMAND_REQUEST_SET_DA_VOLTAGE:
		tmp_param = UART0Tgetch();
		if (tmp_param == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		tmp = UART0Tgetch();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}


		if (!tmp) {
		    DADR0 = tmp_param;
		} else if (tmp==1) {
		    DADR1 = tmp_param;
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		    break;
		}
		UART0putch(COMMAND_RESPONSE_SET_DA_VOLTAGE);
	    break;
	    case COMMAND_REQUEST_SET_DETECTOR_NUMBER:
		tmp_param = UART0Tgetch();
		if (tmp_param == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		tmp = UART0Tgetch();
		if (tmp == -1) {
		    UART0putch(COMMAND_RESPONSE_RETRY);
		    UART0putch(COMMAND_ERROR_UART);
		    break;
		}

		if (!tmp) {
		    if (tmp_param > MAX_DETECTORS_NUMBER1) {
			UART0putch(COMMAND_RESPONSE_ERROR);
			UART0putch(COMMAND_ERROR_INVALID_PARAMETER);
			break;
		    }
		    
		    ddFreeDetector(&dd11);
		    err=ddInitDetector(&dd11,tmp_param);
		    if (err) debug(DEBUG_FATAL,err);
		    ddFreeDetector(&dd12);
		    err=ddInitDetector(&dd12,tmp_param);
		    if (err) debug(DEBUG_FATAL,err);
		    
		} else if (tmp==1) {
		    if (tmp_param > MAX_DETECTORS_NUMBER2) {
			UART0putch(COMMAND_RESPONSE_ERROR);
			UART0putch(COMMAND_ERROR_INVALID_PARAMETER);
			break;
		    }

		    ddFreeDetector(&dd21);
		    err=ddInitDetector(&dd21,tmp_param);
		    if (err) debug(DEBUG_FATAL,err);
		    ddFreeDetector(&dd22);
		    err=ddInitDetector(&dd22,tmp_param);
		    if (err) debug(DEBUG_FATAL,err);
		    
		} else {
		    UART0putch(COMMAND_RESPONSE_ERROR);
		    UART0putch(COMMAND_ERROR_INVALID_INSTALLATION_ID);
		    break;
		}
		
		DropSync();
		UART0putch(COMMAND_REQUEST_SET_DETECTOR_NUMBER);
	    break;
	    case COMMAND_REQUEST_WATCHDOG:
		RSRR_WT = 3;
		UART0putch(COMMAND_REQUEST_WATCHDOG);
	    break;
	    case COMMAND_REQUEST_RESET:
		UART0putch(COMMAND_RESPONSE_RESET);
		STCR_SRST = 0;
		while (1);
	    case -1:
		UART0putch(COMMAND_RESPONSE_RETRY);
		UART0putch(COMMAND_ERROR_UART);
	    break;
	    default:
		UART0skip();
		UART0putch(COMMAND_RESPONSE_ERROR);
		UART0putch(COMMAND_ERROR_INVALID_REQUEST);
		UART0putchW(cmd);
	}

	WatchDog();

	if (insync) break;
    }
  }
}
