#include <string.h>
#include <stdlib.h>

#include "setup.h"
#include "commands.h"
#include "time.h"
#include "sync.h"
#include "watchdog.h"
#include "detector.h"
#include "error.h"
#include "serial.h"
#include "ladc.h"
#include "i2c.h"
#include "spectrum.h"
#include "tools.h"
#include "layout.h"

unsigned char calibration = 0;

int COMMANDprocess(int input, unsigned short int cmd) {
	int err;
	long int tmp, tmp_param;
	char request[MAX_CONTROL_SIZE + 1], response[MAX_CONTROL_SIZE + 1];
	char *stmp;

restart:
	switch (cmd) {
	    case COMMAND_REQUEST_SYNC:
		mls = UARTgetchW(input);

		if (mls == (unsigned long)-1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    SYNCdrop();
		    break;
		}
		
		if (mls < REQUIRED_INITIALIZATION_TIME) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_ENOUGH_TIME);

		    SYNCdrop();
		    break;
		}

		UARTputch(input, COMMAND_RESPONSE_SYNC);

		if (!rest_flag) {
		    ddResetData(ddp1);
		    rest_flag = 1;
		}
		
		reset_flag = 0;

		iter = 0;
		calibration = 0;
		sync = 0;

		buffer1_start = buffer1_end = 0;
		buffer1_new_iteration = 0;
		
		store_events = 0;
		mmevents = 0;
		
		ERRORsetup();
		SPECTRUMreset();

		Event1 = 0; Overload1 = 0; irq_disabled = 0;
		LADCenableIRQ();
		
		insync = 1;
	    break;
	    case COMMAND_REQUEST_GET_SYNC:
		if (!SYNCcheck()) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UARTputch(input, COMMAND_RESPONSE_GET_SYNC);
		UARTputchW(input, 6);
		UARTputchW(input, TIMEremaining());
		UARTputchDW(input, iter);
	    break;
	    case COMMAND_REQUEST_CALIBRATION_START:
		mls = 0x7FFFFFFF;
		UARTputch(input, COMMAND_RESPONSE_CALIBRATION_START);
		SYNCdrop();
		calibration = 1;
	        while (!UARTreceived(input)) { WDfeed(); }
		cmd = UARTgetch(input);
		goto restart;
	    break;
	    case COMMAND_REQUEST_CALIBRATION:
		tmp = mls;
		if (!calibration) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UARTputch(input, COMMAND_RESPONSE_CALIBRATION);
		UARTputchW(input, 0x7FFFFFFF - tmp);
	        while (!UARTreceived(input)) { WDfeed(); }
		cmd = UARTgetch(input);
		goto restart;
	    break;
	    case COMMAND_REQUEST_CALIBRATION_FINISH:
		tmp = mls;
		if (!calibration) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		}
		UARTputch(input, COMMAND_RESPONSE_CALIBRATION_FINISH);
		UARTputchW(input, 0x7FFFFFFF - tmp);
		calibration = 0;
	    break;
	    case COMMAND_REQUEST_GET_CURRENT:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (!iter) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    UARTputch(input, COMMAND_RESPONSE_GET_CURRENT);
		    ddSendCurrentData(input, dd1);
		} else {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_CURRENT_COINCIDENCE:
		tmp = UARTgetchW(input);

		if (tmp==-1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		if (!iter) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    UARTputch(input, COMMAND_RESPONSE_GET_CURRENT_COINCIDENCE);
		    ddSendCurrentCoincidenceData(input, dd1);
		} else {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET:
		tmp = UARTgetchW(input);
		
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if ((rest_flag)||(iter<2)) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    if (!insync) UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    else if (iter) UARTputch(input, COMMAND_ERROR_EXCHANGE_TIME_IS_OVER);
		    else UARTputch(input, COMMAND_ERROR_NO_DATA);
		    break;
		} 

		if (tmp==0) {
		    UARTputch(input, COMMAND_RESPONSE_GET);
		    ddSendData(input, ddp1);
		} else {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_COINCIDENCE:
		tmp = UARTgetchW(input);
		
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if ((rest_flag)||(iter<2)) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    if (!insync) UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    else if (iter) UARTputch(input, COMMAND_ERROR_EXCHANGE_TIME_IS_OVER);
		    else UARTputch(input, COMMAND_ERROR_NO_DATA);
		    break;
		} 

		if (tmp==0) {
		    UARTputch(input, COMMAND_RESPONSE_GET_COINCIDENCE);
		    ddSendCoincidenceData(input, ddp1);
		} else {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_SPECTRUM:
		tmp = UARTgetchW(input);

		if (tmp==-1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		if (!iter) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    UARTputch(input, COMMAND_RESPONSE_GET_SPECTRUM);
		    ddSendSpectrum(input, dd1);
		} else {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_GET_EVENTS:
		tmp = UARTgetchW(input);

		if (tmp==-1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		if (!iter) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_NOT_INITIALIZED);
		    break;
		} 
		
		if (tmp==0) {
		    store_events = 1;
		    
		    UARTputch(input, COMMAND_RESPONSE_GET_EVENTS);
		    ddSendEvents(input, dd1);
		} else {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_INSTALLATION_ID);
		}
	    break;
	    case COMMAND_REQUEST_INFO:
		UARTputch(input, COMMAND_RESPONSE_INFO);
		UARTputchW(input, 16);
		UARTputchW(input, PROTOCOL_VERSION);
		UARTputch(input, sizeof(DataType));
		UARTputchW(input, Fcclk/1000000);

		UARTputchW(input, TYPE); UARTputchW(input, VERSION);
		UARTputchDW(input, handled_events);
		UARTputch(input, 1); UARTputchW(input, MAX_DETECTORS_NUMBER);
	    break;
	    case COMMAND_REQUEST_SET_COUNTING_DURATION:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		UARTputch(input, COMMAND_RESPONSE_SET_COUNTING_DURATION);
		TIMEduration(tmp);
		SYNCdrop();
	    break;
	    case COMMAND_REQUEST_SET_SYNC_DIVIDER:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		    
		}
		UARTputch(input, COMMAND_RESPONSE_SET_SYNC_DIVIDER);
		SYNCdivider(tmp);
	    break;
	    case COMMAND_REQUEST_SET_SYNC:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		SYNCcorrection(tmp);
		UARTputch(input, COMMAND_RESPONSE_SET_SYNC);
	    break;
	    case COMMAND_REQUEST_ERRORS:
		UARTputch(input, COMMAND_RESPONSE_ERRORS);
		UARTputchW(input, ERRORget());
	    break;
	    case COMMAND_REQUEST_PREPARE_SYNC:
		UARTputch(input, COMMAND_RESPONSE_PREPARE_SYNC);
	        while (!UARTreceived(input)) { WDfeed(); }
		cmd = UARTgetch(input);
		goto restart;
	    break;
	    case COMMAND_REQUEST_SET_SOFTWARE_THRESHOLD:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp>=MAX_SENSORS_NUMBER) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		tmp_param = UARTgetchW(input);
		if (tmp_param == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		UARTputch(input, COMMAND_RESPONSE_SET_SOFTWARE_THRESHOLD);
		thresholds[tmp] = tmp_param - 1;
	    break;
	    case COMMAND_REQUEST_SET_THRESHOLD:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp>=MAX_SENSORS_NUMBER) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		tmp_param = UARTgetchW(input);
		if (tmp_param == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		UARTputch(input, COMMAND_RESPONSE_SET_THRESHOLD);

		if (Threshold[tmp] != tmp_param) {
		    Threshold[tmp] = tmp_param;
		    
//		    VICIntEnClr = LADC_INT;  	// Disable LADC IRQ
            	    WriteEEPROM(E_Thresholds + tmp, tmp_param);
            	    WriteDAC(tmp, tmp_param);
//		    VICIntEnable = LADC_INT;	// Enable LADC IRQ
		}
	    break;
	    case COMMAND_REQUEST_GET_THRESHOLD:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp>=MAX_SENSORS_NUMBER) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		UARTputch(input, COMMAND_RESPONSE_GET_THRESHOLD);
		UARTputchW(input, Threshold[tmp]);
	    break;
	    case COMMAND_REQUEST_SET_HV:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp>=MAX_SENSORS_NUMBER) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		tmp_param = UARTgetchW(input);
		if (tmp_param == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		UARTputch(CONTROL_CONNECTION, 'V');
		PrintNumber(CONTROL_CONNECTION, tmp+1);
		UARTputch(CONTROL_CONNECTION, ' ');
		PrintNumber(CONTROL_CONNECTION, tmp_param);
		UARTputch(CONTROL_CONNECTION, ' ');

		tmp = UARTgets(CONTROL_CONNECTION, response, MAX_CONTROL_SIZE);
		if (tmp == -1) { 
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_CONTROL_RS485);
		    break;
		}
			
		UARTputch(input, COMMAND_RESPONSE_SET_HV);
	    break;
	    case COMMAND_REQUEST_GET_HV:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp>=MAX_SENSORS_NUMBER) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		UARTputch(CONTROL_CONNECTION, 'H');
		PrintNumber(CONTROL_CONNECTION, tmp+1);
		UARTputch(CONTROL_CONNECTION, ' ');

		tmp = UARTgets(CONTROL_CONNECTION, response, MAX_CONTROL_SIZE);
		if (tmp == -1) { 
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_CONTROL_RS485);
		    break;
		}
		
		stmp = strchr(response, '=');
		if (!stmp) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_CONTROL_RS485);
		    break;
		}
		
		UARTputch(input, COMMAND_RESPONSE_GET_HV);
		UARTputchW(input, atoi(stmp+1));
	    break;
	    case COMMAND_REQUEST_CONTROL:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp > MAX_CONTROL_SIZE) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		if ((UARTgets(input, request, tmp + 1) == -1)||(strlen(request)!=tmp)) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		tmp = UARTrequest(CONTROL_CONNECTION, request, response, MAX_CONTROL_SIZE);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		UARTputch(input, COMMAND_RESPONSE_CONTROL);
		UARTputchW(input, strlen(response));
		UARTputs(input, response);
	    break;
	    case COMMAND_REQUEST_RESET_FPGA:
		UARTputch(input, COMMAND_RESPONSE_RESET_FPGA);
		LADCresetBoard();
	    break;
	    case COMMAND_REQUEST_SPECTRUM_CONFIGURE_OUTPUT:
		// Channel, CursorPosition, Flags(LogarithmFlag)
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		if (tmp != 3) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		    
		tmp = UARTgetch(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
	    
		if (tmp >= LADCchannels) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		ChannelToShow = tmp;

		tmp = UARTgetch(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}

		LogarithmFlag = tmp?1:0;
	    
		tmp = UARTgetch(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
	    
		if (tmp > 127) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		CursorPosition = tmp; 

		UARTputch(input, COMMAND_RESPONSE_SPECTRUM_CONFIGURE_OUTPUT);
	    break;
	    case COMMAND_REQUEST_SPECTRUM_RESET:
		UARTputch(input, COMMAND_RESPONSE_SPECTRUM_RESET);
		SPECTRUMreset();
	    break;
	    
/*	    case COMMAND_REQUEST_SET_TRIGGER_DURATION:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		if (tmp > 32767) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}
		
		TMRLR1 = tmp * 2;
		UARTputch(input, COMMAND_RESPONSE_SET_TRIGGER_DURATION);
	    break;*/
	    case COMMAND_REQUEST_WATCHDOG:
		UARTputch(input, COMMAND_REQUEST_WATCHDOG);
	    break;
	    case COMMAND_REQUEST_RESET:
		UARTputch(input, COMMAND_RESPONSE_RESET);
		WDsetup();
		while (1);
	    case COMMAND_REQUEST_PING:
		UARTputch(input, COMMAND_RESPONSE_PING);
	    break;
	    case COMMAND_REQUEST_LAYOUT_INFO:
		UARTputch(input, COMMAND_RESPONSE_LAYOUT_INFO);
#ifdef HAVE_LAYOUT
		UARTputchW(input, 9);
		UARTputch(input, 1);
		UARTputchW(input, MAX_SENSORS_NUMBER);
		UARTputchW(input, MAX_RULESETS);
		UARTputchW(input, MAX_SPECTRUMS);
		UARTputchW(input, MAX_RULES);
#else /* HAVE_LAYOUT */
		UARTputch(input, 1);
		UARTputch(input, 0);
#endif /* HAVE_LAYOUT */
	    break;
#ifdef HAVE_LAYOUT
	    case COMMAND_REQUEST_LAYOUT_RESET:
		UARTputch(input, COMMAND_RESPONSE_LAYOUT_RESET);
		LAYOUTreset();
		SYNCdrop();
	    break;
	    case COMMAND_REQUEST_LAYOUT_USED_DETECTORS:
		tmp = UARTgetchW(input);
		if (tmp == -1) {
		    UARTputch(input, COMMAND_RESPONSE_RETRY);
		    UARTputch(input, COMMAND_ERROR_UART);
		    break;
		}
		
		if (tmp > LADCchannels) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, COMMAND_ERROR_INVALID_PARAMETER);
		    break;
		}

		err = LAYOUTsetDetectors(tmp);
		
		if (err) {
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, err);
		} else UARTputch(input, COMMAND_RESPONSE_LAYOUT_USED_DETECTORS);
	    break;
	    case COMMAND_REQUEST_LAYOUT_REGISTER_RULESET:
		err = LAYOUTregisterRuleSet(input);
		if (err) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, err);
		} else UARTputch(input, COMMAND_REQUEST_LAYOUT_REGISTER_RULESET);
	    break;
	    case COMMAND_REQUEST_LAYOUT_REGISTER_RULE:
		err = LAYOUTregisterRule(input);
		if (err) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, err);
		} else UARTputch(input, COMMAND_REQUEST_LAYOUT_REGISTER_RULE);
	    break;
	    case COMMAND_REQUEST_LAYOUT_REGISTER_RULESPEC:
		err = LAYOUTregisterRuleSpec(input);
		if (err) {
		    UARTskip(input);
		    UARTputch(input, COMMAND_RESPONSE_ERROR);
		    UARTputch(input, err);
		} else UARTputch(input, COMMAND_REQUEST_LAYOUT_REGISTER_RULESPEC);
	    break;
#endif /* HAVE_LAYOUT */
	    case -1:
		UARTputch(input, COMMAND_RESPONSE_RETRY);
		UARTputch(input, COMMAND_ERROR_UART);
	    break;
	    default:
		UARTskip(input);
		UARTputch(input, COMMAND_RESPONSE_ERROR);
		UARTputch(input, COMMAND_ERROR_INVALID_REQUEST);
		UARTputchW(input, cmd);
	}
	
	return 0;

}
