#ifndef _CONFIG_H
#define _CONFIG_H

#include <lpc214x.h>

/** 
 *   Frequency setup. Should couply with values specified in startup.S:
 *   PLLCFG_Val and VPBDIV_Val variables.
**/

#define PROTOCOL_VERSION		7
#define VERSION 			4
#define COMMUNICATION_DWORD

#define TYPE_NAMT			1
#define TYPE_SNT			2
#define TYPE_ANALYZER			3

#if defined(BUILD_SNT)
# define TYPE				TYPE_SNT
# define MAX_DETECTORS_NUMBER		4

# define BUFFER_SIZE 			768
# define EVENT_BUFFER_SIZE		768
# define SPECTRUMchannels		LADCchannels

# undef PLAIN
# undef HAVE_LAYOUT
#elif defined(BUILD_ANALYZER)
# define TYPE				TYPE_ANALYZER
# define MAX_DETECTORS_NUMBER		16
# define MAX_RULESETS			64	/* RuleSets limit */
# define MAX_RULES			16	/* Rules per RuleSet */
# define MAX_SPECTRUMS			8	/* RS associated spectrums limit */

# define BUFFER_SIZE 			256
# define EVENT_BUFFER_SIZE		256
# define SPECTRUMchannels		(MAX_DETECTORS_NUMBER + MAX_SPECTRUMS)

# define PLAIN
# define HAVE_LAYOUT
#else /* Build Type */
# define TYPE				TYPE_SNT
# define MAX_DETECTORS_NUMBER		8
# define BUFFER_SIZE 			255
# define EVENT_BUFFER_SIZE		512
# define SPECTRUMchannels		LADCchannels

# undef PLAIN
# undef HAVE_LAYOUT
#endif /* Build Type */

#define DEFAULT_SOFTWARE_THRESHOLD 	0

#ifdef PLAIN
# define MAX_SENSORS_NUMBER		MAX_DETECTORS_NUMBER
#else /* PLAIN */
# define MAX_SENSORS_NUMBER		(2 * MAX_DETECTORS_NUMBER)
#endif /* PLAIN */


//#define REGISTER_EVENTS_MANY_MANY
#define REGISTER_EVENTS_MANY_MANY_THRESHOLDED
#define THRESHOLD_EVENTS_MANY_MANY		25
#define REGISTER_EVENTS_MANY4_MANY4
#define REGISTER_EVENTS_HORIZONTAL_LAYER1
#define TRESHOLD_EVENTS_HORIZONTAL_LAYER1	25
#define REGISTER_EVENTS_HORIZONTAL_LAYER2
#define TRESHOLD_EVENTS_HORIZONTAL_LAYER2	25

#ifdef REGISTER_EVENTS_MANY_MANY
# undef REGISTER_EVENTS_MANY4_MANY4
# undef REGISTER_EVENTS_MANY_MANY_THRESHOLDED
#endif /* REGISTER_EVENTS_MANY_MANY */

#define Fosc		12000000
#define Fcclk		(Fosc * 5)
#define Fcco		(Fcclk * 4)
#define Fpclk		(Fcclk / 1) * 1  	/* (Fcclk / VPDIV) * 1 */

#define RS485_SPEED	9600 /* baud for HV control */
#define UART_SPEED	115200 /* baud */
#define UART_TIMEOUT 	60 /* ms */

#define PC_CONNECTION 		1
#define CONTROL_CONNECTION 	2
#define MAX_CONTROL_SIZE	64

#define DEFAULT_COUNTING_DURATION 10000	/* us */
#define DEAD_COUNTING_DURATION 10 /* Dead time. Only processing performed */
#define COUNTING_DIVIDER 4 /* Dead time. Read requests are not accpeted */
#define DEFAULT_SYNC_DIVIDER 500 /* Synchronization adjustement step */
#define REQUIRED_INITIALIZATION_TIME 100 /* ms */

#ifdef COMMUNICATION_DWORD
#define uart_function(v) UARTputchDW(input, v)
typedef unsigned long DataType;
#else
#define uart_function(v) UARTputchW(input, v)
typedef unsigned short int DataType;
#endif /* COMMUNICATION_DWORD */

#define bit(i) (1<<i)
typedef unsigned char uint8_t;
typedef unsigned short int uint16_t;
typedef unsigned long int uint32_t;

#endif /* _CONFIG_H */
