#include <stdio.h>
#include <stdlib.h>

#define _DETECTOR_C 

#include "detector.h"
#include "spectrum.h"
#include "serial.h"
#include "time.h"
#include "sync.h"
#include "i2c.h"
#include "defs.h"
#include "error.h"
#include "ladc.h"
#include "snt.h"
#include "event_types.h"

DetectorDataS dd11, dd12;
DetectorData dd1, ddp1, ddt1;

uint32_t handled_events = 0;
unsigned char store_events = 0;
unsigned short mmevents = 0;
DetectorEventS mmevent[EVENT_BUFFER_SIZE + 1];

uint16_t thresholds[MAX_DETECTORS_NUMBER*2];

int ddInitDetector(DetectorData dd, unsigned char detectors_number) {
    dd->detectors_number=detectors_number;
    ddResetData(dd);
    return 0;
}

void ddFreeDetector(DetectorData dd) {
}

void ddResetData(DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number, detectors_number2;
    DataType *detectors1, *detectors2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    coincidences=dd->coincidences;
    
    for (i=0;i<detectors_number;i++) {
	detectors1[i]=0;
	detectors2[i]=0;
    }
    
    for (i=0;i<detectors_number2;i++) {
	coincidences[i]=0;
    }
    
    dd->many_many = 0;
    dd->many_zero = 0;
    dd->zero_many = 0;
    dd->zero_one = 0;
    dd->one_zero = 0;
    dd->many_one = 0;
    dd->one_many = 0;
    dd->total = 0;
}


int ddSendData(int input, DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number;
    DataType *detectors1, *detectors2;

    detectors_number = dd->detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    
    UARTputchW(input, 6+2*detectors_number*sizeof(DataType));
    UARTputchDW(input, iter-1);
    UARTputchW(input, counting_duration + rest_counting_duration + sync_flag);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors1[i]);

    for (i=0;i<detectors_number;i++)
	uart_function(detectors2[i]);
    
    return 0;
}


int ddSendCurrentData(int input, DetectorData dd) {
    unsigned char i;
    unsigned char detectors_number;
    DataType *detectors1, *detectors2;

    detectors_number = dd->detectors_number;
    detectors1=dd->detectors1;
    detectors2=dd->detectors2;
    
    UARTputchW(input, 6+2*detectors_number*sizeof(DataType));
    UARTputchDW(input, iter);
    if (rest_flag) UARTputchW(input, counting_duration + rest_counting_duration + sync_flag - mls);
    else UARTputchW(input, counting_duration - mls);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors1[i]);

    for (i=0;i<detectors_number;i++)
	uart_function(detectors2[i]);
    
    return 0;
}

int ddSendCoincidenceData(int input, DetectorData dd) {
    unsigned char detectors_number;
    unsigned short i, detectors_number2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    coincidences=dd->coincidences;

    UARTputchW(input, 6+detectors_number2*sizeof(DataType)+8*sizeof(DataType));
    UARTputchDW(input, iter-1);
    UARTputchW(input, counting_duration + rest_counting_duration + sync_flag);

    for (i=0;i<detectors_number2;i++)
	uart_function(coincidences[i]);
    
    uart_function(dd->many_many);
    uart_function(dd->many_zero);
    uart_function(dd->zero_many);
    uart_function(dd->zero_one);
    uart_function(dd->one_zero);
    uart_function(dd->many_one);
    uart_function(dd->one_many);
    uart_function(dd->total);
    
    return 0;
}

int ddSendCurrentCoincidenceData(int input, DetectorData dd) {
    unsigned char detectors_number;
    unsigned short i, detectors_number2;
    DataType *coincidences;

    detectors_number = dd->detectors_number;
    detectors_number2 = detectors_number*detectors_number;
    coincidences=dd->coincidences;

    UARTputchW(input, 6+detectors_number2*sizeof(DataType)+8*sizeof(DataType));
    UARTputchDW(input, iter);
    if (rest_flag) UARTputchW(input, counting_duration + rest_counting_duration + sync_flag - mls);
    else UARTputchW(input, counting_duration - mls);

    for (i=0;i<detectors_number2;i++)
	uart_function(coincidences[i]);
    
    uart_function(dd->many_many);
    uart_function(dd->many_zero);
    uart_function(dd->zero_many);
    uart_function(dd->zero_one);
    uart_function(dd->one_zero);
    uart_function(dd->many_one);
    uart_function(dd->one_many);
    uart_function(dd->total);
    
    return 0;
}

int ddSendSpectrum(int input, DetectorData dd) {
    unsigned char i,j;
    unsigned char detectors_number;

    detectors_number = 2*dd->detectors_number;
    
    UARTputchW(input, detectors_number*127*sizeof(DataType));

    for (i=0;i<detectors_number;i++) {
	Process1Events(-1);
	for (j=1;j<128;j++) uart_function(Spectrum[i][j]);
    }
    
    return 0;
}

int ddSendEvents(int input, DetectorData dd) {
    int i, j, events;
    unsigned char detectors_number;
    
    detectors_number = 2*dd->detectors_number;

    events = mmevents;
    mmevents = 0;
    
    UARTputchW(input, (8 + detectors_number)*events);
    for (i=0;i<events;i++) {
	if ((i&7)==7) Process1Events(i-mmevents);
	UARTputchDW(input, mmevent[i].iter);
	UARTputchDW(input, mmevent[i].us);
	for (j=0;j<detectors_number;j++) UARTputch(input, mmevent[i].detectors[j]);
    }
    
    return 0;
}

int EVENTregister(DetectorData dd, unsigned long event_time, unsigned char *data) {
	int i;
	
	if (mmevents<EVENT_BUFFER_SIZE) {
	    for (i=0;i<2 * dd->detectors_number;i++)
		mmevent[mmevents].detectors[i] = data[i];

	    mmevent[mmevents].us = event_time;

	    if (buffer1_new_iteration) {
		mmevent[mmevents].iter = iter-1;
	    } else {
		mmevent[mmevents].iter = iter;
	    }
	    mmevents++;

	    return 1;
	} else {
	    ERRORregister(MCU_ERROR_EVENT_BUFFER_FULL);
	    
	    return 0;
	}
}

int ddHandleEvent(DetectorData dd, unsigned long event_time, unsigned char *data) {
    int i;
    int res = 0;
    signed char x=-1, y=-1;

    unsigned char data1 = 0, data2 = 0;

#ifdef REGISTER_EVENTS_MANY4_MANY4
    unsigned char n1 = 0, n2 = 0;
#endif /* REGISTER_EVENTS_MANY4_MANY4 */
#if defined(REGISTER_EVENTS_HORIZONTAL_LAYER1) || defined(REGISTER_EVENTS_HORIZONTAL_LAYER2) || defined(REGISTER_EVENTS_MANY4_MANY4) || defined(REGISTER_EVENTS_MANY_MANY_THRESHOLDED)
    unsigned char above = 0;
#endif /* ALL_EVENTS */

    unsigned char detectors_number = dd->detectors_number;

    for (i=0;i<LADCchannels;i++) {
	if (data[i]) {
	    Spectrum[i][data[i]]++;
	    if (Spectrum[i][data[i]] > SpectrumMaximum[i])
		SpectrumMaximum[i] = Spectrum[i][data[i]];
	}
    }
	
    for (i=0;i<detectors_number;i++) {
	if (data[i]>thresholds[i]) data1|=bit(i);
	if (data[i+detectors_number]>thresholds[i+detectors_number]) data2|=bit(i);
    }
    
    for (i=detectors_number-1;i>=0;i--) {	
	if (data1&bit(i)) {
	    dd->detectors1[i]++;
	    if (data1==bit(i)) x=i+1;
	    else {
		x=0;
#ifdef REGISTER_EVENTS_MANY4_MANY4
		n1++;
#endif /* REGISTER_EVENTS_MANY4_MANY4 */
	    }
	}
	if (data2&bit(i)) {
	    dd->detectors2[i]++;
	    if (data2==bit(i)) y=i+1;
	    else {
		y=0;
#ifdef REGISTER_EVENTS_MANY4_MANY4
		n2++;
#endif /* REGISTER_EVENTS_MANY4_MANY4 */
	    }
	}
    }

	/* We shouldn't do it only if nothing proccesed since new_iter and 
	Event1 flag. So we skip the check: User should consider problems under
	overload */
    if ((buffer1_new_iteration)&&((buffer1_start+1) == buffer1_new_iteration)/*&&(!Event1)*/) {
	buffer1_new_iteration = 0;
    }
    
    if ((x>0)&&(y>0)) dd->coincidences[(x-1)*detectors_number+(y-1)]++;
    else if ((!x)&&(!y)) {
	dd->many_many++;
#if defined(REGISTER_EVENTS_MANY_MANY) || defined(REGISTER_EVENTS_MANY4_MANY4) || defined(REGISTER_EVENTS_MANY_MANY_THRESHOLDED)
	if (store_events) {

# if defined(REGISTER_EVENTS_MANY4_MANY4) || defined(REGISTER_EVENTS_MANY_MANY_THRESHOLDED)

#  ifdef REGISTER_EVENTS_MANY4_MANY4
	    if ((n1 > 3)&&(n2 > 3)) above = 1;
	    else {
#  endif /* REGISTER_EVENTS_MANY4_MANY4 */
#  ifdef REGISTER_EVENTS_MANY_MANY_THRESHOLDED
		n1 = 0; n2 = 0;
		for (i=detectors_number-1;i>=0;i--) {
		    if (data[i] > THRESHOLD_EVENTS_MANY_MANY) n1++;
		    if (data[i+detectors_number] > THRESHOLD_EVENTS_MANY_MANY) n2++;
		}
		if ((n1 > 1)&&(n2 > 1)) above = 1;
#  endif /* REGISTER_EVENTS_MANY_MANY_THRESHOLDED */
#  ifdef REGISTER_EVENTS_MANY4_MANY4
	    }
#  endif /* REGISTER_EVENTS_MANY4_MANY4 */
	    if (above) {
# endif /* REGISTER_EVENTS_MANY4_MANY4 || REGISTER_EVENTS_MANY_MANY_THRESHOLDED */
		res = EVENTregister(dd, event_time, data);
# if defined(REGISTER_EVENTS_MANY4_MANY4) || defined(REGISTER_EVENTS_MANY_MANY_THRESHOLDED)
    	    }
# endif /* REGISTER_EVENTS_MANY4_MANY4 || REGISTER_EVENTS_MANY_MANY_THRESHOLDED */
	}
#endif /* REGISTER_EVENTS_MANY_MANY || REGISTER_EVENTS_MANY4_MANY4 || defined(REGISTER_EVENTS_MANY_MANY_THRESHOLDED) */
    } else if ((!x)&&(y<0)) {
	dd->many_zero++;
#ifdef REGISTER_EVENTS_HORIZONTAL_LAYER1
	if (store_events) {
	    for (i=0;i<detectors_number;i++) {
		if ((data[i]>TRESHOLD_EVENTS_HORIZONTAL_LAYER1)&&((++above)==2)) {
		    res = EVENTregister(dd, event_time, data);
		    break;
		}
	    }
	}
#endif /* REGISTER_EVENTS_HORIZONTAL_LAYER1 */
    } else if ((x<0)&&(!y)) {
	dd->zero_many++;

#ifdef REGISTER_EVENTS_HORIZONTAL_LAYER2
	if (store_events) {
	    for (i=0;i<detectors_number;i++) {
		if ((data[i+detectors_number]>TRESHOLD_EVENTS_HORIZONTAL_LAYER1)&&((++above)==2)) {
		    res = EVENTregister(dd, event_time, data);
		    break;
		}
	    }
	}
#endif /* REGISTER_EVENTS_HORIZONTAL_LAYER2 */

    } else if ((x<0)&&(y>0)) dd->zero_one++;
    else if ((x>0)&&(y<0)) dd->one_zero++;
    else if ((!x)&&(y>0)) dd->many_one++;
    else if ((x>0)&&(!y)) dd->one_many++;
    
    dd->total++;    
    
    return res;
}


void DETECTORsetup() {
    int i;

    handled_events = 0;
    
#ifdef REGISTER_EVENTS_MANY_MANY
    handled_events |= EVENT_TYPE_MANY_MANY;
#endif /* REGISTER_EVENTS_MANY_MANY */

#ifdef REGISTER_EVENTS_MANY4_MANY4
    handled_events |= EVENT_TYPE_MANY4_MANY4;
#endif /* REGISTER_EVENTS_MANY4_MANY4 */

#ifdef REGISTER_EVENTS_HORIZONTAL_LAYER1
    handled_events |= EVENT_TYPE_HORIZONTAL_LAYER1;
#endif /* REGISTER_EVENTS_HORIZONTAL_LAYER1 */

#ifdef REGISTER_EVENTS_HORIZONTAL_LAYER2
    handled_events |= EVENT_TYPE_HORIZONTAL_LAYER2;
#endif /* REGISTER_EVENTS_HORIZONTAL_LAYER2 */

#ifdef REGISTER_EVENTS_MANY_MANY_THRESHOLDED
    handled_events |= EVENT_TYPE_MANY_MANY_THRESHOLDED;
#endif /* REGISTER_EVENTS_MANY_MANY_THRESHOLDED */

    for (i=0;i<MAX_DETECTORS_NUMBER*2;i++) thresholds[i] = DEFAULT_SOFTWARE_THRESHOLD;
    
    ddInitDetector(&dd11,MAX_DETECTORS_NUMBER);
    ddInitDetector(&dd12,MAX_DETECTORS_NUMBER);
    dd1=&dd11;ddp1=&dd12;
    
    mmevents = 0;
}
