#include <stdio.h>
#include <stdlib.h>

#define _DETECTOR_C 

#include "detector_plain.h"
#include "layout.h"
#include "spectrum.h"
#include "serial.h"
#include "time.h"
#include "sync.h"
#include "i2c.h"
#include "defs.h"
#include "error.h"
#include "ladc.h"
#include "snt.h"
#include "event_types.h"

DetectorDataS dd11, dd12;
DetectorData dd1, ddp1, ddt1;

uint32_t handled_events = 0;
unsigned char store_events = 0;
unsigned short mmevents = 0;
DetectorEventS mmevent[EVENT_BUFFER_SIZE + 1];

uint16_t thresholds[MAX_DETECTORS_NUMBER];

int ddInitDetector(DetectorData dd, unsigned char dn) {
    ddResetData(dd);
    return 0;
}

void ddFreeDetector(DetectorData dd) {
}

void ddResetData(DetectorData dd) {
    unsigned short i;

    dd->total = 0;    
    for (i=0;i<detectors_number;i++) dd->detectors[i] = 0;
    for (i=0;i<coincidences_number;i++) dd->coincidences[i] = 0;
}


int ddSendData(int input, DetectorData dd) {
    unsigned short i;
    DataType *detectors;

    detectors=dd->detectors;
    
    UARTputchW(input, 6+detectors_number*sizeof(DataType));
    UARTputchDW(input, iter-1);
    UARTputchW(input, counting_duration + rest_counting_duration + sync_flag);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors[i]);

    return 0;
}


int ddSendCurrentData(int input, DetectorData dd) {
    unsigned short i;
    DataType *detectors;

    detectors=dd->detectors;
    
    UARTputchW(input, 6+detectors_number*sizeof(DataType));
    UARTputchDW(input, iter);
    if (rest_flag) UARTputchW(input, counting_duration + rest_counting_duration + sync_flag - mls);
    else UARTputchW(input, counting_duration - mls);
    
    for (i=0;i<detectors_number;i++)
	uart_function(detectors[i]);

    return 0;
}

int ddSendCoincidenceData(int input, DetectorData dd) {
    unsigned short i;

    UARTputchW(input, 6+coincidences_number*sizeof(DataType));
    UARTputchDW(input, iter-1);
    UARTputchW(input, counting_duration + rest_counting_duration + sync_flag);

    for (i=0;i<coincidences_number;i++)
	uart_function(dd->coincidences[i]);
    
    return 0;
}

int ddSendCurrentCoincidenceData(int input, DetectorData dd) {
    unsigned short i;

    UARTputchW(input, 6+coincidences_number*sizeof(DataType));
    UARTputchDW(input, iter);
    if (rest_flag) UARTputchW(input, counting_duration + rest_counting_duration + sync_flag - mls);
    else UARTputchW(input, counting_duration - mls);

    for (i=0;i<coincidences_number;i++)
	uart_function(dd->coincidences[i]);
    
    return 0;
}

int ddSendSpectrum(int input, DetectorData dd) {
    unsigned short i,j;
    
    UARTputchW(input, spectrums_number*127*sizeof(DataType));

    for (i=0;i<spectrums_number;i++) {
	Process1Events(-1);
	for (j=1;j<128;j++) uart_function(Spectrum[i][j]);
    }
    
    return 0;
}

int ddSendEvents(int input, DetectorData dd) {
    int i, j, events;

    events = mmevents;
    mmevents = 0;
    
    UARTputchW(input, (8 + detectors_number)*events);
    for (i=0;i<events;i++) {
	if ((i&7)==7) Process1Events(i - mmevents);
	UARTputchDW(input, mmevent[i].iter);
	UARTputchDW(input, mmevent[i].us);
	for (j=0;j<detectors_number;j++) UARTputch(input, mmevent[i].detectors[j]);
    }
    
    return 0;
}

int EVENTregister(DetectorData dd, unsigned long event_time, unsigned char *data) {
    int i;
	
    if (mmevents<EVENT_BUFFER_SIZE) {
	for (i=0;i<detectors_number;i++)
	    mmevent[mmevents].detectors[i] = data[i];

	mmevent[mmevents].us = event_time;

	if (buffer1_new_iteration) {
	    mmevent[mmevents].iter = iter-1;
	} else {
	    mmevent[mmevents].iter = iter;
	}
	mmevents++;

	return 1;
    } else {
	ERRORregister(MCU_ERROR_EVENT_BUFFER_FULL);
	    
	return 0;
    }
}

int ddHandleEvent(DetectorData dd, unsigned long event_time, unsigned char *data) {
    short int i, j, k;
    short int c = 0; /* current coincidence */

    
    int eventres = 0;
    unsigned char res, ruleres[rulesets], ruleok[rulesets];
    short int source, threshold;

	/* We shouldn't do it only if nothing proccesed since new_iter and 
	Event1 flag. So we skip the check: User should consider problems under
	overload */
    if ((buffer1_new_iteration)&&((buffer1_start+1) == buffer1_new_iteration)/*&&(!Event1)*/) {
	buffer1_new_iteration = 0;
    }

    
    
    for (i=0;i<detectors_number;i++) {
	if (data[i]) {
	    SPECTRUMregister(i, data[i]);
	    if (data[i] > thresholds[i]) dd->detectors[i]++;
	}
    }
    
    
    for (j=0;j<rulesets;j++) {
	for (res = 0, k = (ruleset[j].config>>2) - 1;k>=0;k--) {
	    source = ruleset[j].rule[k].source;
	    threshold = ruleset[j].rule[k].threshold;
	    
	    if (source > 0) {
		if (threshold < 0) {
		    if (threshold == -127)  {
			if (data[source-1] <= thresholds[source-1]) res++;
		    } else {
			if (data[source-1] < (-threshold)) res++;
		    }
		} else {
		    if (threshold) {
			if (data[source-1] >= threshold) res++;
		    } else {
			if (data[source-1] > thresholds[source-1]) res++;
		    }
		}
	    } else if (source < 0) {
		if (threshold < 0) {
		    if (threshold == -127) {
			if (!ruleok[-1-source]) res++;
		    } else {
			if (ruleres[-1-source] < (-threshold)) res++;
		    }
		} else {
		    if (threshold) {
			if (ruleres[-1-source] >= threshold) res++;
		    } else {
			if (ruleok[-1-source]) res++;
		    }
		}
	    }
	}

	ruleres[j] = res;
	
	if (ruleset[j].success > 0) {
	    if (res >= ruleset[j].success) {
		if (ruleset[j].config&1) {
		    dd->coincidences[c++]++;
		}
		if ((!eventres)&&(ruleset[j].config&2)) {
		    eventres = EVENTregister(dd, event_time, data);
		}
		
		ruleok[j] = 1;
	    } else {
		if (ruleset[j].config&1) c++;

		ruleok[j] = 0;
	    }
	} else {
	    if (res < (-ruleset[j].success)) {
		if (ruleset[j].config&1) {
		    dd->coincidences[c++]++;
		}
		if ((!eventres)&&(ruleset[j].config&2)) {
		    eventres = EVENTregister(dd, event_time, data);
		}
		
		ruleok[j] = 1;
	    } else {
		if (ruleset[j].config&1) c++;

		ruleok[j] = 0;
	    }
	}
    }


    for (j=0;j<rulespecs;j++) {
	if (ruleok[rulespec[j].ruleset]) {
	    SPECTRUMregister(j + detectors_number, data[rulespec[j].sensor]);
	}
    }
	    

    dd->total++;    
    
    return eventres;
}

void DETECTORsetup() {
    int i;

    handled_events = 0xFF;

    for (i=0;i<MAX_DETECTORS_NUMBER;i++) thresholds[i] = DEFAULT_SOFTWARE_THRESHOLD;
    
    ddInitDetector(&dd11,MAX_DETECTORS_NUMBER);
    ddInitDetector(&dd12,MAX_DETECTORS_NUMBER);
    dd1=&dd11;ddp1=&dd12;
    
    mmevents = 0;
}
