#include "time.h"

#include "defs.h"
#include "i2c.h"

// I2C0ONSET bits
#define I2EN	0x40
#define STA		0x20
#define STO		0x10
#define SI		0x08
#define AA		0x04

// I2C0ONCLR bits
#define I2ENC	0x40
#define STAC	0x20
#define SIC		0x08
#define AAC		0x04

// I2C0STAT Codes
#define START			0x08
#define RPT_START		0x10
#define MT_SLA_ACK		0x18
#define MT_DATA_ACK		0x28
#define MR_SLA_ACK		0x40
#define MR_DATA_ACK		0x50
#define MR_DATA_NACK	0x58

void InitI2C()
{
  PINSEL0 |= 0x50;

  I2C0CONCLR = 0xFF; 
  I2C0SCLL = 75;		//400 kHz for a VPBDIV  = 1
  I2C0SCLH = 75;
  I2C0CONSET = I2EN;
}

void SendSlaveAddress(U8 sla)
{
  I2C0CONSET = STA;	
  if (sla & 0x01) {		// read request
    I2C0CONCLR = SIC;
	while(I2C0STAT != RPT_START);
    I2C0DAT = sla;
    I2C0CONCLR = SIC | STAC;
    while(I2C0STAT != MR_SLA_ACK);
  }
  else {				// write request
	while (I2C0STAT != START);
    I2C0DAT = sla;
    I2C0CONCLR = SIC | STAC;
    while(I2C0STAT != MT_SLA_ACK);
  }
}

U8 ReadI2C()
{
  I2C0CONCLR = SIC;
  while((I2C0STAT != MR_DATA_ACK) && !(I2C0CONSET & SI));

  return I2C0DAT;	
}

void WriteI2C(U8 data)
{
  I2C0DAT = data;
  I2C0CONCLR = SIC;
  while((I2C0STAT != MT_DATA_ACK) && !(I2C0CONSET & SI));
}

void SendSTOP()
{
  I2C0CONCLR = SIC;
  I2C0CONSET = STO;
} 

#define EEPROM_Addr	0xA0

U8 ReadEEPROM(U16 addr)
{
  U8 tmp;

  SendSlaveAddress(EEPROM_Addr);
  WriteI2C(addr);
	
  SendSlaveAddress(EEPROM_Addr | (0x0E & (U8)(addr >> 7)) | 1);		

  tmp = ReadI2C();

  SendSTOP();
  return tmp;
}

void WriteEEPROM(U16 addr, U8 data)
{
  VICIntEnClr = BODbit;   // Disable BOD interruption	
  SendSlaveAddress(EEPROM_Addr | (0x0E & (U8)(addr >> 7)));
  WriteI2C(addr);

  WriteI2C(data);
	
  SendSTOP();
  VICIntEnable   = BODbit;  // Enable BOD interruption

  TIMEdelay(25000);
}

