#define _LAYOUT_C

#include "layout.h"
#include "serial.h"
#include "commands.h"

#ifdef HAVE_LAYOUT
unsigned short rulesets;
RuleSetS ruleset[MAX_RULES];
unsigned short rulespecs;
RuleSpectrumS rulespec[MAX_SPECTRUMS];

unsigned short detectors_number = 0;
unsigned short coincidences_number = 0;
unsigned short spectrums_number = 0;
#endif /* HAVE_LAYOUT */


void LAYOUTsetup() {
    LAYOUTreset();
}

void LAYOUTreset() {
#ifdef HAVE_LAYOUT
    rulesets = 0;
    rulespecs = 0;
    
    detectors_number = MAX_SENSORS_NUMBER;
    coincidences_number = 0;
    spectrums_number = detectors_number;
#endif /* HAVE_LAYOUT */
}

int LAYOUTsetDetectors(short detectors) {
#ifdef HAVE_LAYOUT
    spectrums_number += (detectors - detectors_number);
    detectors_number = detectors;

    return 0;
#else
    return COMMAND_ERROR_NOT_SUPPORTED;
#endif /* HAVE_LAYOUT */
}

int LAYOUTregisterRuleSet(int input) {
#ifdef HAVE_LAYOUT
    int size;
    int flags, success;

    size = UARTgetchW(input);
    if (size < 0) return COMMAND_ERROR_UART;
    if (size != 4) return COMMAND_ERROR_INVALID_REQUEST;

    flags = UARTgetchW(input);
    if (flags < 0) return COMMAND_ERROR_UART;
    
    success = UARTgetchW(input);
    if (success < 0) return COMMAND_ERROR_UART;
    

    success = (short int)success;
    
    if (flags&(~3)) return COMMAND_ERROR_INVALID_PARAMETER;

    if (rulesets == MAX_RULESETS) return COMMAND_ERROR_NOT_SUPPORTED;

    if (flags&1) coincidences_number++;
    ruleset[rulesets].config = flags;
    ruleset[rulesets].success = success;
    rulesets++;    

    return 0;
#else
    return COMMAND_ERROR_NOT_SUPPORTED;
#endif /* HAVE_LAYOUT */
}

int LAYOUTregisterRule(int input) {
#ifdef HAVE_LAYOUT
    int size;
    int source, threshold;
    int rules;

    size = UARTgetchW(input);
    if (size < 0) return COMMAND_ERROR_UART;
    if (size != 4) return COMMAND_ERROR_INVALID_REQUEST;

    
    source = UARTgetchW(input);
    if (source < 0) return COMMAND_ERROR_UART;
    
    threshold = UARTgetchW(input);
    if (threshold < 0) return COMMAND_ERROR_UART;
    
    source = (short int)source;
    threshold = (short int)threshold;
    
    if (((source>0)&&(source>detectors_number))||((source<0)&&((-source)>=rulesets))) return COMMAND_ERROR_INVALID_PARAMETER;

    rules = ruleset[rulesets - 1].config >> 2;
    if (rules == MAX_RULES) return COMMAND_ERROR_NOT_SUPPORTED;
    
    ruleset[rulesets - 1].config = ((rules + 1)<<2)|(ruleset[rulesets - 1].config&3);
    ruleset[rulesets - 1].rule[rules].source = source;
    ruleset[rulesets - 1].rule[rules].threshold = threshold;

    return 0;
#else
    return COMMAND_ERROR_NOT_SUPPORTED;
#endif /* HAVE_LAYOUT */
    
}

int LAYOUTregisterRuleSpec(int input) {
#ifdef HAVE_LAYOUT
    int size;
    int ruleset, sensor;

    size = UARTgetchW(input);
    if (size < 0) return COMMAND_ERROR_UART;
    if (size != 4) return COMMAND_ERROR_INVALID_REQUEST;
    
    ruleset = UARTgetchW(input);
    if (ruleset < 0) return COMMAND_ERROR_UART;
    
    sensor = UARTgetchW(input);
    if (sensor < 0) return COMMAND_ERROR_UART;
    
    if ((!ruleset)||(!sensor)||(rulesets > rulesets)||(sensor > detectors_number)) return COMMAND_ERROR_INVALID_PARAMETER;
    if (rulespecs == MAX_SPECTRUMS) return COMMAND_ERROR_NOT_SUPPORTED;

    spectrums_number++;    
    rulespec[rulespecs].ruleset = ruleset - 1;
    rulespec[rulespecs].sensor = sensor - 1;
    rulespecs++;

    return 0;
#else
    return COMMAND_ERROR_NOT_SUPPORTED;
#endif /* HAVE_LAYOUT */
}

