#include "setup.h"
#include "time.h"
#include "sync.h"

#include "serial.h"

#define CR     0x0D

void UARTsetup(unsigned long int baud1, unsigned long int baud2) {
    unsigned short int Fdiv;
    unsigned int mval = 7, daval = 0; 	/* MulVal & DivAddVal: disabled */
    
    PINSEL0 |= 0x00050005;		// Enable RxD1 and TxD1

    Fdiv = ((unsigned long int)Fpclk) / (16 *  baud1);

    /*
	00000011
	      11 - Word length (00 - 5 bit, 01 - 6 bit, 10 - 7 bit, 11 - 8 bit)
	     0   - Stop bits (0 - 1 stop bit, 1 - 2 stop bits)
	    0	 - Enable parity (if 1)
	  00	 - Parity (00 - odd, 01 - even, 10 - forced 1, forced 0)
	 0	 - Force TXD to 0 (if 1)
        0	 - DLAB (Enable access to Divisor Latches)
    */
    
    U0LCR = 0x83;                   /* DLAB = 1: 8 bits, no Parity, 1 Stop bit  */
    U0FDR = daval + (mval*16);
    U0DLM = Fdiv / 256;							
    U0DLL = Fdiv % 256;
    U0LCR = 0x03;                   /* DLAB = 0                                 */

    Fdiv = ((unsigned long int)Fpclk) / (16 *  baud2);

    U1LCR = 0x83;                   /* DLAB = 1: 8 bits, no Parity, 1 Stop bit  */
    U1FDR = daval + (mval*16);
    U1DLM = Fdiv / 256;							
    U1DLL = Fdiv % 256;
    U1LCR = 0x03;                   /* DLAB = 0                                 */

    PINSEL1 |= 0x0C;			// For RS485
}


int sendchar (int ch)  {                 

  if (ch == '\n')  {
    while (!(U0LSR & 0x20));
    U0THR = CR;
  }
  while (!(U0LSR & 0x20));
  return (U0THR = ch);
}

int getkey (void)  {
  while (!(U0LSR & 0x01));

  return (U0RBR);
}


#define UART_ERROR_MASK 	0x80
#define UART_INPUT_MASK		0x01
#define UART_OUTPUT_MASK	0x20
#define RS485_OUTPUT_MASK	0x40


#define RREADY0	(U0LSR&UART_INPUT_MASK)
#define RREADY1	(U1LSR&UART_INPUT_MASK)
#define RREADY(i)  ((i-1)?RREADY1:RREADY0)

#define WREADY0 (U0LSR&UART_OUTPUT_MASK)
#define WREADY1 (U1LSR&UART_OUTPUT_MASK)
#define WREADY(i)  ((i-1)?WREADY1:WREADY0)

#define WREADY0_485 (U0LSR&RS485_OUTPUT_MASK)
#define WREADY1_485 (U1LSR&RS485_OUTPUT_MASK)
#define WREADY485(i)  ((i-1)?WREADY1_485:WREADY0_485)

#define ERROR0 (U0LSR&UART_ERROR_MASK)
#define ERROR1 (U1LSR&UART_ERROR_MASK)
#define ERROR(i)  ((i-1)?ERROR1:ERROR0)

#define skip(i) \
    tmp = mls; passed = 0; \
    while (passed < UART_TIMEOUT) { \
	while (RREADY##i) { \
	    res = (U##i##RBR); \
	} \
	if (mls != tmp) { \
	    tmp = mls; \
	    passed++; \
	} \
    } \
    res = ERROR(i)
    
#define getbyte(i, power) \
    while (!(RREADY##i)) \
    if (ERROR(i)) { \
	skip(i); \
	return -1; \
    } \
    res += (U##i##RBR)<<power;

#define trybyte(i, power) \
    while ((!(RREADY##i))&&(passed < UART_TIMEOUT)) { \
	if (mls != tmp) { \
	    tmp = mls; \
	    passed++; \
	} \
    } \
    if (ERROR(i)) { \
	skip(i); \
	return -1; \
    } else if (!RREADY##i) { \
	return -1; \
    } else if (ERROR(i)) { \
	return -1; \
    } \
    res += (U##i##RBR)<<power;
    
#define sendbyte(i, byte) \
    while (!(WREADY##i)); \
    U##i##THR = byte;
    
#define sendbyte485(i, byte) \
    while (!(WREADY##i##_485)); \
    enabled_interrupts = VICIntEnable; \
    VICIntEnClr        = enabled_interrupts; \
    U##i##THR = ch; \
    T1EMR = Bit(2); \
    T1MR2 = T1TC + 10*Fpclk/RS485_SPEED; \
    T1EMR |= Bit(8); \
    VICIntEnable = enabled_interrupts; \


int UARTreceived(int i) {
    if ((i&1)&&(RREADY0)) return 1;
    else if ((i&2)&&(RREADY1)) return 2;
    return 0;
}

int UART1received() {
    if (RREADY0) return 1;
    return 0;
}

int UART2received() {
    if (RREADY1) return 2;
    return 0;
}


int UARTgetch(int i) {
    int res = 0;
    unsigned long tmp = mls, passed = 0;

    if (i-1) {
	trybyte(1, 0);
    } else {
	trybyte(0, 0);
    }
    
    return res;
}

int UARTgetchW(int i) {
    int res = 0;
    unsigned long tmp = mls, passed = 0;

    if (i-1) {
	trybyte(1, 0);
	trybyte(1, 8);
    } else {
	trybyte(0, 0);
	trybyte(0, 8);
    }
    
    return res;
}

int UARTgetchDW(int i) {
    int res = 0;
    unsigned long tmp = mls, passed = 0;

    if (i-1) {
	trybyte(1, 0);
	trybyte(1, 8);
	trybyte(1, 16);
	trybyte(1, 24);
    } else {
	trybyte(0, 0);
	trybyte(0, 8);
	trybyte(0, 16);
	trybyte(0, 24);
    }
    
    return res;
}

#include "defs.h"
void UARTputch(int i, int ch) {
    uint32_t enabled_interrupts;
    
    if (i-1) {
	sendbyte485(1, (unsigned char)ch);
    } else {
	sendbyte(0, (unsigned char)ch);
    }
}

void UARTputchW(int i, int ch) {
    uint32_t enabled_interrupts;

    if (i-1) {
	sendbyte485(1, (unsigned char)ch);
	sendbyte485(1, (unsigned char)(ch>>8));
    } else {
	sendbyte(0, (unsigned char)ch);
	sendbyte(0, (unsigned char)(ch>>8));
    }
}

void UARTputchDW(int i, int ch) {
    uint32_t enabled_interrupts;

    if (i-1) {
	sendbyte485(1, (unsigned char)ch);
	sendbyte485(1, (unsigned char)(ch>>8));
	sendbyte485(1, (unsigned char)(ch>>16));
	sendbyte485(1, (unsigned char)(ch>>24));
    } else {
	sendbyte(0, (unsigned char)ch);
	sendbyte(0, (unsigned char)(ch>>8));
	sendbyte(0, (unsigned char)(ch>>16));
	sendbyte(0, (unsigned char)(ch>>24));
    }
}

int UARTputs(int i, const char *str) {
    while (*str) { UARTputch(i, *str++); }
    return 0;
}

int UARTgets(int i, char *str, int size) {
    int j, data;
    
    for (j=0;j<size;j++) {
	data = UARTgetch(i);
	if (data == -1) {
	    str[j] = 0;  
	    return 0;
	} else str[j] = data;
    }
    
    return -1;
}

int UARTrequest(int i, const char *req, char *resp, int size) {
    int err;
    UARTskip(i);

    err = UARTputs(i, req);
    if (err) return err;

    return UARTgets(i, resp, size);
}

int UARTskip(int i) {
    unsigned char read = 0;
    unsigned long tmp = mls, passed = 0;
	
    while(passed < UART_TIMEOUT) {
	while (UARTreceived(i)) {
	    if (UARTgetch(i)!=-1) read++;
	}

	if (mls != tmp) {
	    tmp = mls;
	    passed++;
	}
    }

    while (UARTreceived(i)) {
	if (UARTgetch(i)!=-1) read++;
    }

    return read;
}
