#include <math.h>
#include <irq.h>

#define _SPECTRUM_C

#include "defs.h"
#include "config.h"
#include "spectrum.h"

#define DACoffset   256
#define DACcursor   (DACoffset - 64)
#define DACmax      700.0
#define SpectrumPositionTime 230000   /* 3.8333ms */

unsigned char CursorFlag = 0;
unsigned char LogarithmFlag = 0;
unsigned char CursorPosition = 0;
unsigned char ChannelToShow = 0;

unsigned long Spectrum[SPECTRUMchannels][128];
unsigned long SpectrumMaximum[SPECTRUMchannels];

static void Spectrum_CaptureHandler(void) __irq;

void SPECTRUMsetup()
{
    SPECTRUMreset();
    
    PINSEL1 |= 0x00080000;	// set p0.25 to DAC output   
    
    T1PR = 0;
    T1MR0 = T1TC + SpectrumPositionTime;
    T1TCR = 1;
    T1MCR = 1;

    install_irq(TIMER1_INT, (void*)Spectrum_CaptureHandler);
}

static void Spectrum_CaptureHandler(void) {
    static unsigned char position;

    // Set the next capture value
    T1MR0  = T1TC + SpectrumPositionTime;

    if (position++ == 127) position = 0;

    if (position == 0) DACR = 0; // Synchronization pulse
    else if ((position == CursorPosition) & (!(++CursorFlag % 3)))
	DACR = DACcursor;
    else {
	if (!(LogarithmFlag & 1))
	    DACR = (U32)((DACmax/SpectrumMaximum[ChannelToShow])*Spectrum[ChannelToShow][position] + DACoffset) << 6;
	else
	    DACR = (U32)(34*log10(1.0 + Spectrum[ChannelToShow][position]) + DACoffset) << 6;
    }

    T1IR = 0x01;      // MR0 Interruption Flag;
    VICVectAddr = 0;
}

void SPECTRUMreset() {
    int i,j;
    
    for (i=0;i<SPECTRUMchannels;i++) {
	SpectrumMaximum[i] = 1;
	for (j=0;j<128;j++) Spectrum[i][j] = 0;
    }
}

void SPECTRUMregister(unsigned short spec, unsigned char code) {
    Spectrum[spec][code]++;
    if (Spectrum[spec][code] > SpectrumMaximum[spec]) SpectrumMaximum[spec] = Spectrum[spec][code];
}
