#include <stdio.h>

#include <dsconnectionserver.h>
#include <dsconnection.h>

#define BufferSize 16384

void Usage(char **argv) {
    printf("Usage: %s [options] server port\n", argv[0]);
    printf(" --tcp      Use TCP protocol (default)\n");
    printf(" --udp      Use UDP protocol\n");
    printf("\n");
    printf("Default protocols/ports: \n");
    printf(" 1st Mile   UDP/5001\n");
    printf(" XPort      TCP/10001\n");
    printf("\n");
    exit(0);
}

main(int argc, char *argv[]) {
    int err;
    size_t readed;
    char *buf[BufferSize+1];
    DSConnectionContext con;

    int i;
    int opos = 0;
        
    int protocol = 0, port = 0;
    char *server = NULL;

    if (strstr(argv[0], "xport")) {
	protocol = 1;
	port = 10001;
    } else if (strstr(argv[0], "mile")) {
	protocol = 0;
	port = 5001;
    }
    
    
    for (i=1;i<argc;i++) {
	if (!strncmp(argv[i], "--", 2)) {
	    if (!strcmp(argv[i], "--udp")) {
		protocol = 0;
	    } else if (!strcmp(argv[i], "--tcp")) {
		protocol = 1;
	    } else {
		Usage(argv);
	    }
	} else {
	    switch (opos++) {
		case 0:
		    server = argv[i];
		break;
		case 1:
		    port = atoi(argv[i]);
		break;
		default:
		    Usage(argv);
	    }
	}
    }
    
    if ((!server)||(!port)) Usage(argv);
    

    dsLibInit();

    con = dsConnectionCreateContext(NULL);
    if (con) {
	if (protocol) err = dsConnectionConnectTCPServer(con, NULL, server, port);
	else err = dsConnectionConnectUDPServer(con, NULL, server, port);
    } else err = DS_ERR_MEM;
    
    if (!err) {
	printf("Connected to %s:%u using %s protocol\n", server, port, protocol?"TCP":"UDP");
	while (!dsConnectionRead(con, buf, BufferSize, 0, &readed)) {
	    fwrite(buf, readed, 1, stdout);
	}
	dsConnectionDestroyContext(con);
    }
    
    dsLibFree();
}
