<?xml version="1.0"?> 
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">
 <xsl:output method="html" />

 <xsl:template match="@*">
  <xsl:if test="name(.)!='id' and name(.)!='type'">
    <xsl:if test="position()=1"><br/></xsl:if>
    <xsl:if test="position()!=1">, </xsl:if>
    <xsl:choose>
	<xsl:when test="name(.)='layer'">
	    Layer: <xsl:value-of select="."/>
	</xsl:when>
	<xsl:when test="name(.)='sensor'">
	    Sensor: <xsl:value-of select="."/>
	</xsl:when>
	<xsl:when test="name(.)='energy_min'">
	    Minimal Energy: <xsl:value-of select="."/>
	</xsl:when>
	<xsl:when test="name(.)='energy_max'">
	    Maximal Energy: <xsl:value-of select="."/>
	</xsl:when>
	<xsl:when test="name(.)='direction'">
	    Direction: <xsl:value-of select="."/>
	</xsl:when>
	<xsl:otherwise>
	    <xsl:value-of select="name(.)"/>: <xsl:value-of select="."/>
	</xsl:otherwise>
    </xsl:choose>
  </xsl:if>
 </xsl:template>

 <xsl:template name="direction_attribute">
    [<xsl:value-of select="@direction"/>]
 </xsl:template>

 <xsl:template name="coincidence">
    <xsl:call-template name="direction_attribute"/>
    <xsl:choose>
	<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
	<xsl:otherwise/>
    </xsl:choose>
    <xsl:apply-templates select="@*[name()!='layer' and name()!='sensor' and name()!='id' and name()!='type' and name()!='direction']"/>
 </xsl:template>

 <xsl:template name="intensity">
    <xsl:choose>
	<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
	<xsl:otherwise/>
    </xsl:choose>
    <xsl:if test="@layer or @sensor">
	<xsl:apply-templates select="@layer|@sensor"/>
    </xsl:if>
    <xsl:if test="@energy_min or @energy_max or @direction">
	<xsl:apply-templates select="@energy_min|@energy_max|@direction"/>
    </xsl:if>
 </xsl:template>

 <xsl:template match="Channel">
    <xsl:choose>
	<xsl:when test="@type='temperature'">
	    Temperature
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise>(C)</xsl:otherwise>
	    </xsl:choose>
	</xsl:when>
	<xsl:when test="@type='pressure'">
	    Pressure
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise>(mb)</xsl:otherwise>
	    </xsl:choose>
	</xsl:when>
	<xsl:when test="@type='humidity'">
	    Humidity
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise>(%)</xsl:otherwise>
	    </xsl:choose>
	</xsl:when>
	<xsl:when test="@type='voltage'">
	    High Voltage
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise>(V)</xsl:otherwise>
	    </xsl:choose>
	    <xsl:if test="@layer or @sensor">
		<xsl:apply-templates select="@layer|@sensor"/>
	    </xsl:if>
	</xsl:when>
	<xsl:when test="@type='threshold'">
	    ADC Threshold
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise>(V)</xsl:otherwise>
	    </xsl:choose>
	    <xsl:if test="@layer or @sensor">
		<xsl:apply-templates select="@layer|@sensor"/>
	    </xsl:if>
	</xsl:when>
	<xsl:when test="@type='adc'">
	    ADC Output
	    <xsl:if test="@layer or @sensor">
		<xsl:apply-templates select="@layer|@sensor"/>
	    </xsl:if>
	</xsl:when>
	<xsl:when test="@type='spectrum'">
	    Spectrum, X=<xsl:value-of select="@id"/>
	</xsl:when>

	<xsl:when test="@type='intensity'">
	    Flux Intensity
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:when test="@type='intensity_eas'">
	    Intensity of EAS Events
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:when test="@type='intensity_charged'">
	    Intensity of the Charged Particles Flux
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:when test="@type='intensity_muon'">
	    Intensity of the Muon Flux
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:when test="@type='intensity_neutron'">
	    Intensity of the Neutron Flux
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:when test="@type='intensity_cn'">
	    Intensity of the Neutrons and Charged Partciles
	    <xsl:call-template name="intensity"/>
	</xsl:when>
	<xsl:otherwise>
	    Other Sensor
	    <xsl:if test="@type">
		(<xsl:value-of select="@type"/>)
	    </xsl:if>
	    <xsl:choose>
		<xsl:when test="@units"><xsl:value-of select="@units"/></xsl:when>
		<xsl:otherwise/>
	    </xsl:choose>
	    <xsl:if test="@layer or @sensor">
		<xsl:apply-templates select="@layer|@sensor"/>
	    </xsl:if>
	    <xsl:apply-templates select="@*[name(.)!='layer' and name(.)!='sensor' and name()!='id' and name()!='type']"/>
	</xsl:otherwise>
    </xsl:choose>
 </xsl:template>

 <xsl:template match="Coincidence">
    <xsl:choose>
	<xsl:when test="@type='intensity'">
	    Coincidence <xsl:call-template name="coincidence"/>
	</xsl:when>
	<xsl:when test="@type='intensity_charged'">
	    Charged Particle Coincidence <xsl:call-template name="coincidence"/>
	</xsl:when>
	<xsl:when test="@type='intensity_muon'">
	    Muon Coincidence <xsl:call-template name="coincidence"/>
	</xsl:when>
	<xsl:when test="@type='intensity_neutron'">
	    Neutron Coincidence <xsl:call-template name="coincidence"/>
	</xsl:when>

	<xsl:otherwise>
	    Coincidence <xsl:call-template name="coincidence"/>
	</xsl:otherwise>
    </xsl:choose>
 </xsl:template>

 <xsl:template match="Variance">
    Variance of ID:<xsl:value-of select="@targetid"/>
    <xsl:apply-templates select="@*[name(.)!='layer' and name(.)!='sensor' and name()!='id']"/>
 </xsl:template>

 <xsl:template match="Correlation">
    Correlation between ID:<xsl:value-of select="@targetid1"/> and ID:<xsl:value-of select="@targetid2"/>
    <xsl:apply-templates select="@*[name(.)!='layer' and name(.)!='sensor' and name()!='id']"/>
 </xsl:template>

 <xsl:template match="/">
    <p>
    <i>Data obtained at</i>: <xsl:value-of select="Data/Time"/><br/>
    <xsl:if test="Data/Duration">
    <i>Calculation Duration</i>: <xsl:value-of select="Data/Duration"/><br/>
    </xsl:if>
    <xsl:if test="Data/Quality">
    <i>Quality</i>: <xsl:value-of select="Data/Quality"/><br/>
    </xsl:if>
    </p>
    <table border="0" cellpadding="0" cellspacing="0">
    <xsl:for-each select="Data/Value/*">
	<tr>
	    <td class="dataid"><xsl:value-of select="@id"/></td>
	    <td><xsl:apply-templates select="."/></td>
	    <td><xsl:value-of select="."/></td>
	</tr>
    </xsl:for-each>
    </table>
 </xsl:template>
</xsl:stylesheet>
