<?function isAlphaNum($str)
{
	if (preg_match("/[^A-Za-z0-9\.\-_]/",$str)) return false;
	else return true;
}

function page_check($page) {
	if (!isAlphaNum($page)) return true;
	if (strlen($page)>1024) return true;
	if ($page{i}=='/') return true;
	if (substr_count($page,"..")) return true;
	return false;
}


function parse_detector_info(&$dev, &$sd) {
    $sensors = 0;

    $sd["subdevices"] = 1;
    switch ($dev["type"]) {
	case "NAMT":
	    if (preg_match('/Sub\s*Devices:?\s*(\d{1,2})\s*\(([\d\s,]*)\)/i', $dev["properties"], $m)) {
		$sd["subdevices"] = $m[1];
		$tmp = $m[2];
				
		$sd["subdevice"] = array();
		$atmp = preg_split('/,\s*/', $tmp);
		for ($i=0;$i<$sd["subdevices"];$i++) {
		    if ($sd["subdevices"]>1)
			$sd["subdevice"][$i]["name"] = $sd["name"] . "-" . ($i+1);
		    else 
			$sd["subdevice"][$i]["name"] = $sd["name"];
		    
		    $sd["subdevice"][$i]["sensors"] = $atmp[$i]*2;
		    $sd["subdevice"][$i]["channels"] = $atmp[$i]*2;
		    $sensors += $atmp[$i]*2;
		}
	    } else $sensors = 8;
	break;
	case "NANM":
	    if (preg_match('/Hosts:?\s*(\d{1,3})$/i', $dev["properties"], $m)) {
		$sd["subdevice"] = array();
		$sd["subdevice"][0]["name"] = $sd["name"];
	    	$sd["subdevice"][0]["sensors"] = $m[1];
		$sd["subdevice"][0]["channels"] = $m[1];
		$sensors = $m[1];
	    } else $sensors = 0;
	break;
	default:
	    if (preg_match("/Sensors:?\s*(\d+)/i", $dev["properties"], $m)) {
		$sd["subdevice"] = array();
		$sd["subdevice"][0]["name"] = $sd["name"];
	    	$sd["subdevice"][0]["sensors"] = $m[1];
		$sd["subdevice"][0]["channels"] = $m[1];
		$sensors = $m[1];
	    } else {
		switch ($dev["type"]) {
		    case "SEVAN":
			$sensors = 3;
		    break;
		    default:
			$sensors = 0;
		}
	    }
    }

    return $sensors;
}

function parse_info() {
    global $version, $status, $devices, $device, $srvtime;
    
    $mode = 0;
    $version = "Unknown";
    $status = "Stopped";
    $srvtime = "Unknown";
    $devices = 0;
    $device = array();
    
    $fp = popen("adas-simple -l", "r");
    if ($fp) {
	while (!feof($fp)) {
	    $data = fgets($fp);
	    if ($mode == 0) {
		if (preg_match('/VERSION.\s*(\d{1,3}.\d{1,3}(.\d{1,3})?)[^\d]/i', $data, $m)) {
		    $version=$m[1];
		    $mode++;
		}
	    }
	    if ($mode == 1) {
		if (preg_match('/status[^\d]+\s+([\dABCDEF]{1,16})/i', $data, $m)) {
		    if ($m[1]==0) $status="Ok";
		    else $status=$m[1];
		    $mode++;
		}
	    }
	    if ($mode == 2) {
		if (preg_match('/time[^\d]+([\d\-]+T([\d:]+)(\.\d*)?[+\-Z](\d{1,2}:\d{1,2})?)/i', $data, $m)) {
		    $srvtime=$m[1];
		    $mode++;
		}
	    }
	    if ($mode == 3) {
		$tmp = stristr($data, "devices");
		if (preg_match('/^[^\d]+([\d]{1,3})/', $tmp, $m)) {
		    $devices = $m[1];
		    $mode++;
		}
	    }
	    if ($mode >= 4) {
		if (preg_match('/^\s*(\w+)\sdevice\s+.*ID:\s*(\d{1,3}).*Version:\s*(\d{1,4})[^\w\d]+([\w\d]+)[^\w\d]+Status:\s*(\d{1,16})[^\w\d]+Mode:\s*(\d{1,16})[^\w]*(\w.*)?$/i', $data, $m)) {
		    $device[$mode-4]["type"] = $m[1];
		    $device[$mode-4]["id"] = $m[2];
		    $device[$mode-4]["version"] = $m[3];
		    $device[$mode-4]["name"] = $m[4];
		    $device[$mode-4]["status"] = $m[5];
		    $device[$mode-4]["mode"] = $m[6];
		    $device[$mode-4]["properties"] = $m[7];
		    $device[$mode-4]["subdevices"] = 1;
		    
		    switch($device[$mode-4]["status"]) {
			case 0:
			    switch ($device[$mode-4]["mode"]) {
				case 0:
				    $device[$mode-4]["status"] = "Running";
				break;
				case 1:
				    $device[$mode-4]["status"] = "Configuration Mode";
				break;
				default:
				    $device[$mode-4]["status"] = "Running (Unknown Mode)";
			    }
			break;
			case 1:
			    $device[$mode-4]["status"] = "Not Connected";
			break;
			case 2:
			    $device[$mode-4]["status"] = "Initializing";
			break;
			case 3:
			    $device[$mode-4]["status"] = "Reseting";
			break;
		    }

		    $device[$mode-4]["sensors"] = parse_detector_info($device[$mode-4], $device[$mode-4]);
		    
		    $mode++;	    
		}
	    }
	}
	pclose($fp);	
    }
}

function parse_mcu_info($device_name) {
    $mcu = array();
    
    $fp = popen("adas-simple -hi /$device_name", "r");
    if ($fp) {

	$mcu['layout'] = false;
	while (!feof($fp)) {
	    $data = fgets($fp);
	    if (preg_match('/type[^\w\d]+([\w\d]+)/i', $data, $m)) {
		$mcu['type'] = $m[1];
	    }

	    if (preg_match('/version[^\w\d]+([\d]+)/i', $data, $m)) {
		$mcu['version'] = $m[1];
	    }

	    if (preg_match('/mode[^\w\d]+([\d]+)/i', $data, $m)) {
		$mcu['mode'] = $m[1];
	    }

	    if (preg_match('/status[^\w\d]+([\d]+)/i', $data, $m)) {
		$mcu['status'] = $m[1];
	    }
	    
	    if (preg_match('/subdevices[^\w\d]+([\d]+)/i', $data, $m)) {
		$mcu['subdevices'] = $m[1];
		$mcu['subdevice'] = array();
	    }

	    if (preg_match('/subdevice[^\w\d]+([\d]+)/i', $data, $m)) {
		$subdevice = &$mcu['subdevice'][$m[1]-1];
		
		if (preg_match('/sensors[^\w\d]+([\d]+)/i', $data, $m)) {
		    $subdevice['sensors'] = $m[1];
		}
		if (preg_match('/coincidences[^\w\d]+([\d]+)/i', $data, $m)) {
		    $subdevice['coincidences'] = $m[1];
		}
		if (preg_match('/spectrums[^\w\d]+([\d]+)/i', $data, $m)) {
		    $subdevice['spectrums'] = $m[1];
		}
	    }
	    
	    if (preg_match('/Configurable\s+Layout/i', $data)) {
		$mcu['layout'] = array();
		if (preg_match('/sensors[^\w\d]+([\d]+)/i', $data, $m)) {
		    $mcu['layout']['sensors'] = $m[1];
		} else {
		    unset($mcu['layout']);
		    $mcu['layout'] = false;
		}
		
		if (($mcu['layout'])&&(preg_match('/channels[^\w\d]+([\d]+)/i', $data, $m))) {
		    $mcu['layout']['channels'] = $m[1];
		
		    if (preg_match('/rules[^\w\d]+([\d]+)/i', $data, $m)) {
			$mcu['layout']['rules'] = $m[1];
		    }
		    if (preg_match('/spectrums[^\w\d]+([\d]+)/i', $data, $m)) {
			$mcu['layout']['spectrums'] = $m[1];
		    }
		}
		
	    } 
	    
	}
	pclose($fp);
	
	return $mcu;
    }
    return false;
}

function parse_config() {
    global $adas_config, $adas_config_xp;
    
    $fp = @popen("adas-simple -c", "r");
    if ($fp) {
	if (!feof($fp)) {
	    $pdata = fgets($fp);

	    if (preg_match("/(timeout|error)/i", $pdata)) {
		if (file_exists("/etc/adas.cfg")) {
		    $fpf = @popen("cat /etc/adas.cfg", "r");
		    if ($fpf) {
			if (feof($fpf)) pclose($fpf);
			else {
			    pclose($fp);
			    $fp = $fpf;
			    $pdata = fgets($fp);
			}
		    }
		}
	    }
	    
	    if (!preg_match("/(timeout|error)/i", $pdata)) {
		$adas_config_text = "";
		while ($pdata) {
		    $adas_config_text .= $pdata;
		    if (feof($fp)) $pdata = FALSE;
		    else $pdata = fgets($fp);
		}
		
		$adas_config = new DOMDocument;
		$adas_config->validateOnParse = true;

		if (@$adas_config->LoadXML($adas_config_text)) {
		    $adas_config_xp = new DOMXPath($adas_config);
		} else unset($adas_config);
	    }
	} 
	pclose($fp);
    }
}

function get_device($name) {
    global $devices, $device;

    for ($i = 0; $i < $devices; $i++) {
	if (!strcmp($device[$i]["name"], $name)) return $device[$i];
    }
    return false;
}

function get_data_path($detector, $type) {
    global $tempdir, $adas_config, $adas_config_xp;
    
    if (!$adas_config) return $tempdir;
    
    $res = $adas_config_xp->query("/ADAS/Devices/*[@id=\"$detector\"]/Writeout/Writers/*");

    foreach ($res as $writer) {
	$wrname = $writer->nodeValue;
	$res2 = $adas_config_xp->query("/ADAS/Writers/$type" . "Writer[@id=\"$wrname\"]/DataPath");
	if (($res2)&&($res2->length == 1)) return $res2->item(0)->nodeValue;
    }

    return $tempdir;
}


function get_description_path($detector, $type) {
    global $permdir, $adas_config, $adas_config_xp;
    
    if (!$adas_config) return $permdir;
    
    $res = $adas_config_xp->query("/ADAS/Devices/*[@id=\"$detector\"]/Writeout/Writers/*");

    foreach ($res as $writer) {
	$wrname = $writer->nodeValue;
	$res2 = $adas_config_xp->query("/ADAS/Writers/$type" . "Writer[@id=\"$wrname\"]/DescriptionPath");
	if (($res2)&&($res2->length == 1)) return $res2->item(0)->nodeValue;
	else {
	    $res2 = $adas_config_xp->query("/ADAS/Writers/$type" . "Writer[@id=\"$wrname\"]/DataPath");
	    if (($res2)&&($res2->length == 1)) return $res2->item(0)->nodeValue;
	}
    }

    return $permdir;
}


function set_dev() {
    global $dev, $subdev, $device, $devices;
    
    for ($i=0;$i<$devices;$i++) {
	$curdev = &$device[$i];
	if (!strcmp($curdev["name"], $_GET["detector"])) {
	    $dev = $curdev;
	    return $dev;
	}
    }
    return ;
}

function set_subdev() {
    global $dev, $subdev;
    
    if ($_GET["subdevice"]>0) {
	$subdev = $_GET["subdevice"];
	if ($subdev > $dev["subdevices"]) $subdev = 0;
    } else $subdev = 0;

    if ($subdev) $subdev = $dev["subdevice"][$subdev-1];
    
    return $subdev;
}

function is_error($level) {
    switch ($level) {
	case "ASSERTION":
	case "FATAL":
	case "EMERGENCY":
	case "ALERT":
	    return 3;
	case "CRITICAL":
	case "ERROR":
	    return 2;
	case "WARNING":
	    return 1;
	default:
	    return 0;
    }
}


function is_global_error($error) {
    global $device;
    
    foreach ($device as $curdev) {
	if (preg_match("/" . $curdev["name"] . "/", $error["msg"])) return 0;
    }
    
    return 1;
}


function list_detectors() {
    global $adas_config, $adas_config_xp;
    
    $list = array();
    
    $res = $adas_config_xp->query("/ADAS/Devices/*[@disabled = 'false' or @disabled='False' or @disabled='FALSE' or @disabled = 0]/@id");

    foreach ($res as $device) {
	$list[] = $device->nodeValue;
    }

    return $list;
    
}

function list_filedetectors($filedir) {
	global $permdir, $tempdir;
	global $adas_config, $adas_config_xp;
	
	$list = array();
	
        $dh = opendir($permdir);
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match("/^(\w.*)\.xml$/", $file, $m)) {
		    $name = $m[1];
		    if ((is_dir($permdir . $name))||(is_dir($tempdir . $name))) {
			$list[] = $name;
		    }
		}
	    }
	    closedir($dh);
	}

	if (($filedir)&&($adas_config)) {
	    $detectors = list_detectors();

	    $pdirs = array ($permdir . $permdir);
	    foreach ($detectors as $detector) {
		$datapath = get_data_path($detector, "XML");
		$descpath = get_description_path($detector, "XML");

		if (!in_array($datapath . $descpath, $pdirs)) {
		    $dh = @opendir($descpath);
		    if ($dh) {
			while(($file = readdir($dh)) !== false) {
			    if (preg_match("/^(\w.*)\.xml$/", $file, $m)) {
				$name = $m[1];
				if (is_dir($datapath . "/" . $name)) {
				    $list[] = $name;
				}
			    }
			}
			closedir($dh);
		    }
		    array_push($pdirs, $datapath . $descpath);
		}
	    }

	    $list = array_unique($list);
	    sort($list);
	} 

	return $list;
}

function list_filesubs($filedir, $filedetector) {
	global $permdir, $tempdir;

	$list = array();
        
	$dh = @opendir($permdir . "/" . $filedetector);
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match("/^\d+-(.*)\.xml$/", $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}

	if ($filedir) $dh = @opendir($filedir . "/" . $filedetector);
	else $dh = @opendir($tempdir . "/" . $filedetector);
	
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match("/^\d+-(.*)\.xml$/", $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}
	
	$list = array_unique($list);
	sort($list);

	return $list;
}

function list_dates($filedir, $filedetector, $sub) {
	global $permdir, $tempdir;

	$list = array();
        
	if ($sub) {
	    $re="/^(\d+)-" . $sub . "\.xml$/";
	} else {
	    $re="/^(\d+)\.xml$/";
	}
	
	$dh = @opendir($permdir . $filedetector);
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match($re, $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}

	if ($filedir) $dh = @opendir($filedir . $filedetector);
	else $dh = @opendir($tempdir . $filedetector);

	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match($re, $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}
	
	$list = array_unique($list);
	sort($list);

	return $list;
}

function list_formats($filedir, $filedetector, $sub, $date) {
	global $permdir, $tempdir;

	$list = array();
        
	if (isset($sub)) {
	    $re="/^" . $date . "-" . $sub . "\.([^.]*)$/";
	} else {
	    $re="/^" . $date . "\.([^.]*)$/";
	}
	
	$dh = @opendir($permdir . $filedetector);
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match($re, $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}

	if ($filedir) $dh = @opendir($filedir . $filedetector);
	else $dh = @opendir($tempdir . $filedetector);
	if ($dh) {
	    while(($file = readdir($dh)) !== false) {
		if (preg_match($re, $file, $m)) {
		    $list[] = $m[1];
		}
	    }
	    closedir($dh);
	}
	
	$list = array_unique($list);
	sort($list);

	return $list;
}


function simpleredirect($url, $timeout /*ms*/) {
    ?><script language="JavaScript"><?
    if ($timeout) {
	?>setTimeout('window.location="<?echo $url;?>"', <? echo $timeout;?>);<?
    } else {
	?>window.location="<?echo $url;?>";<?
    }
    ?></script><?
}

function redirect($params, $timeout) {
    $rpath = "index.php";
    if (isset($params)) {
	$props = unserialize(stripslashes($params));
	
	$rpath .= "?";
	$printed = 0;
	foreach ($props as $pn => $prop) {
	    if ($printed) $rpath .= "&amp;";
	    else $printed = 1;
	    
	    $rpath .= $pn . "=" . $prop;
	}
    } 
    simpleredirect($rpath, $timeout);
}

function request_authentication($realm) {
    global $unauthorized;
    
    header("WWW-Authenticate: Basic realm=\"" . $realm . "\"");
    header("HTTP/1.0 401 Unauthorized");
    $unauthorized = '<span class="highlight">Error</span>: You should authenticate yourself to control ADAS server.';
}

function current_gmt_day() {
    $tz = date_default_timezone_get();
    date_default_timezone_set("GMT");
    $reqdate = date("Ymd");
    date_default_timezone_set($tz);
    return $reqdate;
}

function gmt_day($reqtime) {
    $tz = date_default_timezone_get();
    date_default_timezone_set("GMT");
    $reqdate = date("Ymd", $reqtime);
    date_default_timezone_set($tz);
    return $reqdate;
}


function authenticate() {
	global $unauthorized, $authenticated;
	global $AuthUserFileName, $AuthGroupFileName, $AuthGroup;
	global $_SESSION, $_SERVER, $HTTP_SERVER_VARS;

	session_start();
	
	if ($_SESSION["reauth"]) {
	    unset($_SESSION["reauth"]);
	    unset($_SERVER['PHP_AUTH_USER']);
	    session_commit();
	    request_authentication("ADAS");
	}
	
	if (isset($_SERVER['PHP_AUTH_USER'])) {
	    $user = $_SERVER['PHP_AUTH_USER'];

	    $AuthUserFile = @file($AuthUserFileName);
	    

	    if ((isset($AuthGroup))&&((($AuthGroupFile = @file($AuthGroupFileName))==FALSE)||(!preg_grep("/$$AuthGroup: $user$/", $AuthGroupFile)))) request_authentication();
	    else if ($AuthUserFile) {
		if (($authUserLine = array_shift(preg_grep("/$user:.*$/", $AuthUserFile)))&&(preg_match("/$user:((..).*)$/", $authUserLine, $matches))) {
		    $authPW = $matches[1];
		    $salt = $matches[2];
		    $userpw = crypt($HTTP_SERVER_VARS['PHP_AUTH_PW'], $salt);
		    if ($userpw == $authPW) $authenticated = 1;
		    else $unauthorized = '<span class="highlight">Error</span>: Invalid password.';
		} else $unauthorized = '<span class="highlight">Error</span>: User not found.';
	    } else $unauthorized = '<span class="highlight">Error</span>: Authentication file is missing.';
	    if (!$authenticated) $_SESSION["reauth"]=1;
	} else request_authentication("ADAS");
}


function get_last_file_record($file) {
        global $MAX_RECORD_LINES;

	$fp = popen("tail -n $MAX_RECORD_LINES $file", "r");
	if ($fp) {
	    $started = 0;
	    $value = FALSE;
	    while (!feof($fp)) {
		$line = fgets($fp);
		if ($started) {
		    $valuetmp .= $line;
		    if (preg_match("/^\s*<\/Data>/",$line)) {
			$value = $valuetmp;
			$started = 0;
		    }
		} else {
		    if (preg_match("/^\s*<Data/",$line)) {
			$started = 1;
			$valuetmp = $line;
		    }
		}
	    }
	    pclose($fp);
	} else $value = FALSE;
	
	return $value;
}

function get_last_data_record($detector, $device, $sub) {
    global $permdir;
    
    $datapath = get_data_path($detector, "XML");

    for ($i = 0; $i < 2; $i++) {
        $tz = date_default_timezone_get();
	date_default_timezone_set("GMT");
	$today = date("Ymd", time() - $i*86400);
	date_default_timezone_set($tz);

	if ($sub) $filename = $datapath . "/" . $device . "/" . $today . "-" . $sub . ".xml";
	else $filename = $datapath . "/" . $device . "/" . $today . ".xml";
	
	if (file_exists($filename)) return get_last_file_record($filename);
    }

    $dates = list_dates(false, $device, $sub);
    if (($dates)&&(count($dates))) {
	if ($sub) $filename = $permdir . "/" . $device . "/" . array_pop($dates) . "-" . $sub . ".xml";
	else $filename = $permdir . "/" . $device . "/" . array_pop($dates) . ".xml";
	return get_last_file_record($filename);
    }
    
    return false;
}

function parse_data_record($desc, $desc_xp, $record) {
	$data = new DOMDocument;
	$data->validateOnParse = true;

	if (@$data->LoadXML($record)) {
	    $data_xp = new DOMXPath($data);
	} else return FALSE;

	$res = $data_xp->query("/Data");
	if (($res)&&($res->length == 1)) $data_node = $res->item(0);
	else return FALSE;
	    
	$res = $data_xp->query("/Data/@layout");
	if (($res)&&($res->length == 1)) $layout = $res->item(0)->nodeValue;
	else return FALSE;

	$res = $data_xp->query("/Data/Value");
	if (($res)&&($res->length == 1)) {
	    $value_node = $res->item(0);
	    $value_vec = $value_node->nodeValue;
	    $value_vec = preg_replace("/(^\s+|\s+$)/", "", $value_vec);
	    $values = preg_split("/\s+/", $value_vec);
	    
	    $data_node->removeChild($value_node);
	} else return FALSE;

	$value_node = $data->createElement("Value");
	if (!$value_node) return FALSE;


	$res = $desc_xp->query("/Installation/Layout[@id='" . $layout . "']/*");
	foreach ($res as $node) {
	    $nodecopy = $data->importNode($node, 1);

	    $nodecopy->replaceChild(new DOMText($values[$node->nodeValue-1]), $nodecopy->firstChild);
	    $value_node->appendChild($nodecopy);
	}
	

	$data_node->appendChild($value_node);

	return $data;
}


function xmldata($detector, $device, $sub) {
    $descpath = get_description_path($detector, "XML");

    $descfile = $descpath . "/" . $device . ".xml";

    $desc = new DOMDocument;
    $desc->validateOnParse = true;

    if (@$desc->load($descfile)) {
	$desc_xp = new DOMXPath($desc);
    } else {
	/* Sometimes sharing violation occurs (while adas-srv reads 
	description file) */
	usleep(100000);
	if (@$desc->load($descfile)) {
	    $desc_xp = new DOMXPath($desc);
	} else {
	    return FALSE;
	}
    }

    $value = get_last_data_record($detector, $device, $sub);
    if (!$value) return FALSE;
	
    $value_doc = parse_data_record($desc, $desc_xp, $value);
    if (!$value_doc) return FALSE;

    return $value_doc;
}

function xmldata_get_time($data) {
    for ($node = $data->documentElement->firstChild; $node; $node = $node->nextSibling) {
	if (($node->nodeType == XML_ELEMENT_NODE)&&(!strcmp($node->localName, "Time"))) break;
    }    

    if ($node) return strtotime($node->textContent);
    return false;    
}


function js_set_iframe_height($iframe_ref) {
?>
<script type="text/javascript">
    function getIFRAME_doc_height_<?echo $iframe_ref;?>(IFrameDoc) {
	return (typeof IFrameDoc.height != 'undefined') ? IFrameDoc.height + 30 : (IFrameDoc.body &amp;&amp; typeof IFrameDoc.body.scrollHeight != 'undefined') ? IFrameDoc.body.scrollHeight + 30 : null;
    }

    function SetFrameContent_<?echo $iframe_ref;?>() {
	var IFrameDoc, oIframe = document.getElementById("<?echo $iframe_ref;?>");
	if (typeof oIframe != null) {
	    if (oIframe.contentDocument) IFrameDoc = oIframe.contentDocument;
	    else if (oIframe.contentWindow) IFrameDoc = oIframe.contentWindow.document;
	    else if (oIframe.document) IFrameDoc = oIframe.document;

	    var hgt = getIFRAME_doc_height_<?echo $iframe_ref;?>(IFrameDoc);
	    if (hgt) oIframe.style.height = String(hgt + 'px');
	}
    }
</script>
<?
}

function get_usb_list() {
    global $USB_DIR, $UDEV_FILE, $UDEV_REGEXP;
    global  $USB_PROPS;
    
    if (is_dir($USB_DIR)) $dir = opendir("/dev/tts/");
    else $dir = FALSE;
    
    $rules = @file($UDEV_FILE);

    $devices = array();
    if ($dir) {
	while (($file=readdir($dir)) !== false) {
	    if (preg_match("/USB\d+/", $file)) {
		array_push($devices, $file);
	    }
	}
    	closedir($dir);
    }
    
    $devices_info = array();

    foreach ($devices as $device) {
	$fp = popen("udevinfo -a -p `udevinfo -q path -n " . $USB_DIR . "/" . $device . "`", "r");
	if ($fp) {
	    $mode = 0;

	    $props = array();
	    $device_info = array();
	    $device_info["id"] = $USB_DIR . "/" . $device;
	    
	    while (!feof($fp)) {
		$info = fgets($fp);
		
		switch ($mode) {
		    case 0:
			if (preg_match("/^\s*BUS=+\"usb\"/i", $info)) $mode = 1;
			unset($info);
		    break;
		    case 1:
			if (preg_match("/^\s*ID=+\"[\d\-]+\"/i", $info)) $mode = 2;
			else $mode = 0;
			unset($info);
		    break;
		    case 2:
			if (preg_match("/^\s*DRIVER=+\"usb\"/i", $info)) $mode = 3;
			else $mode = 0;
			unset($info);
		    break;
		    default:
			if (preg_match("/^\s*BUS=/i", $info)) {
			    if (preg_match("/^\s*BUS=+\"usb\"/i", $info)) $mode = 1;
			    else $mode = 0;
			    unset($info);
			}
		}
		if (!$info) continue;
		
		if (preg_match("/SYSFS{" . $USB_PROPS . "}=+\"(.*)\"/i", $info, $m)) {
		    $props[$m[1]] = $m[2];
		}
	    }
	    
	    $valid_rule = FALSE;
	    $multimatch = 0;
	    
	    if ($rules) {
	      foreach ($rules as $rule) {
		$valid = 1;
		if (preg_match_all("/SYSFS{([^}]+)}==\"([^\"]+)\"/i", $rule, $ms, PREG_SET_ORDER)) {
		    foreach ($ms as $m) {
			$name = $m[1];
			$value = "/" . preg_replace($UDEV_REGEXP, ".+", $m[2]) . "/";
			
			if ((!isset($props[$name]))||(!preg_match($value, $props[$name]))) {
			    $valid = 0;
			    break;
			}
		    }
		} else $valid = 0;
		
		if ($valid) {
		    if ($valid_rule) $multimatch = 1;
		    else {
			if (preg_match("/SYMLINK\+=\"adas-usb\/usb-([^\"]+)\"/", $rule, $m)) {
			    $valid_rule = $m[1];
			}
		    }
		}
	      }    
	    }
	    
	    if ($valid_rule) {
		$device_info["device"] = $valid_rule;
		if ($multimatch) {
		    $device_info["multirule"] = 1;
		}
	    }
		
	    
	    $props_string = serialize($props);
	    foreach ($devices_info as $olddevice) {
		if ((isset($device_info["device"]))&&(isset($olddevice["device"]))&&(!strcmp($device_info["device"], $olddevice["device"]))) {
		    $device_info["multiname"] = 1;
		    $olddevice["multiname"] = 1;
		}
		
		$oldprops_string = serialize($olddevice["props"]);
		if (!strcmp($props_string, $oldprops_string)) {
		    $device_info["dublicate"] = 1;
		    $olddevice["dublicate"] = 1;
		    break;
		} 
	    }
	    
	    $device_info["props"] = $props;
	    array_push($devices_info, $device_info);
	    
	    pclose($fp);
	}
    }
    return $devices_info;
}

function http_request($srv, $request, $func, &$attr) {
    global $CHUNK_SIZE, $BIG_CHUNK_SIZE;
    
    $url = parse_url($srv);

    if (preg_match("/^https:/",$url["sheme"])) $prefix="ssl://";
    else $prefix="";

    if ($url["port"]) $port=$url["port"];
    else $port="80";

    $fp = fsockopen($prefix . $url["host"], $port);
    if (!is_resource($fp)) return false;

    if ($request) {
	fputs($fp,"POST " . $srv . " HTTP/1.1\r\n");
	fputs($fp,"Host: " . $url["host"] . "\r\n");
	fputs($fp,"Content-type: text/xml\r\n");
	fputs($fp,"Content-length: " . strlen($request) . "\r\n\r\n");
	fputs($fp, $request);
    } else {
	fputs($fp,"GET " . $url["path"] . "?" . $url["query"] . " HTTP/1.0\r\n");
	fputs($fp,"Host: " . $url["host"] . "\r\n");
	fputs($fp,"\r\n\r\n");
    }

    $res=fgets($fp);
    if ((!preg_match("/^HTTP\/\d.\d (\d{3})/",$res,$m))||($m[1]!=200)) {
	fclose($fp);
	return false;
    }
    
    while ((!preg_match("/^\r\n$/", $res))&&(!feof($fp)))
	$res=fgets($fp);
    
    if ($func) {
	$res = $func($attr, 1, NULL);
        if (!$res) {
	    fclose($fp);
	    return false;
	}
    
	while((!feof($fp))) {
	    $res = fread($fp, $CHUNK_SIZE);
	    $func($attr, 0, $res);
	}
	
	$res = $func($attr, 2, NULL);
    } else {
	$res = "";
	while((!feof($fp))) {
	    $res .= fread($fp, $BIG_CHUNK_SIZE);
	}
    }
    fclose($fp);

    return $res;
}


function pipe_data(&$attr, $mode, $data) {
    global $CHUNK_SIZE, $BIG_CHUNK_SIZE;

    if (!$mode) {
	fwrite($attr["pipes"][0], $data);
	if ($attr["func"]) {
	    $read = array($attr["pipes"][1]);
	    if (stream_select($read, $write = NULL, $except = NULL, 0, 1000) > 0) {
	        $data = fread($attr["pipes"][1], $CHUNK_SIZE);
		if ($data) $attr["func"]($attr, $mode, $data);
	    }
	}
    } else if ($mode == 1) {
	$descriptorspec = array(
	    0 => array("pipe", "r"),
	    1 => array("pipe", "w"),
	);
	 
	$process = proc_open($attr["cmd"], $descriptorspec, $pipes);
	if (is_resource($process)) {
	    $attr["proc"] = $process;
	    $attr["pipes"] = $pipes;
	    
	    if ($attr["func"]) $attr["func"]($attr, $mode, $data);
	    return true;
	} return false;
    } else if ($mode == 2) {
	fclose($attr["pipes"][0]);
	if ($attr["func"]) {
	    while((!feof($attr["pipes"][1]))) {
		$data = fread($attr["pipes"][1], $CHUNK_SIZE);
		$attr["func"]($attr, 0, $data);
	    }
	    $attr["func"]($attr, $mode, "");
	    $data = true;
	} else {
	    $data = stream_get_contents($attr["pipes"][1]);
	}

	fclose($attr["pipes"][1]);
	proc_close($attr["proc"]);
	return $data;
    }
}

function echo_data($attr, $mode, $data) {
    if (!$mode) echo $data;
    return true;
}


function get_detector_daydata($device, $sub, $day, $start_time, $func, &$arg) {
	global $permdir;
		
        $datapath = get_data_path($detector, "XML");

	if ($sub) $name = $day . "-" . $sub . ".xml";
	else $name = $day . ".xml";

	if (is_file($datapath  . $device . "/" . $name)) {
	    $fp = fopen($datapath . $device . "/" . $name, "r");
	} else if (is_file($permdir . $device . "/" . $name)) {
	    $fp = fopen($permdir . $device . "/" . $name, "r");
	} else {
	    unset ($fp);
	}
	
	if ($fp) {
	    $res = ""; $flag = 0;
	    while (!feof($fp)) {
		$data = fgets($fp);
		if ($data) {
		    if ($flag) {
		        if (preg_match("/<\/Data>/", $data)) {
		    	    $record .= $data;
			    if ((!$start_time)||((preg_match("/<Time>\s*(.*)\s*<\/Time>/", $record, $m))&&(strtotime($m[1])>$start_time))) {
				if ($func) $func($record, $arg);
				else $res .= $record;
			    
				$start_time = 0;
			    }
			    $flag = 0;
			} else {
			    $record .= $data;
			}
		    } else if (preg_match("/<Data[^\w]/", $data)) {
			$record = $data;
			$flag = 1;
		    }
		}
	    }
	    fclose($fp);
	}
	
	return $res;
}

function &gddo_stripper(&$data, $start, $end) {
    while (substr($data, $start, 1)==' ') $start++;
    while (substr($data, -$end-1, 1)==' ') $end++;
    return substr($data, $start, -$end);
}


function get_detector_daydata_optimized($device, $sub, $day, $start_time, $func, &$arg) {
	global $permdir, $MAX_RECORD_LINES;
		
        $datapath = get_data_path($detector, "XML");

	if ($sub) $name = $day . "-" . $sub . ".xml";
	else $name = $day . ".xml";

	if (is_file($datapath  . $device ."/" . $name)) {
	    $fp = fopen($datapath . $device . "/" . $name, "r");
	} else if (is_file($permdir . $device . "/" . $name)) {
	    $fp = fopen($permdir . $device . "/" . $name, "r");
	} else {
	    unset ($fp);
	}
	
	if (($start_time)&&($fp)) {
	    $stat = fstat($fp); 
	    if ($stat) $size = $stat['size'];
	    else $size = 0;
	
	    $lastgood = 0; $lastgoodflag = 0; $curpos = 0;
	    for ($step = round($size/2); $step > 16384; $step = round($step/2)) {
		if ($curpos) fseek($fp, $curpos);
		
		$flag = 1;
		while ($data = fgets($fp)) {
		    for ($i=0;substr($data,$i,1)==' ';$i++);
		    if (strcmp(substr($data,$i+1,4),"Time")) {
			$flag = 0;
		    } else {
			$time = strtotime(gddo_stripper($data, $i+6, 9));
#			echo "$step $curpos $size $start_time $time\n";
			if ($time < $start_time) {
			    $lastgood = $curpos;
			    $lastgoodflag = $flag;
			    $curpos += $step;
			} else {
			    $curpos -= $step;
			}
			break;
		    }
		}
		
		if (feof($fp)) break;
	    }
	    
#	    echo $lastgood . "\n";
	    fseek($fp, $lastgood);
	}
	
	
	if ($fp) {
	    $res = ""; 
	    if ($lastgood) $flag = $lastgoodflag;
	    $flag = 0;
	    
	    while (!feof($fp)) {
		$data = fgets($fp);
		if ($data) {
		    if ($flag) {
			for ($i=0;substr($data,$i,1)==' ';$i++);
			
			switch(substr($data, $i+1, 4)) {
			    case "/Dat":
				if (!$start_time) $func($record, $arg);
				$flag = 0;
			    break;
			    case "Time":
				$record['time'] = &gddo_stripper($data, $i+6, 8);
				if (($start_time)&&(($record['time'])&&(strtotime($record['time'])>$start_time))) {
				    $start_time = 0;
				}
			    break;
			    case "Qual":
				if (!$start_time) $record['qaulity'] = &gddo_stripper($data, $i+9, 11);
			    break;
			    case "Dura":
				if (!$start_time) $record['duration'] = &gddo_stripper($data, $i+10, 12);
			    break;
			    case "Valu":
				if (!$start_time) $record['value'] = &gddo_stripper($data, $i+7, 9);
			    break;
			}
		    } else {
			if (preg_match("/^\s*<Data[^\w]/", $data)) {
			    $record = array();
			    $flag = 1;
			}
		    }
		}
	    }
	    fclose($fp);
	}
	
	return $res;
}


function get_detector_data($device, $sub, $from, $func, &$arg, $optimized) {
    $adates = list_dates(FALSE, $device, $sub);

    if ($adates[0]) $real_start = strtotime($adates[0]);
    else $real_start = time();

    if ($from == false) $reqtime = $real_start;
    else if ($from<1000000000) $reqtime = time() - $from;
    else $reqtime = $from;
    
    if ($reqtime < $real_start) $reqtime = $real_start;
    
    $start_time = $reqtime;

	// Otherwise, at the start of the new day client will miss events
    $tz = date_default_timezone_get();
    date_default_timezone_set("GMT");
    $day_hours = date("G", $reqtime);
    $day_minutes = date("i", $reqtime);
    $day_seconds = date("s", $reqtime);
    $reqtime -= ($day_hours*60 + $day_minutes)*60 + $day_seconds;
    date_default_timezone_set($tz);
        
    
    $res = "";
    while ($reqtime < time()) {
	if ($optimized) get_detector_daydata_optimized($device, $sub, gmt_day($reqtime), $start_time, $func, $arg);
	else $res .= get_detector_daydata($device, $sub, gmt_day($reqtime), $start_time, $func, $arg);
	$reqtime += 86400;
	$start_time = 0;
    }
    return $res;
}


function list_errors($device, $minlevel, $from) {
    $sock = @fsockopen("/tmp/adas/logserv.socket");
    if (!$sock) $sock = @fsockopen("unix:///tmp/adas/logserv.socket");
    $errors = array();
    
    if ($sock) {
	while (!feof($sock)) {
	    $record = fgets($sock);
	    $rec = split(" ", $record, 3);
	    
	    if ($rec) {
		$unixtime = strtotime($rec[0]);
		$level = is_error($rec[1]);
		if (($level>=$minlevel)&&($unixtime>$from)&&((($device)&&(preg_match("/" . $device . "/", $rec[2])))||(is_global_error($rec[2])))) { 
		    $error["msg"] = preg_replace("/\s+$/", "", $rec[2]);
		    $error["level"] = $level;
		    $error["time"] = $unixtime;
		    $errors[] = $error;
		}
	    }
	}
	fclose($sock);    
    }

    return $errors;
}
?>